
#include <extUtil.h>

#define     TEST         0
#define     TEST1        0
#define     MIN          0.
#define     MIN2         1.e-1

int  calcPrinc( float *s, float *p, float *a0, float *a1, float *a2, int flag )
/* *************************************************************************** */
/* berechnet aus dem Spannungstensor die Hauptspannungen und deren Raumwinkel  */
/* s: spannungstensor, p: Principals, a0: alfa_p1, a1: alfa_p2, a2: alfa_p3    */
/*                                                                             */
/* Die Hauptspannungsrichtungen sind so orientiert, das der Vektor der 1.Haupt-*/
/* spannung immer in +z-richtung zeigt und der Vektor der 2. Hauptspannung in  */
/* Richtung +y.                                                                */
/*                                                                             */
/* wenn flag=0  x[0]  >  x[1]  >  x[2]                                         */
/* wenn flag=1 |x[0]| > |x[1]| > |x[2]|                                        */
/* *************************************************************************** */
{
  double a, b, c, y[3], p_[3];
  float p1[3];
  double sd[6], pd[3], alfa[9];
  int i;

  a= -(s[0]+s[1]+s[2]);
  b= (s[0] * s[1]) + (s[1] * s[2]) + (s[2] * s[0]) - (s[3] * s[3]) - (s[4] * s[4]) - (s[5] * s[5] );
  c= -( s[0]*s[1]*s[2] + 2* s[3]*s[4]*s[5] - s[0]*s[4]*s[4] - s[1]*s[5]*s[5] - s[2]*s[3]*s[3] );
#if TEST1  
  printf ("xx:%.12f yy:%.12f zz:%.12f xy:%.12f yz:%.12f zx:%.12f\n", s[0], s[1], s[2], s[3], s[4], s[5]);
  printf ("a :%f b :%f c :%f\n" , a,b,c );
#endif

  /* if all principals 0 */
  if ( ( (a*a)<= MIN )&&( (b*b)<= MIN )&&( (c*c)<= MIN ) )
  {
    p[0]=0.;
    p[1]=0.;
    p[2]=0.;
    a0[0]=0.;
    a0[1]=0.;
    a0[2]=1.;
    a1[0]=0.;
    a1[1]=1.;
    a1[2]=0.;
    a2[0]=-1.;
    a2[1]=0.;
    a2[2]=0.;
    return (1);
  }
  /* if just one principal is != 0 */
  else if ( ( (b*b)<MIN )&&( (c*c)<MIN ) )
  {
    p_[0]=0.;
    p_[1]=0.;
    p_[2]=0.;
    if (flag)
    {
      p_[0]=-a;
    }
    else
    {
      if(-a<0) p_[2]=-a;
      else     p_[0]=-a;
    }
  }
  else if ( gl3grades( a, b, c, p_ ) < 0 )  return (-2);

  if (flag)
    {
    y[0]=p_[0]*p_[0];
    y[1]=p_[1]*p_[1];
    y[2]=p_[2]*p_[2];
    }
  else
    {
    y[0]=p_[0];
    y[1]=p_[1];
    y[2]=p_[2];
    }

  if ( (y[0]>=y[1]) && (y[0]>=y[2]) ) /* 1st greatest */
    {
    p[0]=p_[0];
    if (y[1]>y[2])
      {
      p[1]=p_[1];
      p[2]=p_[2];
      }
    else
      {
      p[1]=p_[2];
      p[2]=p_[1];
      }
    }
  else if ( (y[1]>=y[0]) && (y[1]>=y[2]) ) /* 2nt greatest */
    {
    p[0]=p_[1];
    if (y[0]>y[2])
      {
      p[1]=p_[0];
      p[2]=p_[2];
      }
    else
      {
      p[1]=p_[2];
      p[2]=p_[0];
      }
    }
  else if ( (y[2]>=y[0]) && (y[2]>=y[1]) ) /* 3rd greatest */
    {
    p[0]=p_[2];
    if (y[0]>y[1])
      {
      p[1]=p_[0];
      p[2]=p_[1];
      }
    else
      {
      p[1]=p_[1];
      p[2]=p_[0];
      }
    }
   else
   {
     printf("ERROR in calcPrinc P:%f %f %f\n",y[0],y[1],y[2] );
   } 


   for (i=0; i<6; i++) sd[i]=s[i];
   for (i=0; i<3; i++) pd[i]=p[i];
   calcPvector( sd, pd, alfa );
    a0[0]=alfa[0];
    a0[1]=alfa[1];
    a0[2]=alfa[2];
    a1[0]=alfa[3];
    a1[1]=alfa[4];
    a1[2]=alfa[5];
    a2[0]=alfa[6];
    a2[1]=alfa[7];
    a2[2]=alfa[8];


    /* kontrolle des letzten vektors mit Vektorprod.  */
    v_prod( a0, a1, p1);
    if(( (p1[0]*p1[0]-a2[0]*a2[0]) > MIN2 )|| ( (p1[1]*p1[1]-a2[1]*a2[1]) > MIN2 )||
    ( (p1[2]*p1[2]-a2[2]*a2[2]) > MIN2 ))
      {
#if TEST
      printf(" WARNING: in calcPrinc, some directions of maxPrinc in error \n");
      printf("  xx:%f yy:%f zz:%f xy:%f yz:%f zx:%f\n", s[0], s[1], s[2], s[3], s[4], s[5]);
      printf("  p1: %f p2: %f p3: %f \n", p[0], p[1], p[2] );
      printf("  v1x:%f v2x:%f v3x:%f v1xv2_x:%f \n", a0[0], a1[0], a2[0], p1[0] );
      printf("  v1y:%f v2y:%f v3y:%f v1xv2_y:%f \n", a0[1], a1[1], a2[1], p1[1] );
      printf("  v1z:%f v2z:%f v3z:%f v1xv2_z:%f \n", a0[2], a1[2], a2[2], p1[2] );
      printf("  v1r:%f v2r:%f v3r:%f v1xv2_r:%f \n", sqrt(a0[0]*a0[0]+a0[1]*a0[1]+a0[2]*a0[2]) ,
        sqrt(a1[0]*a1[0]+a1[1]*a1[1]+a1[2]*a1[2]), sqrt(a2[0]*a2[0]+a2[1]*a2[1]+a2[2]*a2[2]) ,
        sqrt(p1[0]*p1[0]+p1[1]*p1[1]+p1[2]*p1[2]) );
#endif
      return (-1);
    }
    return(1);
}
