/* --------------------------------------------------------------------  */
/*                          CALCULIX                                     */
/*                   - GRAPHICAL INTERFACE -                             */
/*                                                                       */
/*     A 3-dimensional pre- and post-processor for finite elements       */
/*              Copyright (C) 1996 Klaus Wittig                          */
/*                                                                       */
/*     This program is free software; you can redistribute it and/or     */
/*     modify it under the terms of the GNU General Public License as    */
/*     published by the Free Software Foundation; version 2 of           */
/*     the License.                                                      */
/*                                                                       */
/*     This program is distributed in the hope that it will be useful,   */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of    */ 
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      */
/*     GNU General Public License for more details.                      */
/*                                                                       */
/*     You should have received a copy of the GNU General Public License */
/*     along with this program; if not, write to the Free Software       */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         */
/* --------------------------------------------------------------------  */

/*
TODO
improved elem-edges for all elems.
*/

#include <cgx.h>

#define     TEST            0

extern float foregrndcol_rgb[4];

extern Normalen *norm;
extern float      ds;


/*------------------------------------------------------------------------ */
/* Berechnen der normalen auf den element faces                            */
/*------------------------------------------------------------------------ */

void getCTri3Normalen( CTri3 *ctri3, Nodes *node, int *num_etype, Normalen *norm )
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
   calcNormale_tri3(i, ctri3[i].nod[0], ctri3[i].nod[1], ctri3[i].nod[2],0, node, norm);
  }
}

void getCTri6Normalen( CTri6 *ctri6, Nodes *node, int *num_etype, Normalen *norm )
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
   calcNormale_tri3(i, ctri6[i].nod[0], ctri6[i].nod[3], ctri6[i].nod[5],0, node, norm);
   calcNormale_tri3(i, ctri6[i].nod[2], ctri6[i].nod[5], ctri6[i].nod[4],1, node, norm);
   calcNormale_tri3(i, ctri6[i].nod[4], ctri6[i].nod[5], ctri6[i].nod[3],2, node, norm);
   calcNormale_tri3(i, ctri6[i].nod[3], ctri6[i].nod[1], ctri6[i].nod[4],3, node, norm);
  }
}

void getCQuad4Normalen( CQuad4 *cquad4, Nodes *node, int *num_etype, Normalen *norm )
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
    calcNormale_quad4(i,cquad4[i].nod[0], cquad4[i].nod[1], cquad4[i].nod[2],
                  cquad4[i].nod[3], 0, node, norm);
  }
}

void getCQuad8Normalen( CHexa8 *cquad8, Nodes *node, int *num_etype,  Normalen *norm)
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
   calcNormale_tri3(i, cquad8[i].nod[ 0], cquad8[i].nod[ 4], cquad8[i].nod[ 7], 0, node, norm);
   calcNormale_tri3(i, cquad8[i].nod[ 7], cquad8[i].nod[ 4], cquad8[i].nod[ 6], 1, node, norm);
   calcNormale_tri3(i, cquad8[i].nod[ 6], cquad8[i].nod[ 3], cquad8[i].nod[ 7], 2, node, norm);
   calcNormale_tri3(i, cquad8[i].nod[ 2], cquad8[i].nod[ 6], cquad8[i].nod[ 5], 3, node, norm);
   calcNormale_tri3(i, cquad8[i].nod[ 5], cquad8[i].nod[ 6], cquad8[i].nod[ 4], 4, node, norm);
   calcNormale_tri3(i, cquad8[i].nod[ 4], cquad8[i].nod[ 1], cquad8[i].nod[ 5], 5, node, norm);
  }
}

void getCHexa8Normalen( CHexa8 *chexa8, Nodes *node, int *num_etype, Normalen *norm )
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
    calcNormale_quad4(i,chexa8[i].nod[1], chexa8[i].nod[2],
                  chexa8[i].nod[6], chexa8[i].nod[5], 0, node, norm);

    calcNormale_quad4(i,chexa8[i].nod[5], chexa8[i].nod[6],
                  chexa8[i].nod[7], chexa8[i].nod[4], 1, node, norm);

    calcNormale_quad4(i,chexa8[i].nod[4], chexa8[i].nod[7],
                  chexa8[i].nod[3], chexa8[i].nod[0], 2, node, norm);

    calcNormale_quad4(i,chexa8[i].nod[5], chexa8[i].nod[4],
                  chexa8[i].nod[0], chexa8[i].nod[1], 3, node, norm);

    calcNormale_quad4(i,chexa8[i].nod[1], chexa8[i].nod[0],
                  chexa8[i].nod[3], chexa8[i].nod[2], 4, node, norm);

    calcNormale_quad4(i,chexa8[i].nod[2], chexa8[i].nod[3],
                  chexa8[i].nod[7], chexa8[i].nod[6], 5, node, norm);
  }
}

void getCPenta6Normalen( CPenta6 *cpenta6, Nodes *node, int *num_etype, Normalen *norm )
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
    calcNormale_tri3(i, cpenta6[i].nod[ 0], cpenta6[i].nod[ 2], cpenta6[i].nod[ 1], 0, node, norm);
    calcNormale_tri3(i, cpenta6[i].nod[ 5], cpenta6[i].nod[ 3], cpenta6[i].nod[ 4], 1, node, norm);
    calcNormale_quad4(i,cpenta6[i].nod[0], cpenta6[i].nod[1],
                  cpenta6[i].nod[4], cpenta6[i].nod[3], 2, node, norm);
    calcNormale_quad4(i,cpenta6[i].nod[1], cpenta6[i].nod[2],
                  cpenta6[i].nod[5], cpenta6[i].nod[4], 3, node, norm);
    calcNormale_quad4(i,cpenta6[i].nod[3], cpenta6[i].nod[5],
                  cpenta6[i].nod[2], cpenta6[i].nod[0], 4, node, norm);
  }
}

void getCTet4Normalen( CTet4 *ctet4, Nodes *node, int *num_etype,  Normalen *norm)
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
   calcNormale_tri3(i, ctet4[i].nod[ 0], ctet4[i].nod[ 1], ctet4[i].nod[ 3], 0, node, norm);
   calcNormale_tri3(i, ctet4[i].nod[ 1], ctet4[i].nod[ 2], ctet4[i].nod[ 3], 1, node, norm);
   calcNormale_tri3(i, ctet4[i].nod[ 2], ctet4[i].nod[ 0], ctet4[i].nod[ 3], 2, node, norm);
   calcNormale_tri3(i, ctet4[i].nod[ 0], ctet4[i].nod[ 2], ctet4[i].nod[ 1], 3, node, norm);
  }
}

void getCTet10Normalen( CTet10 *ctet10, Nodes *node, int *num_etype,  Normalen *norm)
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
   calcNormale_tri3(i, ctet10[i].nod[ 0], ctet10[i].nod[ 4], ctet10[i].nod[ 7], 0, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 7], ctet10[i].nod[ 4], ctet10[i].nod[ 8], 1, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 8], ctet10[i].nod[ 4], ctet10[i].nod[ 1], 2, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 7], ctet10[i].nod[ 8], ctet10[i].nod[ 3], 3, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 1], ctet10[i].nod[ 5], ctet10[i].nod[ 8], 4, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 8], ctet10[i].nod[ 5], ctet10[i].nod[ 9], 5, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 9], ctet10[i].nod[ 5], ctet10[i].nod[ 2], 6, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 8], ctet10[i].nod[ 9], ctet10[i].nod[ 3], 7, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 2], ctet10[i].nod[ 6], ctet10[i].nod[ 9], 8, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 9], ctet10[i].nod[ 6], ctet10[i].nod[ 7], 9, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 7], ctet10[i].nod[ 6], ctet10[i].nod[ 0],10, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 9], ctet10[i].nod[ 7], ctet10[i].nod[ 3],11, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 0], ctet10[i].nod[ 6], ctet10[i].nod[ 4],12, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 4], ctet10[i].nod[ 6], ctet10[i].nod[ 5],13, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 5], ctet10[i].nod[ 6], ctet10[i].nod[ 2],14, node, norm);
   calcNormale_tri3(i, ctet10[i].nod[ 4], ctet10[i].nod[ 5], ctet10[i].nod[ 1],15, node, norm);
  }
}

void getCHexa20Normalen( CHexa20 *chexa20, Nodes *node, int *num_etype,  Normalen *norm)
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
   calcNormale_tri3(i, chexa20[i].nod[ 1], chexa20[i].nod[ 9], chexa20[i].nod[13], 0, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 9], chexa20[i].nod[21], chexa20[i].nod[13], 1, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[13], chexa20[i].nod[21], chexa20[i].nod[ 5], 2, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[21], chexa20[i].nod[17], chexa20[i].nod[ 5], 3, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 5], chexa20[i].nod[17], chexa20[i].nod[16], 4, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[17], chexa20[i].nod[25], chexa20[i].nod[16], 5, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[16], chexa20[i].nod[25], chexa20[i].nod[ 4], 6, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[25], chexa20[i].nod[19], chexa20[i].nod[ 4], 7, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 4], chexa20[i].nod[19], chexa20[i].nod[12], 8, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[19], chexa20[i].nod[23], chexa20[i].nod[12], 9, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[12], chexa20[i].nod[23], chexa20[i].nod[ 0],10, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[23], chexa20[i].nod[11], chexa20[i].nod[ 0],11, node, norm);

   calcNormale_tri3(i, chexa20[i].nod[ 9], chexa20[i].nod[ 2], chexa20[i].nod[21],12, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 2], chexa20[i].nod[14], chexa20[i].nod[21],13, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[21], chexa20[i].nod[14], chexa20[i].nod[17],14, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[14], chexa20[i].nod[ 6], chexa20[i].nod[17],15, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[17], chexa20[i].nod[ 6], chexa20[i].nod[25],16, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 6], chexa20[i].nod[18], chexa20[i].nod[25],17, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[25], chexa20[i].nod[18], chexa20[i].nod[19],18, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[18], chexa20[i].nod[ 7], chexa20[i].nod[19],19, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[19], chexa20[i].nod[ 7], chexa20[i].nod[23],20, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 7], chexa20[i].nod[15], chexa20[i].nod[23],21, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[23], chexa20[i].nod[15], chexa20[i].nod[11],22, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[15], chexa20[i].nod[ 3], chexa20[i].nod[11],23, node, norm);

   calcNormale_tri3(i, chexa20[i].nod[ 7], chexa20[i].nod[18], chexa20[i].nod[15],24, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[18], chexa20[i].nod[22], chexa20[i].nod[15],25, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[15], chexa20[i].nod[22], chexa20[i].nod[ 3],26, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[22], chexa20[i].nod[10], chexa20[i].nod[ 3],27, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 3], chexa20[i].nod[10], chexa20[i].nod[11],28, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[10], chexa20[i].nod[24], chexa20[i].nod[11],29, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[11], chexa20[i].nod[24], chexa20[i].nod[ 0],30, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[24], chexa20[i].nod[ 8], chexa20[i].nod[ 0],31, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 0], chexa20[i].nod[ 8], chexa20[i].nod[12],32, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 8], chexa20[i].nod[20], chexa20[i].nod[12],33, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[12], chexa20[i].nod[20], chexa20[i].nod[ 4],34, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[20], chexa20[i].nod[16], chexa20[i].nod[ 4],35, node, norm);

   calcNormale_tri3(i, chexa20[i].nod[18], chexa20[i].nod[ 6], chexa20[i].nod[22],36, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 6], chexa20[i].nod[14], chexa20[i].nod[22],37, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[22], chexa20[i].nod[14], chexa20[i].nod[10],38, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[14], chexa20[i].nod[ 2], chexa20[i].nod[10],39, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[10], chexa20[i].nod[ 2], chexa20[i].nod[24],40, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 2], chexa20[i].nod[ 9], chexa20[i].nod[24],41, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[24], chexa20[i].nod[ 9], chexa20[i].nod[ 8],42, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 9], chexa20[i].nod[ 1], chexa20[i].nod[ 8],43, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 8], chexa20[i].nod[ 1], chexa20[i].nod[20],44, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[ 1], chexa20[i].nod[13], chexa20[i].nod[20],45, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[20], chexa20[i].nod[13], chexa20[i].nod[16],46, node, norm);
   calcNormale_tri3(i, chexa20[i].nod[13], chexa20[i].nod[ 5], chexa20[i].nod[16],47, node, norm);
  }
}


void getCPenta20Normalen( CPenta20 *cpenta20, Nodes *node, int *num_etype,  Normalen *norm)
{
  register int i;

  for (i=0; i<*num_etype; i++ )
  {
   calcNormale_tri3(i, cpenta20[i].nod[ 1], cpenta20[i].nod[10], cpenta20[i].nod[ 6], 0, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 6], cpenta20[i].nod[10], cpenta20[i].nod[15], 1, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 6], cpenta20[i].nod[15], cpenta20[i].nod[ 0], 2, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 0], cpenta20[i].nod[15], cpenta20[i].nod[ 9], 3, node, norm);

   calcNormale_tri3(i, cpenta20[i].nod[10], cpenta20[i].nod[ 4], cpenta20[i].nod[15], 4, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[15], cpenta20[i].nod[ 4], cpenta20[i].nod[12], 5, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[15], cpenta20[i].nod[12], cpenta20[i].nod[ 9], 6, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 9], cpenta20[i].nod[12], cpenta20[i].nod[ 3], 7, node, norm);


   calcNormale_tri3(i, cpenta20[i].nod[ 2], cpenta20[i].nod[11], cpenta20[i].nod[ 7], 8, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 7], cpenta20[i].nod[11], cpenta20[i].nod[16], 9, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 7], cpenta20[i].nod[16], cpenta20[i].nod[ 1],10, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 1], cpenta20[i].nod[16], cpenta20[i].nod[10],11, node, norm);

   calcNormale_tri3(i, cpenta20[i].nod[11], cpenta20[i].nod[ 5], cpenta20[i].nod[16],12, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[16], cpenta20[i].nod[ 5], cpenta20[i].nod[13],13, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[16], cpenta20[i].nod[13], cpenta20[i].nod[10],14, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[10], cpenta20[i].nod[13], cpenta20[i].nod[ 4],15, node, norm);


   calcNormale_tri3(i, cpenta20[i].nod[ 0], cpenta20[i].nod[ 9], cpenta20[i].nod[ 8],16, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 8], cpenta20[i].nod[ 9], cpenta20[i].nod[17],17, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 8], cpenta20[i].nod[17], cpenta20[i].nod[ 2],18, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 2], cpenta20[i].nod[17], cpenta20[i].nod[11],19, node, norm);

   calcNormale_tri3(i, cpenta20[i].nod[ 9], cpenta20[i].nod[ 3], cpenta20[i].nod[17],20, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[17], cpenta20[i].nod[ 3], cpenta20[i].nod[14],21, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[17], cpenta20[i].nod[14], cpenta20[i].nod[11],22, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[11], cpenta20[i].nod[14], cpenta20[i].nod[ 5],23, node, norm);


   calcNormale_tri3(i, cpenta20[i].nod[ 2], cpenta20[i].nod[ 7], cpenta20[i].nod[ 8],24, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 8], cpenta20[i].nod[ 7], cpenta20[i].nod[18],25, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 8], cpenta20[i].nod[18], cpenta20[i].nod[ 0],26, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 0], cpenta20[i].nod[18], cpenta20[i].nod[ 0],27, node, norm);

   calcNormale_tri3(i, cpenta20[i].nod[ 7], cpenta20[i].nod[ 1], cpenta20[i].nod[18],28, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[18], cpenta20[i].nod[ 1], cpenta20[i].nod[ 6],29, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[18], cpenta20[i].nod[ 6], cpenta20[i].nod[ 0],30, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 0], cpenta20[i].nod[ 6], cpenta20[i].nod[ 0],31, node, norm);


   calcNormale_tri3(i, cpenta20[i].nod[ 4], cpenta20[i].nod[13], cpenta20[i].nod[12],32, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[12], cpenta20[i].nod[13], cpenta20[i].nod[19],33, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[12], cpenta20[i].nod[19], cpenta20[i].nod[ 3],34, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 3], cpenta20[i].nod[19], cpenta20[i].nod[ 3],35, node, norm);

   calcNormale_tri3(i, cpenta20[i].nod[13], cpenta20[i].nod[ 5], cpenta20[i].nod[19],36, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[19], cpenta20[i].nod[ 5], cpenta20[i].nod[14],37, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[19], cpenta20[i].nod[14], cpenta20[i].nod[ 3],38, node, norm);
   calcNormale_tri3(i, cpenta20[i].nod[ 3], cpenta20[i].nod[14], cpenta20[i].nod[ 3],39, node, norm);
  }
}



/*------------------------------------------------------------------------ */
/* Displaylisten fuer die freien kanten der elemente                       */
/*------------------------------------------------------------------------ */

void drawModelEdges( GLuint list_model_edges, int color, float width,
                     int numEdges, Nodes *node, Edges *edge )
{
  register int i;

#if TEST
  printf ("in drawModelEdges numEdges:%d\n",numEdges );
#endif 

  glNewList( list_model_edges, GL_COMPILE );
#if RGB_MODE
   glColor3f    ( foregrndcol_rgb[0],foregrndcol_rgb[1],foregrndcol_rgb[2] );
#else
  glIndexi    ( color );
#endif
   glLineWidth(width);
   glBegin ( GL_LINES );
   for (i=0; i<numEdges; i++ )
   {
      glVertex3fv ( &node[edge[i].p1].nx );
      glVertex3fv ( &node[edge[i].p2].nx );
   }
   glEnd();
  glEndList();
}


/*------------------------------------------------------------------------ */
/* Displaylisten fuer die auf der oberflaeche liegenden kanten der elemente */
/*------------------------------------------------------------------------ */

void drawSurfEdges( GLuint list_surf_edges, int color, float width, int  elemGroupNr,
                     int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CHexa8 *surf8,
                     CTri3 *ctri3, CTri6 *ctri6, CQuad4 *cquad4, CBeam *cbeam, CBeam3 *cbeam3, CTri3 *surf3, CTri6 *surf6, CHexa8 *cquad8 )
{
  register int i;

#if TEST
  printf ("in drawSurfEdges\n");
#endif 

  glNewList( list_surf_edges, GL_COMPILE );
#if RGB_MODE
  glColor3f    ( foregrndcol_rgb[0],foregrndcol_rgb[1],foregrndcol_rgb[2] );
#else
  glIndexi    ( color );
#endif
  glLineWidth(width);
  if (num_stype[7])
  {
   getCTri3Normalen( surf3, node, &num_stype[7], norm );
   for (i=0; i<num_stype[7]; i++ )
   {
    if(surf3[i].mat <1) if((surf3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3f ( node[surf3[i].nod[0]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[surf3[i].nod[0]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[surf3[i].nod[0]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[surf3[i].nod[1]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[surf3[i].nod[1]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[surf3[i].nod[1]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[surf3[i].nod[2]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[surf3[i].nod[2]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[surf3[i].nod[2]].nz+norm[i].side[0][2]*1.e-3*ds );
    glEnd();
    }
   }
  }
  if (num_stype[8])
  {
   getCTri6Normalen( surf6, node, &num_stype[8], norm );
   for (i=0; i<num_stype[8]; i++ )
   {
    if(surf6[i].mat <1) if ((surf6[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3f ( node[surf6[i].nod[0]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[surf6[i].nod[0]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[surf6[i].nod[0]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[surf6[i].nod[3]].nx+norm[i].side[2][0]*1.e-3*ds, 
                   node[surf6[i].nod[3]].ny+norm[i].side[2][1]*1.e-3*ds,
                   node[surf6[i].nod[3]].nz+norm[i].side[2][2]*1.e-3*ds );
      glVertex3f ( node[surf6[i].nod[1]].nx+norm[i].side[3][0]*1.e-3*ds, 
                   node[surf6[i].nod[1]].ny+norm[i].side[3][1]*1.e-3*ds,
                   node[surf6[i].nod[1]].nz+norm[i].side[3][2]*1.e-3*ds );
      glVertex3f ( node[surf6[i].nod[4]].nx+norm[i].side[2][0]*1.e-3*ds, 
                   node[surf6[i].nod[4]].ny+norm[i].side[2][1]*1.e-3*ds,
                   node[surf6[i].nod[4]].nz+norm[i].side[2][2]*1.e-3*ds );
      glVertex3f ( node[surf6[i].nod[2]].nx+norm[i].side[1][0]*1.e-3*ds, 
                   node[surf6[i].nod[2]].ny+norm[i].side[1][1]*1.e-3*ds,
                   node[surf6[i].nod[2]].nz+norm[i].side[1][2]*1.e-3*ds );
      glVertex3f ( node[surf6[i].nod[5]].nx+norm[i].side[2][0]*1.e-3*ds, 
                   node[surf6[i].nod[5]].ny+norm[i].side[2][1]*1.e-3*ds,
                   node[surf6[i].nod[5]].nz+norm[i].side[2][2]*1.e-3*ds );
    glEnd();
    }
   }
  }
  if (num_stype[9])
  {
   getCQuad4Normalen( surf4, node, &num_stype[9], norm);
   for (i=0; i<num_stype[9]; i++ )
   {
    if (surf4[i].mat <1) if((surf4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3f ( node[surf4[i].nod[0]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[surf4[i].nod[0]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[surf4[i].nod[0]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[surf4[i].nod[1]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[surf4[i].nod[1]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[surf4[i].nod[1]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[surf4[i].nod[2]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[surf4[i].nod[2]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[surf4[i].nod[2]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[surf4[i].nod[3]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[surf4[i].nod[3]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[surf4[i].nod[3]].nz+norm[i].side[0][2]*1.e-3*ds );
    glEnd();
    }
   }
  }
  if (num_stype[10])
  {
   getCQuad8Normalen(  surf8, node, &num_stype[10], norm );
   for (i=0; i<num_stype[10]; i++ )
   {
    if(surf8[i].mat <1) if((surf8[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3f ( node[surf8[i].nod[0]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[surf8[i].nod[0]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[surf8[i].nod[0]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[surf8[i].nod[4]].nx+norm[i].side[1][0]*1.e-3*ds, 
                   node[surf8[i].nod[4]].ny+norm[i].side[1][1]*1.e-3*ds,
                   node[surf8[i].nod[4]].nz+norm[i].side[1][2]*1.e-3*ds );
      glVertex3f ( node[surf8[i].nod[1]].nx+norm[i].side[5][0]*1.e-3*ds, 
                   node[surf8[i].nod[1]].ny+norm[i].side[5][1]*1.e-3*ds,
                   node[surf8[i].nod[1]].nz+norm[i].side[5][2]*1.e-3*ds );
      glVertex3f ( node[surf8[i].nod[5]].nx+norm[i].side[4][0]*1.e-3*ds, 
                   node[surf8[i].nod[5]].ny+norm[i].side[4][1]*1.e-3*ds,
                   node[surf8[i].nod[5]].nz+norm[i].side[4][2]*1.e-3*ds );
      glVertex3f ( node[surf8[i].nod[2]].nx+norm[i].side[3][0]*1.e-3*ds, 
                   node[surf8[i].nod[2]].ny+norm[i].side[3][1]*1.e-3*ds,
                   node[surf8[i].nod[2]].nz+norm[i].side[3][2]*1.e-3*ds );
      glVertex3f ( node[surf8[i].nod[6]].nx+norm[i].side[4][0]*1.e-3*ds, 
                   node[surf8[i].nod[6]].ny+norm[i].side[4][1]*1.e-3*ds,
                   node[surf8[i].nod[6]].nz+norm[i].side[4][2]*1.e-3*ds );
      glVertex3f ( node[surf8[i].nod[3]].nx+norm[i].side[2][0]*1.e-3*ds, 
                   node[surf8[i].nod[3]].ny+norm[i].side[2][1]*1.e-3*ds,
                   node[surf8[i].nod[3]].nz+norm[i].side[2][2]*1.e-3*ds );
      glVertex3f ( node[surf8[i].nod[7]].nx+norm[i].side[1][0]*1.e-3*ds, 
                   node[surf8[i].nod[7]].ny+norm[i].side[1][1]*1.e-3*ds,
                   node[surf8[i].nod[7]].nz+norm[i].side[1][2]*1.e-3*ds );
    glEnd();
    }
   }
  }
  if (num_etype[7])
  {
   getCTri3Normalen( ctri3, node, &num_etype[7], norm );
   for (i=0; i<num_etype[7]; i++ )
   {
    if( (ctri3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3f ( node[ctri3[i].nod[0]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[ctri3[i].nod[0]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[ctri3[i].nod[0]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[ctri3[i].nod[1]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[ctri3[i].nod[1]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[ctri3[i].nod[1]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[ctri3[i].nod[2]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[ctri3[i].nod[2]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[ctri3[i].nod[2]].nz+norm[i].side[0][2]*1.e-3*ds );
    glEnd();
    }
   }
  }
  if (num_etype[8])
  {
   getCTri6Normalen( ctri6, node, &num_etype[8], norm );
   for (i=0; i<num_etype[8]; i++ )
   {
    if( (ctri6[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3f ( node[ctri6[i].nod[0]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[ctri6[i].nod[0]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[ctri6[i].nod[0]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[ctri6[i].nod[3]].nx+norm[i].side[2][0]*1.e-3*ds, 
                   node[ctri6[i].nod[3]].ny+norm[i].side[2][1]*1.e-3*ds,
                   node[ctri6[i].nod[3]].nz+norm[i].side[2][2]*1.e-3*ds );
      glVertex3f ( node[ctri6[i].nod[1]].nx+norm[i].side[3][0]*1.e-3*ds, 
                   node[ctri6[i].nod[1]].ny+norm[i].side[3][1]*1.e-3*ds,
                   node[ctri6[i].nod[1]].nz+norm[i].side[3][2]*1.e-3*ds );
      glVertex3f ( node[ctri6[i].nod[4]].nx+norm[i].side[2][0]*1.e-3*ds, 
                   node[ctri6[i].nod[4]].ny+norm[i].side[2][1]*1.e-3*ds,
                   node[ctri6[i].nod[4]].nz+norm[i].side[2][2]*1.e-3*ds );
      glVertex3f ( node[ctri6[i].nod[2]].nx+norm[i].side[1][0]*1.e-3*ds, 
                   node[ctri6[i].nod[2]].ny+norm[i].side[1][1]*1.e-3*ds,
                   node[ctri6[i].nod[2]].nz+norm[i].side[1][2]*1.e-3*ds );
      glVertex3f ( node[ctri6[i].nod[5]].nx+norm[i].side[2][0]*1.e-3*ds, 
                   node[ctri6[i].nod[5]].ny+norm[i].side[2][1]*1.e-3*ds,
                   node[ctri6[i].nod[5]].nz+norm[i].side[2][2]*1.e-3*ds );
    glEnd();
    }
   }
  }
  if (num_etype[9])
  {
   getCQuad4Normalen( cquad4, node, &num_etype[9], norm);
   for (i=0; i<num_etype[9]; i++ )
   {
    if( (cquad4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3f ( node[cquad4[i].nod[0]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[cquad4[i].nod[0]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[cquad4[i].nod[0]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[cquad4[i].nod[1]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[cquad4[i].nod[1]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[cquad4[i].nod[1]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[cquad4[i].nod[2]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[cquad4[i].nod[2]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[cquad4[i].nod[2]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[cquad4[i].nod[3]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[cquad4[i].nod[3]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[cquad4[i].nod[3]].nz+norm[i].side[0][2]*1.e-3*ds );
    glEnd();
    }
   }
  }
  if (num_etype[10])
  {
   getCQuad8Normalen(  cquad8, node, &num_etype[10], norm );
   for (i=0; i<num_etype[10]; i++ )
   {
    if( (cquad8[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3f ( node[cquad8[i].nod[0]].nx+norm[i].side[0][0]*1.e-3*ds, 
                   node[cquad8[i].nod[0]].ny+norm[i].side[0][1]*1.e-3*ds,
                   node[cquad8[i].nod[0]].nz+norm[i].side[0][2]*1.e-3*ds );
      glVertex3f ( node[cquad8[i].nod[4]].nx+norm[i].side[1][0]*1.e-3*ds, 
                   node[cquad8[i].nod[4]].ny+norm[i].side[1][1]*1.e-3*ds,
                   node[cquad8[i].nod[4]].nz+norm[i].side[1][2]*1.e-3*ds );
      glVertex3f ( node[cquad8[i].nod[1]].nx+norm[i].side[5][0]*1.e-3*ds, 
                   node[cquad8[i].nod[1]].ny+norm[i].side[5][1]*1.e-3*ds,
                   node[cquad8[i].nod[1]].nz+norm[i].side[5][2]*1.e-3*ds );
      glVertex3f ( node[cquad8[i].nod[5]].nx+norm[i].side[4][0]*1.e-3*ds, 
                   node[cquad8[i].nod[5]].ny+norm[i].side[4][1]*1.e-3*ds,
                   node[cquad8[i].nod[5]].nz+norm[i].side[4][2]*1.e-3*ds );
      glVertex3f ( node[cquad8[i].nod[2]].nx+norm[i].side[3][0]*1.e-3*ds, 
                   node[cquad8[i].nod[2]].ny+norm[i].side[3][1]*1.e-3*ds,
                   node[cquad8[i].nod[2]].nz+norm[i].side[3][2]*1.e-3*ds );
      glVertex3f ( node[cquad8[i].nod[6]].nx+norm[i].side[4][0]*1.e-3*ds, 
                   node[cquad8[i].nod[6]].ny+norm[i].side[4][1]*1.e-3*ds,
                   node[cquad8[i].nod[6]].nz+norm[i].side[4][2]*1.e-3*ds );
      glVertex3f ( node[cquad8[i].nod[3]].nx+norm[i].side[2][0]*1.e-3*ds, 
                   node[cquad8[i].nod[3]].ny+norm[i].side[2][1]*1.e-3*ds,
                   node[cquad8[i].nod[3]].nz+norm[i].side[2][2]*1.e-3*ds );
      glVertex3f ( node[cquad8[i].nod[7]].nx+norm[i].side[1][0]*1.e-3*ds, 
                   node[cquad8[i].nod[7]].ny+norm[i].side[1][1]*1.e-3*ds,
                   node[cquad8[i].nod[7]].nz+norm[i].side[1][2]*1.e-3*ds );
    glEnd();
    }
   }
  }
  if (num_etype[11])
  {
   glBegin ( GL_LINES );
   for (i=0; i<num_etype[11]; i++ )
   {
    if( (cbeam[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glVertex3fv ( &node[cbeam[i].nod[0]].nx );
      glVertex3fv ( &node[cbeam[i].nod[1]].nx );
    }
   }
   glEnd();
  }
  if (num_etype[12])
  {
   for (i=0; i<num_etype[12]; i++ )
   {
    if( (cbeam3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
   glBegin ( GL_LINE_STRIP );
      glVertex3fv ( &node[cbeam3[i].nod[0]].nx );
      glVertex3fv ( &node[cbeam3[i].nod[2]].nx );
      glVertex3fv ( &node[cbeam3[i].nod[1]].nx );
   glEnd();
    }
   }
  }
  glEndList();
}


/*------------------------------------------------------------------------ */
/* Displaylisten fuer alle kanten der elemente                             */
/*------------------------------------------------------------------------ */

void drawElemEdges( GLuint list_elem_edges, int color, float width, int  elemGroupNr,
                     int *num_etype, Nodes *node, CHexa20 *chexa20, CHexa8 *chexa8,
		    CTri3 *ctri3, CTri6 *ctri6, CQuad4 *cquad4, CBeam *cbeam, CBeam3 *cbeam3, CTet4 *ctet4, CTet10 *ctet10, CHexa8 *cquad8, CPenta6 *cpenta6, CPenta20 *cpenta20 )
{
  register int i;

#if TEST
  printf ("in drawElemEdges\n");
#endif 

  glNewList( list_elem_edges, GL_COMPILE );
#if RGB_MODE
  glColor3f    ( foregrndcol_rgb[0],foregrndcol_rgb[1],foregrndcol_rgb[2] );
#else
  glIndexi    ( color );
#endif
  glLineWidth( width );
  if (num_etype[1])
  {
   for (i=0; i<num_etype[1]; i++ )
   {
    if( (chexa8[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[chexa8[i].nod[0]].nx );
      glVertex3fv ( &node[chexa8[i].nod[1]].nx );
      glVertex3fv ( &node[chexa8[i].nod[2]].nx );
      glVertex3fv ( &node[chexa8[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[chexa8[i].nod[4]].nx );
      glVertex3fv ( &node[chexa8[i].nod[5]].nx );
      glVertex3fv ( &node[chexa8[i].nod[6]].nx );
      glVertex3fv ( &node[chexa8[i].nod[7]].nx );
    glEnd();
    glBegin ( GL_LINES );
      glVertex3fv ( &node[chexa8[i].nod[0]].nx );
      glVertex3fv ( &node[chexa8[i].nod[4]].nx );
      glVertex3fv ( &node[chexa8[i].nod[1]].nx );
      glVertex3fv ( &node[chexa8[i].nod[5]].nx );
      glVertex3fv ( &node[chexa8[i].nod[2]].nx );
      glVertex3fv ( &node[chexa8[i].nod[6]].nx );
      glVertex3fv ( &node[chexa8[i].nod[3]].nx );
      glVertex3fv ( &node[chexa8[i].nod[7]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[2])
  {
   for (i=0; i<num_etype[2]; i++ )
   {
    if( (cpenta6[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[cpenta6[i].nod[0]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[2]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[1]].nx );
    glEnd();
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[cpenta6[i].nod[3]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[4]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[5]].nx );
    glEnd();
    glBegin ( GL_LINES );
      glVertex3fv ( &node[cpenta6[i].nod[0]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[3]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[1]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[4]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[2]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[5]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[3])
  {
   for (i=0; i<num_etype[3]; i++ )
   {
    if( (ctet4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_STRIP);
      glVertex3fv ( &node[ctet4[i].nod[0]].nx );
      glVertex3fv ( &node[ctet4[i].nod[1]].nx );
      glVertex3fv ( &node[ctet4[i].nod[2]].nx );
      glVertex3fv ( &node[ctet4[i].nod[3]].nx );
      glVertex3fv ( &node[ctet4[i].nod[1]].nx );
    glEnd();
    glBegin ( GL_LINE_STRIP );
      glVertex3fv ( &node[ctet4[i].nod[2]].nx );
      glVertex3fv ( &node[ctet4[i].nod[0]].nx );
      glVertex3fv ( &node[ctet4[i].nod[3]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[4])
  {
   for (i=0; i<num_etype[4]; i++ )
   {
    if( (chexa20[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[chexa20[i].nod[0]].nx );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
      glVertex3fv ( &node[chexa20[i].nod[3]].nx );
      glVertex3fv ( &node[chexa20[i].nod[10]].nx );
      glVertex3fv ( &node[chexa20[i].nod[2]].nx );
      glVertex3fv ( &node[chexa20[i].nod[9]].nx );
      glVertex3fv ( &node[chexa20[i].nod[1]].nx );
      glVertex3fv ( &node[chexa20[i].nod[8]].nx );
    glEnd();
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[chexa20[i].nod[4]].nx );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
      glVertex3fv ( &node[chexa20[i].nod[5]].nx );
      glVertex3fv ( &node[chexa20[i].nod[17]].nx );
      glVertex3fv ( &node[chexa20[i].nod[6]].nx );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
      glVertex3fv ( &node[chexa20[i].nod[7]].nx );
      glVertex3fv ( &node[chexa20[i].nod[19]].nx );
    glEnd();
    glBegin ( GL_LINES );
      glVertex3fv ( &node[chexa20[i].nod[0]].nx );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glVertex3fv ( &node[chexa20[i].nod[4]].nx );
      glVertex3fv ( &node[chexa20[i].nod[1]].nx );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glVertex3fv ( &node[chexa20[i].nod[5]].nx );
      glVertex3fv ( &node[chexa20[i].nod[2]].nx );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glVertex3fv ( &node[chexa20[i].nod[6]].nx );
      glVertex3fv ( &node[chexa20[i].nod[3]].nx );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glVertex3fv ( &node[chexa20[i].nod[7]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[5])
  {
   for (i=0; i<num_etype[5]; i++ )
   {
    if( (cpenta20[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[cpenta20[i].nod[0]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 8]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[2]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 7]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[1]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[6]].nx );
    glEnd();
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[cpenta20[i].nod[3]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[14]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[5]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[4]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[12]].nx );
    glEnd();
    glBegin ( GL_LINES );
      glVertex3fv ( &node[cpenta20[i].nod[0]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[3]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 1]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 4]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[2]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[5]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[6])
  {
   for (i=0; i<num_etype[6]; i++ )
   {
    if( (ctet10[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_STRIP);
      glVertex3fv ( &node[ctet10[i].nod[0]].nx );
      glVertex3fv ( &node[ctet10[i].nod[7]].nx );
      glVertex3fv ( &node[ctet10[i].nod[3]].nx );
      glVertex3fv ( &node[ctet10[i].nod[8]].nx );
      glVertex3fv ( &node[ctet10[i].nod[1]].nx );
      glVertex3fv ( &node[ctet10[i].nod[5]].nx );
      glVertex3fv ( &node[ctet10[i].nod[2]].nx );
      glVertex3fv ( &node[ctet10[i].nod[9]].nx );
      glVertex3fv ( &node[ctet10[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_LINE_STRIP );
      glVertex3fv ( &node[ctet10[i].nod[2]].nx );
      glVertex3fv ( &node[ctet10[i].nod[6]].nx );
      glVertex3fv ( &node[ctet10[i].nod[0]].nx );
      glVertex3fv ( &node[ctet10[i].nod[4]].nx );
      glVertex3fv ( &node[ctet10[i].nod[1]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[7])
  {
   for (i=0; i<num_etype[7]; i++ )
   {
    if( (ctri3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[ctri3[i].nod[0]].nx );
      glVertex3fv ( &node[ctri3[i].nod[1]].nx );
      glVertex3fv ( &node[ctri3[i].nod[2]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[8])
  {
   for (i=0; i<num_etype[8]; i++ )
   {
    if( (ctri6[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[ctri6[i].nod[0]].nx );
      glVertex3fv ( &node[ctri6[i].nod[3]].nx );
      glVertex3fv ( &node[ctri6[i].nod[1]].nx );
      glVertex3fv ( &node[ctri6[i].nod[4]].nx );
      glVertex3fv ( &node[ctri6[i].nod[2]].nx );
      glVertex3fv ( &node[ctri6[i].nod[5]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[9])
  {
   for (i=0; i<num_etype[9]; i++ )
   {
    if( (cquad4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[cquad4[i].nod[0]].nx );
      glVertex3fv ( &node[cquad4[i].nod[1]].nx );
      glVertex3fv ( &node[cquad4[i].nod[2]].nx );
      glVertex3fv ( &node[cquad4[i].nod[3]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[10])
  {
   for (i=0; i<num_etype[10]; i++ )
   {
    if( (cquad8[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINE_LOOP );
      glVertex3fv ( &node[cquad8[i].nod[0]].nx );
      glVertex3fv ( &node[cquad8[i].nod[4]].nx );
      glVertex3fv ( &node[cquad8[i].nod[1]].nx );
      glVertex3fv ( &node[cquad8[i].nod[5]].nx );
      glVertex3fv ( &node[cquad8[i].nod[2]].nx );
      glVertex3fv ( &node[cquad8[i].nod[6]].nx );
      glVertex3fv ( &node[cquad8[i].nod[3]].nx );
      glVertex3fv ( &node[cquad8[i].nod[7]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[11])
  {
   glBegin ( GL_LINES );
   for (i=0; i<num_etype[11]; i++ )
   {
    if( (cbeam[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glVertex3fv ( &node[cbeam[i].nod[0]].nx );
      glVertex3fv ( &node[cbeam[i].nod[1]].nx );
    }
   }
   glEnd();
  }
  if (num_etype[12])
  {
   for (i=0; i<num_etype[12]; i++ )
   {
    if( (cbeam3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
   glBegin ( GL_LINE_STRIP );
      glVertex3fv ( &node[cbeam3[i].nod[0]].nx );
      glVertex3fv ( &node[cbeam3[i].nod[1]].nx );
      glVertex3fv ( &node[cbeam3[i].nod[2]].nx );
   glEnd();
    }
   }
  }
  glEndList();
}


/*------------------------------------------------------------------------ */
/* Displaylisten fuer die beleuchteten elemente im surface mode            */
/*------------------------------------------------------------------------ */

void drawSurfaces_light( GLuint list_surf_light, unsigned char cullFlag, int  elemGroupNr, Normalen *norm,
                     int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CTri3 *surf3,
                     CTri3 *ctri3, CTri6 *ctri6, CQuad4 *cquad4, CBeam *cbeam, CBeam3 *cbeam3, CHexa8 *cquad8 )
{
  int i;

#if TEST
  printf ("in drawSurfaces_light\n");
#endif 
  glNewList( list_surf_light, GL_COMPILE );
  if (num_stype[9])
  {
   if(cullFlag) glEnable ( GL_CULL_FACE );
   getCQuad4Normalen( surf4, node, &num_stype[9], norm);
   glBegin ( GL_TRIANGLES );
   for (i=0; i<num_stype[9]; i++ )
   {
    if(surf4[i].mat >-1) if((surf4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[surf4[i].nod[0]].nx );
      glVertex3fv ( &node[surf4[i].nod[1]].nx );
      glVertex3fv ( &node[surf4[i].nod[3]].nx );

      glVertex3fv ( &node[surf4[i].nod[2]].nx );
      glVertex3fv ( &node[surf4[i].nod[3]].nx );
      glVertex3fv ( &node[surf4[i].nod[1]].nx );
    }
   }
   glEnd();
  }
  if (num_stype[7])
  {
   if(cullFlag) glEnable ( GL_CULL_FACE );
   getCTri3Normalen( surf3, node, &num_stype[7], norm );
   glBegin ( GL_TRIANGLES );
   for (i=0; i<num_stype[7]; i++ )
   {
    if(surf3[i].mat >-1) if( (surf3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      /*
      printf("no %f %f %f\n", norm[i].side[0][0],norm[i].side[0][1],norm[i].side[0][2]);
      printf("ko %f %f %f\n", node[surf3[i].nod[0]].nx, node[surf3[i].nod[0]].ny, node[surf3[i].nod[0]].nz );
      */
      glNormal3fv ( norm[i].side[0] );
      glEdgeFlag ( GL_TRUE );
      glVertex3fv ( &node[surf3[i].nod[0]].nx );
      /*glEdgeFlag ( GL_FALSE );*/
      glVertex3fv ( &node[surf3[i].nod[1]].nx );
      glVertex3fv ( &node[surf3[i].nod[2]].nx );
      glEdgeFlag ( GL_TRUE );
    }
   }
   glEnd();
  }
  if (num_etype[7])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   getCTri3Normalen( ctri3, node, &num_etype[7], norm );
   glBegin ( GL_TRIANGLES );
   for (i=0; i<num_etype[7]; i++ )
   {
    if( (ctri3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[ctri3[i].nod[0]].nx );
      glVertex3fv ( &node[ctri3[i].nod[1]].nx );
      glVertex3fv ( &node[ctri3[i].nod[2]].nx );
    }
   }
   glEnd();
  }
  if (num_etype[8])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   getCTri6Normalen( ctri6, node, &num_etype[8], norm );
   glBegin ( GL_TRIANGLES );
   for (i=0; i<num_etype[8]; i++ )
   {
    if( (ctri6[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[ctri6[i].nod[0]].nx );
      glVertex3fv ( &node[ctri6[i].nod[3]].nx );
      glVertex3fv ( &node[ctri6[i].nod[5]].nx );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[ctri6[i].nod[2]].nx );
      glVertex3fv ( &node[ctri6[i].nod[5]].nx );
      glVertex3fv ( &node[ctri6[i].nod[4]].nx );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[ctri6[i].nod[4]].nx );
      glVertex3fv ( &node[ctri6[i].nod[5]].nx );
      glVertex3fv ( &node[ctri6[i].nod[3]].nx );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[ctri6[i].nod[3]].nx );
      glVertex3fv ( &node[ctri6[i].nod[1]].nx );
      glVertex3fv ( &node[ctri6[i].nod[4]].nx );
    }
   }
   glEnd();
  }
  if (num_etype[9])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   getCQuad4Normalen( cquad4, node, &num_etype[9], norm );
   for (i=0; i<num_etype[9]; i++ )
   {
    if( (cquad4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[cquad4[i].nod[0]].nx );
      glVertex3fv ( &node[cquad4[i].nod[1]].nx );
      glVertex3fv ( &node[cquad4[i].nod[3]].nx );
      glVertex3fv ( &node[cquad4[i].nod[2]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[10])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   getCQuad8Normalen( cquad8, node, &num_etype[10], norm );
   for (i=0; i<num_etype[10]; i++ )
   {
    if( (cquad8[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[cquad8[i].nod[0]].nx );
      glVertex3fv ( &node[cquad8[i].nod[4]].nx );
      glVertex3fv ( &node[cquad8[i].nod[7]].nx );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[cquad8[i].nod[6]].nx );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[cquad8[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[cquad8[i].nod[2]].nx );
      glVertex3fv ( &node[cquad8[i].nod[6]].nx );
      glVertex3fv ( &node[cquad8[i].nod[5]].nx );
      glNormal3fv ( norm[i].side[4] );
      glVertex3fv ( &node[cquad8[i].nod[4]].nx );
      glNormal3fv ( norm[i].side[5] );
      glVertex3fv ( &node[cquad8[i].nod[1]].nx );
    glEnd();
    }
   }
  }
  if (num_etype[11])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   glBegin ( GL_LINES );
   for (i=0; i<num_etype[11]; i++ )
   {
    if( (cbeam[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glNormal3f  ( 1., 0., 0. );
      glVertex3fv ( &node[cbeam[i].nod[0]].nx );
      glVertex3fv ( &node[cbeam[i].nod[1]].nx );
    }
   }
   glEnd();
  }
  if (num_etype[12])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   for (i=0; i<num_etype[12]; i++ )
   {
    if( (cbeam3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
   glBegin ( GL_LINE_STRIP );
      glNormal3f  ( 1., 0., 0. );
      glVertex3fv ( &node[cbeam3[i].nod[0]].nx );
      glVertex3fv ( &node[cbeam3[i].nod[2]].nx );
      glVertex3fv ( &node[cbeam3[i].nod[1]].nx );
   glEnd();
    }
   }
  }
  glEndList();
}



/*------------------------------------------------------------------------ */
/* Displaylisten fuer die beleuchteten elemente im solid mode              */
/*------------------------------------------------------------------------ */

void drawElements_light( GLuint list_elem_light,  unsigned char cullFlag, int  elemGroupNr,  Normalen *norm,
                     int *num_etype, Nodes *node, CHexa20 *chexa20, CHexa8 *chexa8,
			 CTri3 *ctri3, CTri6 *ctri6, CQuad4 *cquad4, CBeam *cbeam, CBeam3 *cbeam3, CTet4 *ctet4, CTet10 *ctet10, CHexa8 *cquad8, CPenta6 *cpenta6, CPenta20 *cpenta20 )
{
  register int i;

#if TEST
  printf ("in drawElements_light\n");
#endif 

  glNewList( list_elem_light, GL_COMPILE );

 if (num_etype[1])
 {
  if(cullFlag) glEnable ( GL_CULL_FACE );
  getCHexa8Normalen( chexa8, node, &num_etype[1], norm );
  for (i=0; i<num_etype[1]; i++ )
  {
   if( (chexa8[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[chexa8[i].nod[1]].nx );
      glVertex3fv ( &node[chexa8[i].nod[2]].nx );
      glVertex3fv ( &node[chexa8[i].nod[5]].nx );
      glVertex3fv ( &node[chexa8[i].nod[6]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[chexa8[i].nod[5]].nx );
      glVertex3fv ( &node[chexa8[i].nod[6]].nx );
      glVertex3fv ( &node[chexa8[i].nod[4]].nx );
      glVertex3fv ( &node[chexa8[i].nod[7]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[chexa8[i].nod[4]].nx );
      glVertex3fv ( &node[chexa8[i].nod[7]].nx );
      glVertex3fv ( &node[chexa8[i].nod[0]].nx );
      glVertex3fv ( &node[chexa8[i].nod[3]].nx );
    glEnd();
    /* math.pos von z->y  */
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[chexa8[i].nod[5]].nx );
      glVertex3fv ( &node[chexa8[i].nod[4]].nx );
      glVertex3fv ( &node[chexa8[i].nod[1]].nx );
      glVertex3fv ( &node[chexa8[i].nod[0]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[4] );
      glVertex3fv ( &node[chexa8[i].nod[1]].nx );
      glVertex3fv ( &node[chexa8[i].nod[0]].nx );
      glVertex3fv ( &node[chexa8[i].nod[2]].nx );
      glVertex3fv ( &node[chexa8[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[5] );
      glVertex3fv ( &node[chexa8[i].nod[2]].nx );
      glVertex3fv ( &node[chexa8[i].nod[3]].nx );
      glVertex3fv ( &node[chexa8[i].nod[6]].nx );
      glVertex3fv ( &node[chexa8[i].nod[7]].nx );
    glEnd();
   }
  }
  if(cullFlag) glDisable ( GL_CULL_FACE );
 }
 if (num_etype[2])
 {
  if(cullFlag) glEnable ( GL_CULL_FACE );
  getCPenta6Normalen( cpenta6, node, &num_etype[2], norm );
  for (i=0; i<num_etype[2]; i++ )
  {
   if( (cpenta6[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
    glBegin ( GL_TRIANGLES      );
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[cpenta6[i].nod[0]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[2]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[1]].nx );
    glEnd();
    glBegin ( GL_TRIANGLES      );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[cpenta6[i].nod[3]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[4]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[5]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[cpenta6[i].nod[0]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[1]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[3]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[4]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[cpenta6[i].nod[1]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[2]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[4]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[5]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[4] );
      glVertex3fv ( &node[cpenta6[i].nod[0]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[3]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[2]].nx );
      glVertex3fv ( &node[cpenta6[i].nod[5]].nx );
    glEnd();
   }
  }
  if(cullFlag) glDisable ( GL_CULL_FACE );
 }
 if (num_etype[3])
 {
  getCTet4Normalen( ctet4, node, &num_etype[3], norm );
  if(cullFlag) glEnable ( GL_CULL_FACE );
  glBegin ( GL_TRIANGLES );
  for (i=0; i<num_etype[3]; i++ )
  {
   if( (ctet4[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[ctet4[i].nod[ 0]].nx );
      glVertex3fv ( &node[ctet4[i].nod[ 1]].nx );
      glVertex3fv ( &node[ctet4[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[ctet4[i].nod[ 1]].nx );
      glVertex3fv ( &node[ctet4[i].nod[ 2]].nx );
      glVertex3fv ( &node[ctet4[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[ctet4[i].nod[ 2]].nx );
      glVertex3fv ( &node[ctet4[i].nod[ 0]].nx );
      glVertex3fv ( &node[ctet4[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[ctet4[i].nod[ 0]].nx );
      glVertex3fv ( &node[ctet4[i].nod[ 2]].nx );
      glVertex3fv ( &node[ctet4[i].nod[ 1]].nx );
   }
  }
  glEnd();
  if(cullFlag) glDisable ( GL_CULL_FACE );
 }
 if (num_etype[4])
 {
  getCHexa20Normalen( chexa20, node, &num_etype[4], norm );
  if(cullFlag) glEnable ( GL_CULL_FACE );
  glBegin ( GL_TRIANGLES );
  for (i=0; i<num_etype[4]; i++ )
  {
   if( (chexa20[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[chexa20[i].nod[ 1]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 9]].nx );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[chexa20[i].nod[ 9]].nx );
      glVertex3fv ( &node[chexa20[i].nod[21]].nx );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glVertex3fv ( &node[chexa20[i].nod[21]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 5]].nx );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[chexa20[i].nod[21]].nx );
      glVertex3fv ( &node[chexa20[i].nod[17]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 5]].nx );
      glNormal3fv ( norm[i].side[4] );
      glVertex3fv ( &node[chexa20[i].nod[ 5]].nx );
      glVertex3fv ( &node[chexa20[i].nod[17]].nx );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
      glNormal3fv ( norm[i].side[5] );
      glVertex3fv ( &node[chexa20[i].nod[17]].nx );
      glVertex3fv ( &node[chexa20[i].nod[25]].nx );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
      glNormal3fv ( norm[i].side[6] );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
      glVertex3fv ( &node[chexa20[i].nod[25]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 4]].nx );
      glNormal3fv ( norm[i].side[7] );
      glVertex3fv ( &node[chexa20[i].nod[25]].nx );
      glVertex3fv ( &node[chexa20[i].nod[19]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 4]].nx );
      glNormal3fv ( norm[i].side[8] );
      glVertex3fv ( &node[chexa20[i].nod[ 4]].nx );
      glVertex3fv ( &node[chexa20[i].nod[19]].nx );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glNormal3fv ( norm[i].side[9] );
      glVertex3fv ( &node[chexa20[i].nod[19]].nx );
      glVertex3fv ( &node[chexa20[i].nod[23]].nx );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glNormal3fv ( norm[i].side[10] );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glVertex3fv ( &node[chexa20[i].nod[23]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[11] );
      glVertex3fv ( &node[chexa20[i].nod[23]].nx );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[12] );
      glVertex3fv ( &node[chexa20[i].nod[ 9]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 2]].nx );
      glVertex3fv ( &node[chexa20[i].nod[21]].nx );
      glNormal3fv ( norm[i].side[13] );
      glVertex3fv ( &node[chexa20[i].nod[ 2]].nx );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glVertex3fv ( &node[chexa20[i].nod[21]].nx );
      glNormal3fv ( norm[i].side[14] );
      glVertex3fv ( &node[chexa20[i].nod[21]].nx );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glVertex3fv ( &node[chexa20[i].nod[17]].nx );
      glNormal3fv ( norm[i].side[15] );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 6]].nx );
      glVertex3fv ( &node[chexa20[i].nod[17]].nx );
      glNormal3fv ( norm[i].side[16] );
      glVertex3fv ( &node[chexa20[i].nod[17]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 6]].nx );
      glVertex3fv ( &node[chexa20[i].nod[25]].nx );
      glNormal3fv ( norm[i].side[17] );
      glVertex3fv ( &node[chexa20[i].nod[ 6]].nx );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
      glVertex3fv ( &node[chexa20[i].nod[25]].nx );
      glNormal3fv ( norm[i].side[18] );
      glVertex3fv ( &node[chexa20[i].nod[25]].nx );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
      glVertex3fv ( &node[chexa20[i].nod[19]].nx );
      glNormal3fv ( norm[i].side[19] );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 7]].nx );
      glVertex3fv ( &node[chexa20[i].nod[19]].nx );
      glNormal3fv ( norm[i].side[20] );
      glVertex3fv ( &node[chexa20[i].nod[19]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 7]].nx );
      glVertex3fv ( &node[chexa20[i].nod[23]].nx );
      glNormal3fv ( norm[i].side[21] );
      glVertex3fv ( &node[chexa20[i].nod[ 7]].nx );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glVertex3fv ( &node[chexa20[i].nod[23]].nx );
      glNormal3fv ( norm[i].side[22] );
      glVertex3fv ( &node[chexa20[i].nod[23]].nx );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
      glNormal3fv ( norm[i].side[23] );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 3]].nx );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
      glNormal3fv ( norm[i].side[24] );
      glVertex3fv ( &node[chexa20[i].nod[ 7]].nx );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glNormal3fv ( norm[i].side[25] );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
      glVertex3fv ( &node[chexa20[i].nod[22]].nx );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glNormal3fv ( norm[i].side[26] );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glVertex3fv ( &node[chexa20[i].nod[22]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[27] );
      glVertex3fv ( &node[chexa20[i].nod[22]].nx );
      glVertex3fv ( &node[chexa20[i].nod[10]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[28] );
      glVertex3fv ( &node[chexa20[i].nod[ 3]].nx );
      glVertex3fv ( &node[chexa20[i].nod[10]].nx );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
      glNormal3fv ( norm[i].side[29] );
      glVertex3fv ( &node[chexa20[i].nod[10]].nx );
      glVertex3fv ( &node[chexa20[i].nod[24]].nx );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
      glNormal3fv ( norm[i].side[30] );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
      glVertex3fv ( &node[chexa20[i].nod[24]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[31] );
      glVertex3fv ( &node[chexa20[i].nod[24]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 8]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[32] );
      glVertex3fv ( &node[chexa20[i].nod[ 0]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 8]].nx );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glNormal3fv ( norm[i].side[33] );
      glVertex3fv ( &node[chexa20[i].nod[ 8]].nx );
      glVertex3fv ( &node[chexa20[i].nod[20]].nx );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glNormal3fv ( norm[i].side[34] );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glVertex3fv ( &node[chexa20[i].nod[20]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 4]].nx );
      glNormal3fv ( norm[i].side[35] );
      glVertex3fv ( &node[chexa20[i].nod[20]].nx );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 4]].nx );
      glNormal3fv ( norm[i].side[36] );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 6]].nx );
      glVertex3fv ( &node[chexa20[i].nod[22]].nx );
      glNormal3fv ( norm[i].side[37] );
      glVertex3fv ( &node[chexa20[i].nod[ 6]].nx );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glVertex3fv ( &node[chexa20[i].nod[22]].nx );
      glNormal3fv ( norm[i].side[38] );
      glVertex3fv ( &node[chexa20[i].nod[22]].nx );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glVertex3fv ( &node[chexa20[i].nod[10]].nx );
      glNormal3fv ( norm[i].side[39] );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 2]].nx );
      glVertex3fv ( &node[chexa20[i].nod[10]].nx );
      glNormal3fv ( norm[i].side[40] );
      glVertex3fv ( &node[chexa20[i].nod[10]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 2]].nx );
      glVertex3fv ( &node[chexa20[i].nod[24]].nx );
      glNormal3fv ( norm[i].side[41] );
      glVertex3fv ( &node[chexa20[i].nod[ 2]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 9]].nx );
      glVertex3fv ( &node[chexa20[i].nod[24]].nx );
      glNormal3fv ( norm[i].side[42] );
      glVertex3fv ( &node[chexa20[i].nod[24]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 9]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 8]].nx );
      glNormal3fv ( norm[i].side[43] );
      glVertex3fv ( &node[chexa20[i].nod[ 9]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 1]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 8]].nx );
      glNormal3fv ( norm[i].side[44] );
      glVertex3fv ( &node[chexa20[i].nod[ 8]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 1]].nx );
      glVertex3fv ( &node[chexa20[i].nod[20]].nx );
      glNormal3fv ( norm[i].side[45] );
      glVertex3fv ( &node[chexa20[i].nod[ 1]].nx );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glVertex3fv ( &node[chexa20[i].nod[20]].nx );
      glNormal3fv ( norm[i].side[46] );
      glVertex3fv ( &node[chexa20[i].nod[20]].nx );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
      glNormal3fv ( norm[i].side[47] );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glVertex3fv ( &node[chexa20[i].nod[ 5]].nx );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
   }
  }
  glEnd();
  if(cullFlag) glDisable ( GL_CULL_FACE );
 }
 if (num_etype[5])
 {
  getCPenta20Normalen( cpenta20, node, &num_etype[5], norm );
  if(cullFlag) glEnable ( GL_CULL_FACE );
  glBegin ( GL_TRIANGLES );
  for (i=0; i<num_etype[5]; i++ )
  {
   if( (cpenta20[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[cpenta20[i].nod[ 1]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 6]].nx );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[cpenta20[i].nod[ 6]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[15]].nx );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[cpenta20[i].nod[ 6]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[15]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[15]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glNormal3fv ( norm[i].side[4] );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 4]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[15]].nx );
      glNormal3fv ( norm[i].side[5] );
      glVertex3fv ( &node[cpenta20[i].nod[15]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 4]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[12]].nx );
      glNormal3fv ( norm[i].side[6] );
      glVertex3fv ( &node[cpenta20[i].nod[15]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[12]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glNormal3fv ( norm[i].side[7] );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[12]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[8] );
      glVertex3fv ( &node[cpenta20[i].nod[ 2]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 7]].nx );
      glNormal3fv ( norm[i].side[9] );
      glVertex3fv ( &node[cpenta20[i].nod[ 7]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[16]].nx );
      glNormal3fv ( norm[i].side[10] );
      glVertex3fv ( &node[cpenta20[i].nod[ 7]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[16]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 1]].nx );
      glNormal3fv ( norm[i].side[11] );
      glVertex3fv ( &node[cpenta20[i].nod[ 1]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[16]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glNormal3fv ( norm[i].side[12] );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 5]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[16]].nx );
      glNormal3fv ( norm[i].side[13] );
      glVertex3fv ( &node[cpenta20[i].nod[16]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 5]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glNormal3fv ( norm[i].side[14] );
      glVertex3fv ( &node[cpenta20[i].nod[16]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glNormal3fv ( norm[i].side[15] );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 4]].nx );
      glNormal3fv ( norm[i].side[16] );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 8]].nx );
      glNormal3fv ( norm[i].side[17] );
      glVertex3fv ( &node[cpenta20[i].nod[ 8]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[17]].nx );
      glNormal3fv ( norm[i].side[18] );
      glVertex3fv ( &node[cpenta20[i].nod[ 8]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[17]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 2]].nx );
      glNormal3fv ( norm[i].side[19] );
      glVertex3fv ( &node[cpenta20[i].nod[ 2]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[17]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glNormal3fv ( norm[i].side[20] );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[17]].nx );
      glNormal3fv ( norm[i].side[21] );
      glVertex3fv ( &node[cpenta20[i].nod[17]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[14]].nx );
      glNormal3fv ( norm[i].side[22] );
      glVertex3fv ( &node[cpenta20[i].nod[17]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[14]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glNormal3fv ( norm[i].side[23] );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[14]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 5]].nx );
      glNormal3fv ( norm[i].side[24] );
      glVertex3fv ( &node[cpenta20[i].nod[ 2]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 7]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 8]].nx );
      glNormal3fv ( norm[i].side[25] );
      glVertex3fv ( &node[cpenta20[i].nod[ 8]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 7]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[18]].nx );
      glNormal3fv ( norm[i].side[26] );
      glVertex3fv ( &node[cpenta20[i].nod[ 8]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[18]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[27] );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[18]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[28] );
      glVertex3fv ( &node[cpenta20[i].nod[ 7]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 1]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[18]].nx );
      glNormal3fv ( norm[i].side[29] );
      glVertex3fv ( &node[cpenta20[i].nod[18]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 1]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 6]].nx );
      glNormal3fv ( norm[i].side[30] );
      glVertex3fv ( &node[cpenta20[i].nod[18]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 6]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[31] );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 6]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[32] );
      glVertex3fv ( &node[cpenta20[i].nod[ 4]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[12]].nx );
      glNormal3fv ( norm[i].side[33] );
      glVertex3fv ( &node[cpenta20[i].nod[12]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[19]].nx );
      glNormal3fv ( norm[i].side[34] );
      glVertex3fv ( &node[cpenta20[i].nod[12]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[19]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[35] );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[19]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[36] );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 5]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[19]].nx );
      glNormal3fv ( norm[i].side[37] );
      glVertex3fv ( &node[cpenta20[i].nod[19]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 5]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[14]].nx );
      glNormal3fv ( norm[i].side[38] );
      glVertex3fv ( &node[cpenta20[i].nod[19]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[14]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[39] );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[14]].nx );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
   }
  }
  glEnd();
  if(cullFlag) glDisable ( GL_CULL_FACE );
 }
 if (num_etype[6])
 {
  getCTet10Normalen( ctet10, node, &num_etype[6], norm );
  if(cullFlag) glEnable ( GL_CULL_FACE );
  glBegin ( GL_TRIANGLES );
  for (i=0; i<num_etype[6]; i++ )
  {
   if( (ctet10[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[ctet10[i].nod[ 0]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 4]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 7]].nx );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[ctet10[i].nod[ 7]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 4]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 8]].nx );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[ctet10[i].nod[ 8]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 4]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 1]].nx );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[ctet10[i].nod[ 7]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 8]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[4] );
      glVertex3fv ( &node[ctet10[i].nod[ 1]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 5]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 8]].nx );
      glNormal3fv ( norm[i].side[5] );
      glVertex3fv ( &node[ctet10[i].nod[ 8]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 5]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 9]].nx );
      glNormal3fv ( norm[i].side[6] );
      glVertex3fv ( &node[ctet10[i].nod[ 9]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 5]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 2]].nx );
      glNormal3fv ( norm[i].side[7] );
      glVertex3fv ( &node[ctet10[i].nod[ 8]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 9]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[8] );
      glVertex3fv ( &node[ctet10[i].nod[ 2]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 6]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 9]].nx );
      glNormal3fv ( norm[i].side[9] );
      glVertex3fv ( &node[ctet10[i].nod[ 9]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 6]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 7]].nx );
      glNormal3fv ( norm[i].side[10] );
      glVertex3fv ( &node[ctet10[i].nod[ 7]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 6]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 0]].nx );
      glNormal3fv ( norm[i].side[11] );
      glVertex3fv ( &node[ctet10[i].nod[ 9]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 7]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 3]].nx );
      glNormal3fv ( norm[i].side[12] );
      glVertex3fv ( &node[ctet10[i].nod[ 0]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 6]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 4]].nx );
      glNormal3fv ( norm[i].side[13] );
      glVertex3fv ( &node[ctet10[i].nod[ 4]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 6]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 5]].nx );
      glNormal3fv ( norm[i].side[14] );
      glVertex3fv ( &node[ctet10[i].nod[ 5]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 6]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 2]].nx );
      glNormal3fv ( norm[i].side[15] );
      glVertex3fv ( &node[ctet10[i].nod[ 4]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 5]].nx );
      glVertex3fv ( &node[ctet10[i].nod[ 1]].nx );
   }
  }
  glEnd();
  if(cullFlag) glDisable ( GL_CULL_FACE );
 }

 if (num_etype[7])
 {
  getCTri3Normalen( ctri3, node, &num_etype[7], norm );
  glBegin ( GL_TRIANGLES );
  for (i=0; i<num_etype[7]; i++ )
  {
   if( (ctri3[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[ctri3[i].nod[0]].nx );
      glVertex3fv ( &node[ctri3[i].nod[1]].nx );
      glVertex3fv ( &node[ctri3[i].nod[2]].nx );
   }
  }
  glEnd();
 }
  if (num_etype[8])
  {
   getCTri6Normalen( ctri6, node, &num_etype[8], norm );
   glBegin ( GL_TRIANGLES );
   for (i=0; i<num_etype[8]; i++ )
   {
    if( (ctri6[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[ctri6[i].nod[0]].nx );
      glVertex3fv ( &node[ctri6[i].nod[3]].nx );
      glVertex3fv ( &node[ctri6[i].nod[5]].nx );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[ctri6[i].nod[2]].nx );
      glVertex3fv ( &node[ctri6[i].nod[5]].nx );
      glVertex3fv ( &node[ctri6[i].nod[4]].nx );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[ctri6[i].nod[4]].nx );
      glVertex3fv ( &node[ctri6[i].nod[5]].nx );
      glVertex3fv ( &node[ctri6[i].nod[3]].nx );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[ctri6[i].nod[3]].nx );
      glVertex3fv ( &node[ctri6[i].nod[1]].nx );
      glVertex3fv ( &node[ctri6[i].nod[4]].nx );
    }
   }
   glEnd();
  }
 if (num_etype[9])
 {
  getCQuad4Normalen( cquad4, node , &num_etype[9], norm );
  for (i=0; i<num_etype[9]; i++ )
  {
   if( (cquad4[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[cquad4[i].nod[0]].nx );
      glVertex3fv ( &node[cquad4[i].nod[1]].nx );
      glVertex3fv ( &node[cquad4[i].nod[3]].nx );
      glVertex3fv ( &node[cquad4[i].nod[2]].nx );
    glEnd();
   }
  }
 }
  if (num_etype[10])
  {
   getCQuad8Normalen( cquad8, node, &num_etype[10], norm );
   for (i=0; i<num_etype[10]; i++ )
   {
    if( (cquad8[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[0] );
      glVertex3fv ( &node[cquad8[i].nod[0]].nx );
      glVertex3fv ( &node[cquad8[i].nod[4]].nx );
      glVertex3fv ( &node[cquad8[i].nod[7]].nx );
      glNormal3fv ( norm[i].side[1] );
      glVertex3fv ( &node[cquad8[i].nod[6]].nx );
      glNormal3fv ( norm[i].side[2] );
      glVertex3fv ( &node[cquad8[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glNormal3fv ( norm[i].side[3] );
      glVertex3fv ( &node[cquad8[i].nod[2]].nx );
      glVertex3fv ( &node[cquad8[i].nod[6]].nx );
      glVertex3fv ( &node[cquad8[i].nod[5]].nx );
      glNormal3fv ( norm[i].side[4] );
      glVertex3fv ( &node[cquad8[i].nod[4]].nx );
      glNormal3fv ( norm[i].side[5] );
      glVertex3fv ( &node[cquad8[i].nod[1]].nx );
    glEnd();
    }
   }
  }
 if (num_etype[11])
 {
  glBegin ( GL_LINES );
  for (i=0; i<num_etype[11]; i++ )
  {
   if( (cbeam[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
      glNormal3f  ( 1., 0., 0. );
      glVertex3fv ( &node[cbeam[i].nod[0]].nx );
      glVertex3fv ( &node[cbeam[i].nod[1]].nx );
   }
  }
  glEnd();
 }
 if (num_etype[12])
 {
  for (i=0; i<num_etype[12]; i++ )
  {
   if( (cbeam3[i].group == elemGroupNr) || (elemGroupNr == -1) )
   {
  glBegin ( GL_LINE_STRIP );
      glNormal3f  ( 1., 0., 0. );
      glVertex3fv ( &node[cbeam3[i].nod[0]].nx );
      glVertex3fv ( &node[cbeam3[i].nod[2]].nx );
      glVertex3fv ( &node[cbeam3[i].nod[1]].nx );
  glEnd();
   }
  }
 }
  glEndList();
}


