/* --------------------------------------------------------------------  */
/*                          CALCULIX                                     */
/*                   - GRAPHICAL INTERFACE -                             */
/*                                                                       */
/*     A 3-dimensional pre- and post-processor for finite elements       */
/*              Copyright (C) 1996 Klaus Wittig                          */
/*                                                                       */
/*     This program is free software; you can redistribute it and/or     */
/*     modify it under the terms of the GNU General Public License as    */
/*     published by the Free Software Foundation; version 2 of           */
/*     the License.                                                      */
/*                                                                       */
/*     This program is distributed in the hope that it will be useful,   */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of    */ 
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      */
/*     GNU General Public License for more details.                      */
/*                                                                       */
/*     You should have received a copy of the GNU General Public License */
/*     along with this program; if not, write to the Free Software       */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         */
/* --------------------------------------------------------------------  */

/*------------------------------------------------------------------------ */
/* Displaylisten fuer die farbdarstellung im rgb mode                    */
/*------------------------------------------------------------------------ */

#include <cgx.h>

#define     TEST            0     /* debugging */

/*------------------------------------------------------------------------ */
/* Displaylisten fuer die elemente im surface mode                         */
/*------------------------------------------------------------------------ */

void drawSurfaces_load_rgb( GLuint list_surf_load,  unsigned char cullFlag, int  elemGroupNr, Colours *col,
                     int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CTri3 *surf3,
                     CTri3 *ctri3, CTri6 *ctri6, CQuad4 *cquad4, CBeam *cbeam, CBeam3 *cbeam3, CHexa8 *cquad8 )
{
  register int i;

  glNewList( list_surf_load, GL_COMPILE );
  if (num_stype[9])
  {
   if(cullFlag) glEnable ( GL_CULL_FACE );
   for (i=0; i<num_stype[9]; i++ )
   {
    if(surf4[i].mat >-1) if( (surf4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[surf4[i].nod[0]].r );
      glVertex3fv ( &node[surf4[i].nod[0]].nx );
      glColor3fv    ( &col[surf4[i].nod[1]].r );
      glVertex3fv ( &node[surf4[i].nod[1]].nx );
      glColor3fv    ( &col[surf4[i].nod[3]].r );
      glVertex3fv ( &node[surf4[i].nod[3]].nx );
      glColor3fv    ( &col[surf4[i].nod[2]].r );
      glVertex3fv ( &node[surf4[i].nod[2]].nx );
    glEnd();
    }
   }
  }
  if (num_stype[7])
  {
   if(cullFlag) glEnable ( GL_CULL_FACE );
   glBegin ( GL_TRIANGLES );
   for (i=0; i<num_stype[7]; i++ )
   {
    if(surf3[i].mat >-1) if( (surf3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glColor3fv    ( &col[surf3[i].nod[0]].r );
      glVertex3fv ( &node[surf3[i].nod[0]].nx );
      glColor3fv    ( &col[surf3[i].nod[1]].r );
      glVertex3fv ( &node[surf3[i].nod[1]].nx );
      glColor3fv    ( &col[surf3[i].nod[2]].r );
      glVertex3fv ( &node[surf3[i].nod[2]].nx );
    }
   }
   glEnd();
  }
  if (num_etype[7])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   glBegin ( GL_TRIANGLES );
   for (i=0; i<num_etype[7]; i++ )
   {
    if( (ctri3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glColor3fv    ( &col[ctri3[i].nod[0]].r );
      glVertex3fv ( &node[ctri3[i].nod[0]].nx );
      glColor3fv    ( &col[ctri3[i].nod[1]].r );
      glVertex3fv ( &node[ctri3[i].nod[1]].nx );
      glColor3fv    ( &col[ctri3[i].nod[2]].r );
      glVertex3fv ( &node[ctri3[i].nod[2]].nx );
    }
   }
   glEnd();
  }
  if (num_etype[8])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   glBegin ( GL_TRIANGLES );
   for (i=0; i<num_etype[8]; i++ )
   {
    if( (ctri6[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glColor3fv    ( &col[ctri6[i].nod[0]].r );
      glVertex3fv  ( &node[ctri6[i].nod[0]].nx );
      glColor3fv    ( &col[ctri6[i].nod[3]].r );
      glVertex3fv  ( &node[ctri6[i].nod[3]].nx );
      glColor3fv    ( &col[ctri6[i].nod[5]].r );
      glVertex3fv  ( &node[ctri6[i].nod[5]].nx );

      glColor3fv    ( &col[ctri6[i].nod[2]].r );
      glVertex3fv  ( &node[ctri6[i].nod[2]].nx );
      glColor3fv    ( &col[ctri6[i].nod[5]].r );
      glVertex3fv  ( &node[ctri6[i].nod[5]].nx );
      glColor3fv    ( &col[ctri6[i].nod[4]].r );
      glVertex3fv  ( &node[ctri6[i].nod[4]].nx );

      glColor3fv    ( &col[ctri6[i].nod[4]].r );
      glVertex3fv  ( &node[ctri6[i].nod[4]].nx );
      glColor3fv    ( &col[ctri6[i].nod[5]].r );
      glVertex3fv  ( &node[ctri6[i].nod[5]].nx );
      glColor3fv    ( &col[ctri6[i].nod[3]].r );
      glVertex3fv  ( &node[ctri6[i].nod[3]].nx );

      glColor3fv    ( &col[ctri6[i].nod[3]].r );
      glVertex3fv  ( &node[ctri6[i].nod[3]].nx );
      glColor3fv    ( &col[ctri6[i].nod[1]].r );
      glVertex3fv  ( &node[ctri6[i].nod[1]].nx );
      glColor3fv    ( &col[ctri6[i].nod[4]].r );
      glVertex3fv  ( &node[ctri6[i].nod[4]].nx );
    }
   }
   glEnd();
  }
  if (num_etype[9])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   for (i=0; i<num_etype[9]; i++ )
   {
    if( (cquad4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cquad4[i].nod[0]].r );
      glVertex3fv ( &node[cquad4[i].nod[0]].nx );
      glColor3fv    ( &col[cquad4[i].nod[1]].r );
      glVertex3fv ( &node[cquad4[i].nod[1]].nx );
      glColor3fv    ( &col[cquad4[i].nod[3]].r );
      glVertex3fv ( &node[cquad4[i].nod[3]].nx );
      glColor3fv    ( &col[cquad4[i].nod[2]].r );
      glVertex3fv ( &node[cquad4[i].nod[2]].nx );
    glEnd();
    }
   }
  }
 if(num_etype[10])
 {
   if(cullFlag) glDisable ( GL_CULL_FACE );
  for (i=0; i<num_etype[10]; i++ )
   {
   if( (cquad8[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cquad8[i].nod[0]].r );
      glVertex3fv ( &node[cquad8[i].nod[0]].nx );
      glColor3fv    ( &col[cquad8[i].nod[4]].r );
      glVertex3fv ( &node[cquad8[i].nod[4]].nx );
      glColor3fv    ( &col[cquad8[i].nod[7]].r );
      glVertex3fv ( &node[cquad8[i].nod[7]].nx );
      glColor3fv    ( &col[cquad8[i].nod[6]].r );
      glVertex3fv ( &node[cquad8[i].nod[6]].nx );
      glColor3fv    ( &col[cquad8[i].nod[3]].r );
      glVertex3fv ( &node[cquad8[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cquad8[i].nod[2]].r );
      glVertex3fv ( &node[cquad8[i].nod[2]].nx );
      glColor3fv    ( &col[cquad8[i].nod[6]].r );
      glVertex3fv ( &node[cquad8[i].nod[6]].nx );
      glColor3fv    ( &col[cquad8[i].nod[5]].r );
      glVertex3fv ( &node[cquad8[i].nod[5]].nx );
      glColor3fv    ( &col[cquad8[i].nod[4]].r );
      glVertex3fv ( &node[cquad8[i].nod[4]].nx );
      glColor3fv    ( &col[cquad8[i].nod[1]].r );
      glVertex3fv ( &node[cquad8[i].nod[1]].nx );
    glEnd();
    }
   }
 }
  if (num_etype[11])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   glBegin ( GL_LINES );
   for (i=0; i<num_etype[11]; i++ )
   {
    if( (cbeam[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glColor3fv    ( &col[cbeam[i].nod[0]].r );
      glVertex3fv ( &node[cbeam[i].nod[0]].nx );
      glColor3fv    ( &col[cbeam[i].nod[1]].r );
      glVertex3fv ( &node[cbeam[i].nod[1]].nx );
    }
   }
   glEnd();
  }
  if (num_etype[12])
  {
   if(cullFlag) glDisable ( GL_CULL_FACE );
   for (i=0; i<num_etype[12]; i++ )
   {
    if( (cbeam3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
   glBegin ( GL_LINE_STRIP );
      glColor3fv    ( &col[cbeam3[i].nod[0]].r );
      glVertex3fv ( &node[cbeam3[i].nod[0]].nx );
      glColor3fv    ( &col[cbeam3[i].nod[2]].r );
      glVertex3fv ( &node[cbeam3[i].nod[2]].nx );
      glColor3fv    ( &col[cbeam3[i].nod[1]].r );
      glVertex3fv ( &node[cbeam3[i].nod[1]].nx );
   glEnd();
    }
   }
  }
  glEndList();
}


/*------------------------------------------------------------------------ */
/* Displaylisten fuer die elemente im solid mode                           */
/*------------------------------------------------------------------------ */

void drawElements_load_rgb( GLuint list_elem_load,  unsigned char cullFlag, int  elemGroupNr, Colours *col,
                     int *num_etype, Nodes *node, CHexa20 *chexa20, CHexa8 *chexa8,
			CTri3 *ctri3, CTri6 *ctri6, CQuad4 *cquad4, CBeam *cbeam, CBeam3 *cbeam3, CTet4 *ctet4, CTet10 *ctet10, CHexa8 *cquad8, CPenta6 *cpenta6, CPenta20 *cpenta20 )
{
  register int i;

  glNewList( list_elem_load, GL_COMPILE );
  if(cullFlag) glEnable ( GL_CULL_FACE );
 if(num_etype[1])
 {
  for (i=0; i<num_etype[1]; i++ )
   {
   if( (chexa8[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[chexa8[i].nod[1]].r );
      glVertex3fv ( &node[chexa8[i].nod[1]].nx );
      glColor3fv    ( &col[chexa8[i].nod[2]].r );
      glVertex3fv ( &node[chexa8[i].nod[2]].nx );
      glColor3fv    ( &col[chexa8[i].nod[5]].r );
      glVertex3fv ( &node[chexa8[i].nod[5]].nx );
      glColor3fv    ( &col[chexa8[i].nod[6]].r );
      glVertex3fv ( &node[chexa8[i].nod[6]].nx );
      glColor3fv    ( &col[chexa8[i].nod[4]].r );
      glVertex3fv ( &node[chexa8[i].nod[4]].nx );
      glColor3fv    ( &col[chexa8[i].nod[7]].r );
      glVertex3fv ( &node[chexa8[i].nod[7]].nx );
      glColor3fv    ( &col[chexa8[i].nod[0]].r );
      glVertex3fv ( &node[chexa8[i].nod[0]].nx );
      glColor3fv    ( &col[chexa8[i].nod[3]].r );
      glVertex3fv ( &node[chexa8[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[chexa8[i].nod[5]].r );
      glVertex3fv ( &node[chexa8[i].nod[5]].nx );
      glColor3fv    ( &col[chexa8[i].nod[4]].r );
      glVertex3fv ( &node[chexa8[i].nod[4]].nx );
      glColor3fv    ( &col[chexa8[i].nod[1]].r );
      glVertex3fv ( &node[chexa8[i].nod[1]].nx );
      glColor3fv    ( &col[chexa8[i].nod[0]].r );
      glVertex3fv ( &node[chexa8[i].nod[0]].nx );
      glColor3fv    ( &col[chexa8[i].nod[2]].r );
      glVertex3fv ( &node[chexa8[i].nod[2]].nx );
      glColor3fv    ( &col[chexa8[i].nod[3]].r );
      glVertex3fv ( &node[chexa8[i].nod[3]].nx );
      glColor3fv    ( &col[chexa8[i].nod[6]].r );
      glVertex3fv ( &node[chexa8[i].nod[6]].nx );
      glColor3fv    ( &col[chexa8[i].nod[7]].r );
      glVertex3fv ( &node[chexa8[i].nod[7]].nx );
    glEnd();
    }
   }
 }
 if(num_etype[2])
 {
  for (i=0; i<num_etype[2]; i++ )
   {
   if( (cpenta6[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cpenta6[i].nod[3]].r );
      glVertex3fv ( &node[cpenta6[i].nod[3]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[0]].r );
      glVertex3fv ( &node[cpenta6[i].nod[0]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[4]].r );
      glVertex3fv ( &node[cpenta6[i].nod[4]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[1]].r );
      glVertex3fv ( &node[cpenta6[i].nod[1]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[5]].r );
      glVertex3fv ( &node[cpenta6[i].nod[5]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[2]].r );
      glVertex3fv ( &node[cpenta6[i].nod[2]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[3]].r );
      glVertex3fv ( &node[cpenta6[i].nod[3]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[0]].r );
      glVertex3fv ( &node[cpenta6[i].nod[0]].nx );
    glEnd();
    glBegin ( GL_TRIANGLES      );
      glColor3fv    ( &col[cpenta6[i].nod[3]].r );
      glVertex3fv ( &node[cpenta6[i].nod[3]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[4]].r );
      glVertex3fv ( &node[cpenta6[i].nod[4]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[5]].r );
      glVertex3fv ( &node[cpenta6[i].nod[5]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[0]].r );
      glVertex3fv ( &node[cpenta6[i].nod[0]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[2]].r );
      glVertex3fv ( &node[cpenta6[i].nod[2]].nx );
      glColor3fv    ( &col[cpenta6[i].nod[1]].r );
      glVertex3fv ( &node[cpenta6[i].nod[1]].nx );
    glEnd();
    }
   }
 }
 if(num_etype[3])
 {
  for (i=0; i<num_etype[3]; i++ )
   {
   if( (ctet4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_FAN );
      glColor3fv    ( &col[ctet4[i].nod[0]].r );
      glVertex3fv ( &node[ctet4[i].nod[0]].nx );
      glColor3fv    ( &col[ctet4[i].nod[1]].r );
      glVertex3fv ( &node[ctet4[i].nod[1]].nx );
      glColor3fv    ( &col[ctet4[i].nod[3]].r );
      glVertex3fv ( &node[ctet4[i].nod[3]].nx );
      glColor3fv    ( &col[ctet4[i].nod[2]].r );
      glVertex3fv ( &node[ctet4[i].nod[2]].nx );
      glColor3fv    ( &col[ctet4[i].nod[1]].r );
      glVertex3fv ( &node[ctet4[i].nod[1]].nx );
    glEnd();
    glBegin ( GL_TRIANGLES );
      glColor3fv    ( &col[ctet4[i].nod[1]].r );
      glVertex3fv ( &node[ctet4[i].nod[1]].nx );
      glColor3fv    ( &col[ctet4[i].nod[2]].r );
      glVertex3fv ( &node[ctet4[i].nod[2]].nx );
      glColor3fv    ( &col[ctet4[i].nod[3]].r );
      glVertex3fv ( &node[ctet4[i].nod[3]].nx );
    glEnd();
    }
   }
 }
 if(num_etype[4])
 {
  for (i=0; i<num_etype[4]; i++ )
   {
   if( (chexa20[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[chexa20[i].nod[1]].r );
      glVertex3fv ( &node[chexa20[i].nod[1]].nx );
      glColor3fv    ( &col[chexa20[i].nod[9]].r );
      glVertex3fv ( &node[chexa20[i].nod[9]].nx );
      glColor3fv    ( &col[chexa20[i].nod[13]].r );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glColor3fv    ( &col[chexa20[i].nod[21]].r );
      glVertex3fv ( &node[chexa20[i].nod[21]].nx );
      glColor3fv    ( &col[chexa20[i].nod[5]].r );
      glVertex3fv ( &node[chexa20[i].nod[5]].nx );
      glColor3fv    ( &col[chexa20[i].nod[17]].r );
      glVertex3fv ( &node[chexa20[i].nod[17]].nx );
      glColor3fv    ( &col[chexa20[i].nod[16]].r );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
      glColor3fv    ( &col[chexa20[i].nod[25]].r );
      glVertex3fv ( &node[chexa20[i].nod[25]].nx );
      glColor3fv    ( &col[chexa20[i].nod[4]].r );
      glVertex3fv ( &node[chexa20[i].nod[4]].nx );
      glColor3fv    ( &col[chexa20[i].nod[19]].r );
      glVertex3fv ( &node[chexa20[i].nod[19]].nx );
      glColor3fv    ( &col[chexa20[i].nod[12]].r );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glColor3fv    ( &col[chexa20[i].nod[23]].r );
      glVertex3fv ( &node[chexa20[i].nod[23]].nx );
      glColor3fv    ( &col[chexa20[i].nod[0]].r );
      glVertex3fv ( &node[chexa20[i].nod[0]].nx );
      glColor3fv    ( &col[chexa20[i].nod[11]].r );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[chexa20[i].nod[9]].r );
      glVertex3fv ( &node[chexa20[i].nod[9]].nx );
      glColor3fv    ( &col[chexa20[i].nod[2]].r );
      glVertex3fv ( &node[chexa20[i].nod[2]].nx );
      glColor3fv    ( &col[chexa20[i].nod[21]].r );
      glVertex3fv ( &node[chexa20[i].nod[21]].nx );
      glColor3fv    ( &col[chexa20[i].nod[14]].r );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glColor3fv    ( &col[chexa20[i].nod[17]].r );
      glVertex3fv ( &node[chexa20[i].nod[17]].nx );
      glColor3fv    ( &col[chexa20[i].nod[6]].r );
      glVertex3fv ( &node[chexa20[i].nod[6]].nx );
      glColor3fv    ( &col[chexa20[i].nod[25]].r );
      glVertex3fv ( &node[chexa20[i].nod[25]].nx );
      glColor3fv    ( &col[chexa20[i].nod[18]].r );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
      glColor3fv    ( &col[chexa20[i].nod[19]].r );
      glVertex3fv ( &node[chexa20[i].nod[19]].nx );
      glColor3fv    ( &col[chexa20[i].nod[7]].r );
      glVertex3fv ( &node[chexa20[i].nod[7]].nx );
      glColor3fv    ( &col[chexa20[i].nod[23]].r );
      glVertex3fv ( &node[chexa20[i].nod[23]].nx );
      glColor3fv    ( &col[chexa20[i].nod[15]].r );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glColor3fv    ( &col[chexa20[i].nod[11]].r );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
      glColor3fv    ( &col[chexa20[i].nod[3]].r );
      glVertex3fv ( &node[chexa20[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[chexa20[i].nod[5]].r );
      glVertex3fv ( &node[chexa20[i].nod[5]].nx );
      glColor3fv    ( &col[chexa20[i].nod[16]].r );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
      glColor3fv    ( &col[chexa20[i].nod[13]].r );
      glVertex3fv ( &node[chexa20[i].nod[13]].nx );
      glColor3fv    ( &col[chexa20[i].nod[20]].r );
      glVertex3fv ( &node[chexa20[i].nod[20]].nx );
      glColor3fv    ( &col[chexa20[i].nod[1]].r );
      glVertex3fv ( &node[chexa20[i].nod[1]].nx );
      glColor3fv    ( &col[chexa20[i].nod[8]].r );
      glVertex3fv ( &node[chexa20[i].nod[8]].nx );
      glColor3fv    ( &col[chexa20[i].nod[9]].r );
      glVertex3fv ( &node[chexa20[i].nod[9]].nx );
      glColor3fv    ( &col[chexa20[i].nod[24]].r );
      glVertex3fv ( &node[chexa20[i].nod[24]].nx );
      glColor3fv    ( &col[chexa20[i].nod[2]].r );
      glVertex3fv ( &node[chexa20[i].nod[2]].nx );
      glColor3fv    ( &col[chexa20[i].nod[10]].r );
      glVertex3fv ( &node[chexa20[i].nod[10]].nx );
      glColor3fv    ( &col[chexa20[i].nod[14]].r );
      glVertex3fv ( &node[chexa20[i].nod[14]].nx );
      glColor3fv    ( &col[chexa20[i].nod[22]].r );
      glVertex3fv ( &node[chexa20[i].nod[22]].nx );
      glColor3fv    ( &col[chexa20[i].nod[6]].r );
      glVertex3fv ( &node[chexa20[i].nod[6]].nx );
      glColor3fv    ( &col[chexa20[i].nod[18]].r );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[chexa20[i].nod[16]].r );
      glVertex3fv ( &node[chexa20[i].nod[16]].nx );
      glColor3fv    ( &col[chexa20[i].nod[4]].r );
      glVertex3fv ( &node[chexa20[i].nod[4]].nx );
      glColor3fv    ( &col[chexa20[i].nod[20]].r );
      glVertex3fv ( &node[chexa20[i].nod[20]].nx );
      glColor3fv    ( &col[chexa20[i].nod[12]].r );
      glVertex3fv ( &node[chexa20[i].nod[12]].nx );
      glColor3fv    ( &col[chexa20[i].nod[8]].r );
      glVertex3fv ( &node[chexa20[i].nod[8]].nx );
      glColor3fv    ( &col[chexa20[i].nod[0]].r );
      glVertex3fv ( &node[chexa20[i].nod[0]].nx );
      glColor3fv    ( &col[chexa20[i].nod[24]].r );
      glVertex3fv ( &node[chexa20[i].nod[24]].nx );
      glColor3fv    ( &col[chexa20[i].nod[11]].r );
      glVertex3fv ( &node[chexa20[i].nod[11]].nx );
      glColor3fv    ( &col[chexa20[i].nod[10]].r );
      glVertex3fv ( &node[chexa20[i].nod[10]].nx );
      glColor3fv    ( &col[chexa20[i].nod[3]].r );
      glVertex3fv ( &node[chexa20[i].nod[3]].nx );
      glColor3fv    ( &col[chexa20[i].nod[22]].r );
      glVertex3fv ( &node[chexa20[i].nod[22]].nx );
      glColor3fv    ( &col[chexa20[i].nod[15]].r );
      glVertex3fv ( &node[chexa20[i].nod[15]].nx );
      glColor3fv    ( &col[chexa20[i].nod[18]].r );
      glVertex3fv ( &node[chexa20[i].nod[18]].nx );
      glColor3fv    ( &col[chexa20[i].nod[7]].r );
      glVertex3fv ( &node[chexa20[i].nod[7]].nx );
    glEnd();
    }
   }
 }
 if(num_etype[5])
 {
  for (i=0; i<num_etype[5]; i++ )
   {
   if( (cpenta20[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cpenta20[i].nod[1]].r );
      glVertex3fv ( &node[cpenta20[i].nod[1]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[10]].r );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 6]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 6]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[15]].r );
      glVertex3fv ( &node[cpenta20[i].nod[15]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 0]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 9]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 0]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 0]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 9]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[8]].r );
      glVertex3fv ( &node[cpenta20[i].nod[8]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[17]].r );
      glVertex3fv ( &node[cpenta20[i].nod[17]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 2]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 2]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[11]].r );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[7]].r );
      glVertex3fv ( &node[cpenta20[i].nod[7]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[16]].r );
      glVertex3fv ( &node[cpenta20[i].nod[16]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[1]].r );
      glVertex3fv ( &node[cpenta20[i].nod[1]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[10]].r );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cpenta20[i].nod[10]].r );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 4]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 4]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[15]].r );
      glVertex3fv ( &node[cpenta20[i].nod[15]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[12]].r );
      glVertex3fv ( &node[cpenta20[i].nod[12]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 9]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 9]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 3]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 3]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[17]].r );
      glVertex3fv ( &node[cpenta20[i].nod[17]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[14]].r );
      glVertex3fv ( &node[cpenta20[i].nod[14]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[11]].r );
      glVertex3fv ( &node[cpenta20[i].nod[11]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 5]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 5]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[16]].r );
      glVertex3fv ( &node[cpenta20[i].nod[16]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[13]].r );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[10]].r );
      glVertex3fv ( &node[cpenta20[i].nod[10]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 4]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 4]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cpenta20[i].nod[ 2]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 2]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 7]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 7]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[8]].r );
      glVertex3fv ( &node[cpenta20[i].nod[8]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[18]].r );
      glVertex3fv ( &node[cpenta20[i].nod[18]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[0]].r );
      glVertex3fv ( &node[cpenta20[i].nod[0]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cpenta20[i].nod[ 7]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 7]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 1]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 1]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[18]].r );
      glVertex3fv ( &node[cpenta20[i].nod[18]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 6]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 6]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[0]].r );
      glVertex3fv ( &node[cpenta20[i].nod[0]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cpenta20[i].nod[ 4]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 4]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[13]].r );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[12]].r );
      glVertex3fv ( &node[cpenta20[i].nod[12]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[19]].r );
      glVertex3fv ( &node[cpenta20[i].nod[19]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[3]].r );
      glVertex3fv ( &node[cpenta20[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cpenta20[i].nod[13]].r );
      glVertex3fv ( &node[cpenta20[i].nod[13]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[ 5]].r );
      glVertex3fv ( &node[cpenta20[i].nod[ 5]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[19]].r );
      glVertex3fv ( &node[cpenta20[i].nod[19]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[14]].r );
      glVertex3fv ( &node[cpenta20[i].nod[14]].nx );
      glColor3fv    ( &col[cpenta20[i].nod[3]].r );
      glVertex3fv ( &node[cpenta20[i].nod[3]].nx );
    glEnd();
    }
   }
 }
 if(num_etype[6])
 {
  for (i=0; i<num_etype[6]; i++ )
   {
   if( (ctet10[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[ctet10[i].nod[1]].r );
      glVertex3fv ( &node[ctet10[i].nod[1]].nx );
      glColor3fv    ( &col[ctet10[i].nod[8]].r );
      glVertex3fv ( &node[ctet10[i].nod[8]].nx );
      glColor3fv    ( &col[ctet10[i].nod[4]].r );
      glVertex3fv ( &node[ctet10[i].nod[4]].nx );
      glColor3fv    ( &col[ctet10[i].nod[7]].r );
      glVertex3fv ( &node[ctet10[i].nod[7]].nx );
      glColor3fv    ( &col[ctet10[i].nod[0]].r );
      glVertex3fv ( &node[ctet10[i].nod[0]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[ctet10[i].nod[2]].r );
      glVertex3fv ( &node[ctet10[i].nod[2]].nx );
      glColor3fv    ( &col[ctet10[i].nod[9]].r );
      glVertex3fv ( &node[ctet10[i].nod[9]].nx );
      glColor3fv    ( &col[ctet10[i].nod[5]].r );
      glVertex3fv ( &node[ctet10[i].nod[5]].nx );
      glColor3fv    ( &col[ctet10[i].nod[8]].r );
      glVertex3fv ( &node[ctet10[i].nod[8]].nx );
      glColor3fv    ( &col[ctet10[i].nod[1]].r );
      glVertex3fv ( &node[ctet10[i].nod[1]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[ctet10[i].nod[0]].r );
      glVertex3fv ( &node[ctet10[i].nod[0]].nx );
      glColor3fv    ( &col[ctet10[i].nod[7]].r );
      glVertex3fv ( &node[ctet10[i].nod[7]].nx );
      glColor3fv    ( &col[ctet10[i].nod[6]].r );
      glVertex3fv ( &node[ctet10[i].nod[6]].nx );
      glColor3fv    ( &col[ctet10[i].nod[9]].r );
      glVertex3fv ( &node[ctet10[i].nod[9]].nx );
      glColor3fv    ( &col[ctet10[i].nod[2]].r );
      glVertex3fv ( &node[ctet10[i].nod[2]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[ctet10[i].nod[2]].r );
      glVertex3fv ( &node[ctet10[i].nod[2]].nx );
      glColor3fv    ( &col[ctet10[i].nod[5]].r );
      glVertex3fv ( &node[ctet10[i].nod[5]].nx );
      glColor3fv    ( &col[ctet10[i].nod[6]].r );
      glVertex3fv ( &node[ctet10[i].nod[6]].nx );
      glColor3fv    ( &col[ctet10[i].nod[4]].r );
      glVertex3fv ( &node[ctet10[i].nod[4]].nx );
      glColor3fv    ( &col[ctet10[i].nod[0]].r );
      glVertex3fv ( &node[ctet10[i].nod[0]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_FAN   );
      glColor3fv    ( &col[ctet10[i].nod[3]].r );
      glVertex3fv ( &node[ctet10[i].nod[3]].nx );
      glColor3fv    ( &col[ctet10[i].nod[7]].r );
      glVertex3fv ( &node[ctet10[i].nod[7]].nx );
      glColor3fv    ( &col[ctet10[i].nod[8]].r );
      glVertex3fv ( &node[ctet10[i].nod[8]].nx );
      glColor3fv    ( &col[ctet10[i].nod[9]].r );
      glVertex3fv ( &node[ctet10[i].nod[9]].nx );
      glColor3fv    ( &col[ctet10[i].nod[7]].r );
      glVertex3fv ( &node[ctet10[i].nod[7]].nx );
    glEnd();
    glBegin ( GL_TRIANGLES   );
      glColor3fv    ( &col[ctet10[i].nod[1]].r );
      glVertex3fv ( &node[ctet10[i].nod[1]].nx );
      glColor3fv    ( &col[ctet10[i].nod[4]].r );
      glVertex3fv ( &node[ctet10[i].nod[4]].nx );
      glColor3fv    ( &col[ctet10[i].nod[5]].r );
      glVertex3fv ( &node[ctet10[i].nod[5]].nx );
    glEnd();
    }
   }
 }
 if(cullFlag) glDisable ( GL_CULL_FACE );
 if(num_etype[7])
 {
  glBegin ( GL_TRIANGLES );
  for (i=0; i<num_etype[7]; i++ )
   {
   if( (ctri3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glColor3fv    ( &col[ctri3[i].nod[0]].r );
      glVertex3fv ( &node[ctri3[i].nod[0]].nx );
      glColor3fv    ( &col[ctri3[i].nod[1]].r );
      glVertex3fv ( &node[ctri3[i].nod[1]].nx );
      glColor3fv    ( &col[ctri3[i].nod[2]].r );
      glVertex3fv ( &node[ctri3[i].nod[2]].nx );
    }
   }
  glEnd();
 }
 if (num_etype[8])
 {
   glBegin ( GL_TRIANGLES );
   for (i=0; i<num_etype[8]; i++ )
   {
    if( (ctri6[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glColor3fv    ( &col[ctri6[i].nod[0]].r );
      glVertex3fv  ( &node[ctri6[i].nod[0]].nx );
      glColor3fv    ( &col[ctri6[i].nod[3]].r );
      glVertex3fv  ( &node[ctri6[i].nod[3]].nx );
      glColor3fv    ( &col[ctri6[i].nod[5]].r );
      glVertex3fv  ( &node[ctri6[i].nod[5]].nx );

      glColor3fv    ( &col[ctri6[i].nod[2]].r );
      glVertex3fv  ( &node[ctri6[i].nod[2]].nx );
      glColor3fv    ( &col[ctri6[i].nod[5]].r );
      glVertex3fv  ( &node[ctri6[i].nod[5]].nx );
      glColor3fv    ( &col[ctri6[i].nod[4]].r );
      glVertex3fv  ( &node[ctri6[i].nod[4]].nx );

      glColor3fv    ( &col[ctri6[i].nod[4]].r );
      glVertex3fv  ( &node[ctri6[i].nod[4]].nx );
      glColor3fv    ( &col[ctri6[i].nod[5]].r );
      glVertex3fv  ( &node[ctri6[i].nod[5]].nx );
      glColor3fv    ( &col[ctri6[i].nod[3]].r );
      glVertex3fv  ( &node[ctri6[i].nod[3]].nx );

      glColor3fv    ( &col[ctri6[i].nod[3]].r );
      glVertex3fv  ( &node[ctri6[i].nod[3]].nx );
      glColor3fv    ( &col[ctri6[i].nod[1]].r );
      glVertex3fv  ( &node[ctri6[i].nod[1]].nx );
      glColor3fv    ( &col[ctri6[i].nod[4]].r );
      glVertex3fv  ( &node[ctri6[i].nod[4]].nx );
    }
   }
   glEnd();
 }
 if(num_etype[9])
 {
  for (i=0; i<num_etype[9]; i++ )
   {
   if( (cquad4[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cquad4[i].nod[0]].r );
      glVertex3fv ( &node[cquad4[i].nod[0]].nx );
      glColor3fv    ( &col[cquad4[i].nod[1]].r );
      glVertex3fv ( &node[cquad4[i].nod[1]].nx );
      glColor3fv    ( &col[cquad4[i].nod[3]].r );
      glVertex3fv ( &node[cquad4[i].nod[3]].nx );
      glColor3fv    ( &col[cquad4[i].nod[2]].r );
      glVertex3fv ( &node[cquad4[i].nod[2]].nx );
    glEnd();
    }
   }
 }
 if(num_etype[10])
 {
  for (i=0; i<num_etype[10]; i++ )
   {
   if( (cquad8[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cquad8[i].nod[0]].r );
      glVertex3fv ( &node[cquad8[i].nod[0]].nx );
      glColor3fv    ( &col[cquad8[i].nod[4]].r );
      glVertex3fv ( &node[cquad8[i].nod[4]].nx );
      glColor3fv    ( &col[cquad8[i].nod[7]].r );
      glVertex3fv ( &node[cquad8[i].nod[7]].nx );
      glColor3fv    ( &col[cquad8[i].nod[6]].r );
      glVertex3fv ( &node[cquad8[i].nod[6]].nx );
      glColor3fv    ( &col[cquad8[i].nod[3]].r );
      glVertex3fv ( &node[cquad8[i].nod[3]].nx );
    glEnd();
    glBegin ( GL_TRIANGLE_STRIP );
      glColor3fv    ( &col[cquad8[i].nod[2]].r );
      glVertex3fv ( &node[cquad8[i].nod[2]].nx );
      glColor3fv    ( &col[cquad8[i].nod[6]].r );
      glVertex3fv ( &node[cquad8[i].nod[6]].nx );
      glColor3fv    ( &col[cquad8[i].nod[5]].r );
      glVertex3fv ( &node[cquad8[i].nod[5]].nx );
      glColor3fv    ( &col[cquad8[i].nod[4]].r );
      glVertex3fv ( &node[cquad8[i].nod[4]].nx );
      glColor3fv    ( &col[cquad8[i].nod[1]].r );
      glVertex3fv ( &node[cquad8[i].nod[1]].nx );
    glEnd();
    }
   }
 }
 if(num_etype[11])
 {
  glBegin ( GL_LINES );
  for (i=0; i<num_etype[11]; i++ )
   {
   if( (cbeam[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
      glColor3fv    ( &col[cbeam[i].nod[0]].r );
      glVertex3fv ( &node[cbeam[i].nod[0]].nx );
      glColor3fv    ( &col[cbeam[i].nod[1]].r );
      glVertex3fv ( &node[cbeam[i].nod[1]].nx );
    }
   }
  glEnd();
 }
 if(num_etype[12])
 {
  for (i=0; i<num_etype[12]; i++ )
   {
   if( (cbeam3[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
  glBegin ( GL_LINE_STRIP );
      glColor3fv    ( &col[cbeam3[i].nod[0]].r );
      glVertex3fv ( &node[cbeam3[i].nod[0]].nx );
      glColor3fv    ( &col[cbeam3[i].nod[2]].r );
      glVertex3fv ( &node[cbeam3[i].nod[2]].nx );
      glColor3fv    ( &col[cbeam3[i].nod[1]].r );
      glVertex3fv ( &node[cbeam3[i].nod[1]].nx );
  glEnd();
    }
   }
 }
 glEndList();
}

void drawSurfaces_elstress_rgb( GLuint list_elem_elstress,  unsigned char cullFlag, int  elemGroupNr,
                     int *num_etype, Nodes *node, CHexa20 *chexa20, CHexa8 *chexa8,
                     CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam )
{
  register int i;

  glNewList( list_elem_elstress, GL_COMPILE );
  if(cullFlag) glDisable ( GL_CULL_FACE );
  for (i=0; i<num_etype[11]; i++ )
   {
   if( (cbeam[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINES );
      glColor3fv    (  &cbeam[i].ncol[0][0] );
      glVertex3fv ( &node[cbeam[i].nod[0]].nx );
      glColor3fv     ( &cbeam[i].ncol[1][0] );
      glVertex3fv ( &node[cbeam[i].nod[1]].nx );
    glEnd();
    }
   }
  glEndList();
}

void drawElements_elstress_rgb( GLuint list_elem_elstress,  unsigned char cullFlag, int  elemGroupNr,
                     int *num_etype, Nodes *node, CHexa20 *chexa20, CHexa8 *chexa8,
                     CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam )
{
  register int i;

  glNewList( list_elem_elstress, GL_COMPILE );
  if(cullFlag) glDisable ( GL_CULL_FACE );
  for (i=0; i<num_etype[11]; i++ )
   {
   if( (cbeam[i].group == elemGroupNr) || (elemGroupNr == -1) )
    {
    glBegin ( GL_LINES );
      glColor3fv    (  &cbeam[i].ncol[0][0] );
      glVertex3fv ( &node[cbeam[i].nod[0]].nx );
      glColor3fv     ( &cbeam[i].ncol[1][0] );
      glVertex3fv ( &node[cbeam[i].nod[1]].nx );
    glEnd();
    }
   }
  glEndList();
}



