#include <math.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glut.h>

#define     pi          3.14159265358979323846264338327950288
#define     MAX_INTEGER 2147483647
#define     MAX_FLOAT   1.e32

#define     MAX_LINE_LENGTH 136
#define     NODES       1000000
#define     ELEMENTS    1000000
#define     MAX_LOADCASES  100000
#define     MAX_NODES_PER_ELEMENT 26
#define     MAX_NAME_LENGTH 8
#define     MAX_SUM_ASCI    4100        /* = (MAX_NAME_LENGTH**2 * 128)/2 = 4096 */

/* intpol2.c, spline.c */
#define     PNTS  2000

/* from #include "f2c.h" */
#define abs(x) ((x) >= 0 ? (x) : -(x))
#define dabs(x) (double)abs(x)
#define smin(a,b) ((a) <= (b) ? (a) : (b))
#define smax(a,b) ((a) >= (b) ? (a) : (b))
#define dmin(a,b) (double)min(a,b)
#define dmax(a,b) (double)max(a,b)

typedef struct {
  unsigned char  model[MAX_LINE_LENGTH]; /* haeder model */
  int   n;         /* number of nodes */
  int   e;         /* number of elements  */
  int   l;         /* number of loadcases (Datasets) */
  int   etype[100];/* number of elements of a certain type */
  int   nmax;      /* maximum nodenumber */
  int   nmin;      /* minimum nodenumber */
  int   emax;      /* maximum elemnumber */
  int   emin;      /* minimum elemnumber */
} Summen;

typedef struct {
  int   nr;              /*   node-nr (node[node-nr].nx        */
  int   indx;            /*   node-index (node[node-index].nr) */
  unsigned char  pflag;  /*   1 if used for display purposes   */
                         /*  -1 if the node is deleted         */
                         /*   0 default                        */
  float nx;              /*   coordinates                      */
  float ny;
  float nz;
} Nodes;

typedef struct {
  int nr;                /* external element number */
  int elem_nr;           /* element-index of a certain type (CBeam etc.) */ 
  int type;              /* element type (1:CHexa8)                      */
  int group;
  int mat;
  int attr;              /* -1:unstructured mesh */
                         /*  0:default           */
                         /*  1:reduced integration */
                         /*  2:incompatible modes */
  int nod[MAX_NODES_PER_ELEMENT+1];
} Elements;

typedef struct {
  unsigned char  **compName;
  unsigned char  **icname;
  unsigned char  name[MAX_LINE_LENGTH];
  unsigned char  dataset_name[MAX_LINE_LENGTH];
  unsigned char  dataset_text[MAX_LINE_LENGTH];
  unsigned char  analysis_name[MAX_LINE_LENGTH];
  float value;
  int analysis_type;
  int step_number;
  int ncomps;
  int irtype;
  int *ictype;
  int *icind1;
  int *icind2;
  int *iexist;
  float **dat;        /* node related data */
  float ***edat;      /* element related data */
  float *max;         /* maximum datum */
  float *min;         /* minimum datum */
  int *nmax;          /* node with maximum datum */
  int *nmin;          /* node with minimum datum */
} Datasets;

typedef struct {
  int nr;
  int elem_nr;
  int type;
  int group;
  int mat;
  int nod[8];
} Faces;

typedef struct {
  int nod[2];
  float ncol[2][3]; /* color-index ncol[node][colour] */
  int elem_nr;
  int group;
  int mat;
} CBeam;

typedef struct {
  int nod[3];
  float ncol[3][3]; 
  int elem_nr;
  int group;
  int mat;
} CBeam3;

typedef struct {
  int nod[3];
  float ncol[3][3];
  int elem_nr;
  int group;
  int mat;
} CTri3;

typedef struct {
  int nod[6];
  float ncol[6][3];
  int elem_nr;
  int group;
  int mat;
} CTri6;

typedef struct {
  int nod[4];
  float ncol[4][3];
  int elem_nr;
  int group;
  int mat;
} CQuad4;

typedef struct {
  int nod[8];
  float ncol[8][3];
  int elem_nr;
  int group;
  int mat;
} CQuad8;

typedef struct {
  int nod[6];
  float ncol[6][3];
  int elem_nr;
  int group;
  int mat;
} CPenta6;

typedef struct {
  int nod[8];
  float ncol[8][3];
  int elem_nr;
  int group;
  int mat;
} CHexa8;

typedef struct {
  int nod[4];
  float ncol[4][3];
  int elem_nr;
  int group;
  int mat;
} CTet4;

typedef struct {
  int nod[10];
  float ncol[10][3];
  int elem_nr;
  int group;
  int mat;
} CTet10;

typedef struct {
  int nod[20];
  float ncol[20][3];
  int elem_nr;
  int group;
  int mat;
} CPenta20;

typedef struct {
  int nod[26];
  float ncol[26][3];
  int elem_nr;
  int group;
  int mat;
} CHexa20;

typedef struct {
  float side[48][3];                 /* side[Nr.][x|y|z] */
} Normalen;

typedef struct {
  int p1;
  int p2;
} Edges;

typedef struct {
  float w;                     /* scalierung in den Einheitswuerfel==max(xyzmax) */
  float x;                     /* Mittelpunktsversatz der unscalierten Vektoren */
  float y;
  float z;
  float vmax, vmin;                            /* max,min Werte der values */
  float bmin, bmax;                            /* max,min Werte der Scala  */
  float xmax, xmin, ymax, ymin, zmax, zmin;    /* max==|max|Abstand eines Punktes vom Zentrum */
  int   nmax, nmin;                            /* nodes der max/min-Werte  */
} Scale;


typedef struct {
  unsigned char name[MAX_LINE_LENGTH];
  unsigned char flag;                  /* if the set is open: 'o' else: 'c' */
  unsigned char type;                  /* ordered entities:1 (seq) or not: 0 (set) */ 
  int anz_n;
  int anz_e;
  int anz_f;
  int anz_p;
  int anz_l;
  int anz_c;
  int anz_s;
  int anz_b;
  int anz_nurl;
  int anz_nurs;
  int anz_se;
  int *node;
  int *elem;
  int *face;
  int *pnt;
  int *line;
  int *lcmb;
  int *surf;
  int *body;
  int *nurl;
  int *nurs;
  int *set;
  int etyp;
  int eattr;       /* -1:unstructured mesh, 0:default, 1:reduced integration, 2:incompatible modes */
} Sets;

typedef struct {
  int nr;
  int *set;
} OpenSets;


/* compiles the actual displayed sets */
typedef struct {
  int nr;
  unsigned char type[MAX_NAME_LENGTH];
  int col;
} Psets;

/* sum of unsigned char of asci letters */
typedef struct {
  int  anza[MAX_SUM_ASCI];         /* nr of aliases stored */
  int  *aindx[MAX_SUM_ASCI];       /* alias indexes       */
  int  anzp[MAX_SUM_ASCI];         /* nr of points stored */
  int  *pindx[MAX_SUM_ASCI];       /* point indexes       */
  int  anzl[MAX_SUM_ASCI];         /* nr of lines stored  */
  int  *lindx[MAX_SUM_ASCI];       /* line indexes        */
  int  anzc[MAX_SUM_ASCI];         /* nr of lcmbs stored  */
  int  *cindx[MAX_SUM_ASCI];       /* lcmb indexes        */
  int  anzs[MAX_SUM_ASCI];         /* nr of surfs stored  */
  int  *sindx[MAX_SUM_ASCI];       /* surf indexes        */
  int  anzb[MAX_SUM_ASCI];         /* nr of bodys stored  */
  int  *bindx[MAX_SUM_ASCI];       /* body indexes        */
  int  anzS[MAX_SUM_ASCI];         /* nr of Nurs stored  */
  int  *Sindx[MAX_SUM_ASCI];       /* nurs indexes        */
  int  anzse[MAX_SUM_ASCI];        /* nr of sets stored  */
  int  *seindx[MAX_SUM_ASCI];      /* set indexes        */
} SumAsci;

typedef struct {
  unsigned char  model[MAX_LINE_LENGTH]; /* haeder model */
  int   alias;                           /* alias-names     */
  int   p;                               /* points */
  int   l;                               /* lines  */
  int   c;                               /* lcmbs  */
  int   s;                               /* surfs  */
  int   b;                               /* bodys  */
  int   nurl;                            /* nurbs lines     */
  int   nurs;                            /* nurbs surfaces  */
  int   sets;   
  int   psets;   
} SumGeo;

typedef struct{
  unsigned char   name[MAX_LINE_LENGTH];
  unsigned char   entitieName[MAX_LINE_LENGTH];
} Alias;


typedef struct {
  unsigned char  name[MAX_NAME_LENGTH];
  float px;     /* coordinates will be scaled during run-time, scalPoints() */
  float py;
  float pz;
  int   nn;
  int   *nod;
} Points;

typedef struct {
  unsigned char  name[MAX_NAME_LENGTH];
  unsigned char  typ;      /* straight:' ', arc:'a', spline:'s', nurbs:'n' */
  int   p1;
  int   p2;
  int   trk;      /* if arc: 3.pnt, if spline: setname, if nurbs: nurblname */
  int   div;
  float bias;
  int   nip;
  float *ip;
  int   nn;
  int   *nod;
  int   ne;
  int   *elem;
  int  etyp;
  int  eattr;       /* -1:unstructured mesh, 0:default, 1:reduced integration, 2:incompatible modes */
} Lines;

typedef struct {
  unsigned char  name[MAX_NAME_LENGTH];
  unsigned char  nl;
  unsigned char  *o;      /* l-orient +- */
  int   *l;               /* lines */
  int   p1;               /* starting point */
  int   p2;               /* end point */
  float cx;               /* CG, not used so far */
  float cy;
  float cz;
} Lcmb;

typedef struct {
  unsigned char  name[MAX_NAME_LENGTH];
  unsigned char  ori;     /* surface-orientation +- */
  int   sh;               /* embedded shape (nurs) */
  unsigned char   sho;    /* embedded shape (nurs) orientation  */
  int   nl;               /* corners (ether line or lcmb) */
  unsigned char  *typ;    /* type: l=line c=lcmb */
  unsigned char  *o;      /* l-orient +- */
  int   *l;               /* index of lines, lcmbs, also used to store the index of a substitute surf (last item) */
  int   nc;               /* number of closed-line-loops (curves) */
  int   *c;               /* number of lines in each curve */
  float cx;               /* CG, in orient determined, not scaled during run! */
  float cy;
  float cz;
  int   nn;
  int   *nod;
  int   ne;
  int   *elem;
  int   etyp;
  int   eattr;       /* -1:unstructured mesh, 0:default, 1:reduced integration, 2:incompatible modes */
  int   patch;            /* number of trimming patch in the related nurbs */
  GLint   npgn;           /* size of feed-back-Buffer pgn */
  GLfloat *pgn;           /* stores poligons of the trimmed nurbs which defines the interiour */
} Gsur;

typedef struct {
  unsigned char  name[MAX_NAME_LENGTH];
  unsigned char  ori;     /* body-orientation +- */
  int   ns;               /* nr. of surfaces (currently must be 6 for meshing) */
  unsigned char  *o;      /* s-orient +- */
  int   *s;               /* surfaces */
  float cx;               /* CG, in orient determined, not scaled during run! */
  float cy;
  float cz;
  int   nn;
  int   *nod;
  int   ne;
  int   *elem;
  int   etyp;
  int  eattr;       /* -1:unstructured mesh, 0:default, 1:reduced integration, 2:incompatible modes */
} Gbod;

typedef struct {
  GLUnurbsObj *Nurb;
  unsigned char  name[MAX_NAME_LENGTH];
  GLint   u_npnt;
  GLint   u_exp;
  GLint   u_nknt;
  GLfloat *uknt;
  GLint   u_stride;
  GLfloat *ctlarray;
  GLfloat *weight;
  GLenum  type;
  GLint   *ctlpnt;
  unsigned char  endFlag;
} Nurbl;

typedef struct {
  GLUnurbsObj *Nurb;
  unsigned char  name[MAX_NAME_LENGTH];
  GLint   u_npnt;
  GLint   u_exp;
  GLint   u_nknt;
  GLint   v_npnt;
  GLint   v_exp;
  GLint   v_nknt;
  GLfloat *uknt;
  GLfloat *vknt;
  GLint   u_stride;
  GLint   v_stride;
  GLfloat *ctlarray;
  GLenum  type;
  GLint   **ctlpnt;
  GLfloat **weight;
  unsigned char  endFlag;

  /* additional values for rendering purposes */
  int ures, vres;    /* division per u or v */
  int patches;        /* number of trimming patches (separate surfaces)*/
  int *nc;             /* number of trimming curves */
  int **np;            /* number of points in each trimming curve */
  float ***uv;         /* u,v coordinates of the spline-points of the trimming curves */
  float ***xyz;        /* x,y,z coordinates of the spline-points of the trimming curves */
  float tx, ty, tz;   /* average position for the name-string */
  float *umax, *vmax;   /* max val for u,v, for rendering purposes */
} Nurbs;



typedef struct {
  float alpha, beta;
  int nadapt;
} Meshp;





int renumberfrd( int firstelem, int firstnode, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr  );

void define_rgb(float v, float *r, float *g, float *b);
void text(float x, float y, float z,unsigned char *msg);
void scala( float dx, float dy, int divisions, float bmin, float bmax, float *col);
void scala_rgb( float dx, float dy, int divisions, float bmin, float bmax, float *col);
void scala_indx( float dx, float dy, int divisions, float bmin, float bmax, int offset, int col);
int button(float dx, float dy, unsigned char *msg, float mx, float my);
void polymark ( int n, float *col_r, float *col_g, float *col_b, float *x,
		float *y, float *z );

float nullstelle(float xmin, float xmax, float funktion(float), float *result);
int  calcPrinc( float *s, float *p, float *a1, float *a2, float *a3, int sortFlag );
int calcPvector( double *s, double *p, double *a );
void stopClock(int zaeler);
void bsort (float *wert, float *wertsort, int *isort, int anzahl);
int *bsort2(float *wert, int anzahl, int n );
int *bsortf(float *wert, int anzahl, int n );
int *bsortfp(float *wert, int anzahl, int n );
int *bsorti( int *wert, int anzahl, int n );
int compare (unsigned char *str1, unsigned char *str2, int length);
int compareStrings (unsigned char *str1, unsigned char *str2);
int elemChecker(Summen *anz, Nodes *node, Elements *elem);
int frecord( FILE *handle1,  unsigned char *string);
void  getGeoDataTria( float *p1, float *p2, float *p3, float *Ix, float *Iy, float *Ixy,
                float *A, float *spx, float *spy);
int getrecord (int bufstart, int bufsize, unsigned char *inputdata, unsigned char *buffer);
int gl3grades( double a, double b, double c, double *x);
int AsplitA( float *pa1, float *pa2, float *pa3, float *pb1, float *pb2, float *pb3, float *ps1, float *ps2);
float AsplitL( float *b, float *eu, float *ev, float *eg );
float interpol(float *x, float *y, int nn, float x0);
float intpol2(float *x, float *y, int n, float x0, int *method );
float intpol3(float *x, float *y, int n, float x0, int *method );
float intpol(float *x, float *y, int n, float x0 );
int nearNodes(float *po, int *n, float *p, int *node, int *k, int *flag);
int nurbl2seq(int nurlNr, Nurbl *nurbl, Points *point);
int iinsert(int **ipnt, int n, int x0 );
int ifind(int **ipnt, int n, int x0 );
int iremove(int **ipnt, int n, int x0 );
void linelength(float *x, float *y, float *z, int n, float *s );
int kbrecord( unsigned char *string);
int p3_finder(int anzahl_nodes, float *nx, float *nz, int *p);
int parser( unsigned char gkey, unsigned char *record );
int readfrd(unsigned char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
int read2frd(unsigned char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
int readFElt(unsigned char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
int readDuns(unsigned char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr, int elem_type );
int readGiff( unsigned char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
int readStl( unsigned char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
int rotiere(float *x, float *y, int n, float phi, float *xneu, float *yneu);
int srecord( unsigned char *handle1,  unsigned char *string);
float stof(unsigned char *string, int a, int b);
int stoi(unsigned char *string, int a, int b);
void stos(unsigned char *string, int a, int b, unsigned char *puffer);
int sins(unsigned char *string, int a, int b, unsigned char *puffer);
int sword( unsigned char *string, unsigned char *word);
float p_angle(float x, float y);
void v_add( float *A, float *B, float *C );
float v_betrag(float *a);
void  v_matmult(float *v, float *m);
float v_norm( float *A, float *C );
void v_prod( float *A, float *B, float *C );
void v_result( float *A, float *B, float *C );
int v_rot(float fi, float *p0, float *v, float *pin, float *pout);
void v_scal( float *A, float *B, float *C );
float v_sprod( float *a, float *b);
float v_angle( float *v0, float *v1 );
float v_distA( float *N0, float *N1, float *N2, float *N, float *vray);
int  v_rec2cyl( float *pr, int axis, int *csys, float *pc );
int v_sgg( float *p1, float *p2, float *e1s, float *e2s, float *ps);

void vl_add( double *A, double *B, double *C );
double vl_betrag(double *a);
void  vl_matmult(double *v, double *m);
double vl_norm( double *A, double *C );
void vl_prod( double *A, double *B, double *C );
void vl_result( double *A, double *B, double *C );
void vl_scal( double *A, double *B, double *C );
double vl_sprod( double *a, double *b);
double vl_angle( double *v0, double *v1, double *v2, double *scg, double *bcg );
void  m_copy(float *s, float *m);
void  m_print( float *m);

int m_prod(int *n,float *a,float *b,float *c);
int m_prodtr(int *n,float *a,float *b,float *c);

void  m_sub(float *ms, float *m, float *s);
int write2frd(unsigned char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase );
int write2nas(unsigned char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase );
int write2aba(unsigned char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase );
int write2ansys(unsigned char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase );
int write2aster(unsigned char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase );
void calcNormale_quad4(int i, int n1, int n2, int n3, int n4, int f, Nodes *node, Normalen *norm);
void calcNormale_tri3(int i, int n1, int n2, int n3, int f, Nodes *node, Normalen *norm );
void calcOneNorm_quad4(int n1, int n2, int n3, int n4, Nodes *node, float *norm);
void calcOneNorm_tri3(int n1, int n2, int n3, Nodes *node, float *norm );



void readStdCmap( Display **ptr_dpy, int *ptr_dpycells, Colormap *ptr_cmap, XColor **ptr_c,
                 unsigned long **ptr_pix, unsigned int *ptr_npixels, int anzCells );

int selectDisplayFacesHe8 (CHexa8 *elems, int numElems, int **pfaces, int *);
int selectDisplayFacesHe20 (CHexa20 *elems, int numElems, int **pfaces, int *);
int selectDisplayFacesTet4 (CTet4 *elems, int numElems, int **ptr, int *edges);
int selectDisplayFacesTet10 (CTet10 *elems, int numElems, int **ptr, int *edges);
int selectDisplayFacesPe6 (CPenta6 *elems, int numElems, int **ptr, int *edges);
int selectDisplayFacesPe20 (CPenta20 *elems, int numElems, int **ptr, int *edges);
int commonEdge3 (int *a, int *b);
int findCTri3Edges(CTri3 *elems, int numElems, int **edges);
float spline_int( int nc, float xneu, float **c );
void createSpline( int n, float **c);
int read2fbd( unsigned char *datin, SumGeo *anz, Points **pntptr, Lines **lineptr, Lcmb **lcmbptr, Gsur **surptr, Gbod **bodptr, Nurbs **nursptr, Sets *set );

int attachhe20_(double *co, int *nk, int *neigh, int *node, double *xi_loc__,double *et_loc__);
int attachhe8_(double *co, int *nk, int *neigh, int *node,double *xi_loc__,double *et_loc__);
int distattachhe20_(double *xi,double *et,double *pneigh,double *pnode,double *dist,double *p);
int distattachhe8_(double *xi,double * et, double *pneigh, double *pnode, double *dist, double *p);
int shapefhe20_(double *xi, double *et, double *coef);
int shapefhe8_(double *xi, double *et, double *coef);

int mesh2d(int *_nt, int _nb, int *npc, double **_pnt_u, double **_pnt_v, int **_pnt_flag, int **_tri3, float _alpha, float _beta, int _nadapt);






















