/* --------------------------------------------------------------------  */
/*                          CALCULIX                                     */
/*                   - GRAPHICAL INTERFACE -                             */
/*                                                                       */
/*     A 3-dimensional pre- and post-processor for finite elements       */
/*              Copyright (C) 1996 Klaus Wittig                          */
/*                                                                       */
/*     This program is free software; you can redistribute it and/or     */
/*     modify it under the terms of the GNU General Public License as    */
/*     published by the Free Software Foundation; either version 2 of    */
/*     the License, or (at your option) any later version.               */
/*                                                                       */
/*     This program is distributed in the hope that it will be useful,   */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of    */ 
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      */
/*     GNU General Public License for more details.                      */
/*                                                                       */
/*     You should have received a copy of the GNU General Public License */
/*     along with this program; if not, write to the Free Software       */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         */
/* --------------------------------------------------------------------  */

/* ALL function prototypes for the program "frdviewer" */

#include "mesher.h"


/* 
choose "RGB_MODE        0" if you are working on a sgi workstation or if you are working 
on a linux system with 8-Bit color depth. In this case rendering is done in the index-mode
This gives best results in the moment.
Otherwise stay with "RGB_MODE        1"
*/
#define     RGB_MODE        1     /* "1" for compilation in the rgb mode */



#define     MINVALUE    -1.e9
#define     MAXVALUE     1.e9
#define     MILLISECONDS_PER_PERIOD 1200
#define     MAX_ANIM_STEPS  100
#define     INI_SCREEN      600
#define     INI_MENU_WIDTH  150
#define     INI_MENU_HEIGHT  60
#define     GAMMA            0.5     /* gamma lightens or darkens an image (Hcpy) */
#define     Z_DEPTH          2.
#define     SET_COLS    7
#define     PICK        1
#define     PICK_LENGTH 5
#define     DEF_LINE_DIV 2
#define     MAX_LINE_DIV 100
#define     NURS_TOLERANCE 30
#define     MAX_PARAM_PER_RECORD 20

#define     GLUT_MENU_POINT_HEIGHT  18  /* seens to be allways that value */
/*
#define     GLUT_FONT      GLUT_BITMAP_8_BY_13
#define     GLUT_FONT      GLUT_BITMAP_9_BY_15
#define     GLUT_FONT      GLUT_BITMAP_TIMES_ROMAN_10
#define     GLUT_FONT      GLUT_BITMAP_HELVETICA_12
#define     GLUT_FONT      GLUT_BITMAP_HELVETICA_18
*/
#define     GLUT_FONT      GLUT_BITMAP_9_BY_15

/* fuer frdviewer.c */
#define     GTOL        0.01          /* geometric tolerance for merging and radial moving */

/* fuer extFunktions.c  */
#define  CMAP_CELLS_LIGHT  34             /* Anzahl der Grautoene in der Colormap */
#define  CMAP_DARKSHIFT     5             /* schwarzverschiebung der hellsten collour-cell */

#define  MAX_LINES_PER_LCMB 14            /* von FAM vorgegeben, in pre_lcmb sind nur 14 zZ. moeglich */
#define  MAX_SURFS_PER_BODY   6
#define  MAX_CORNERS_PER_SURF 4
#define  MAX_CORNERS_PER_BODY 8
#define  SURFS_PER_BODY   40
#define  CORNERS_PER_SURF 40
#define  CORNERS_PER_BODY 40
#define  LINES_PER_CORNER  100
#define     MAX_NOKEY      10000


typedef struct {
  float r, g, b;
} Colours;


/* XFunktions_mesa.h */
void calcOffset(void);
void getColormap(void);
void storeColors(int anz_col, int pix );

/* copyEntity.h */
int createCenterpnt( unsigned char *pkt1, unsigned char *pkt2, unsigned char *pkt3 );
int copy_set( unsigned char *targetnam, unsigned char *trans, int setNr, int *dep_p, int *dep_l, int *dep_c, int *dep_s, int *dep_se);
void pre_swep( unsigned char *record );
void pre_copy( unsigned char *record );
int  body_( unsigned char *name, unsigned char *corner );

/* dataGeo.h */
void descalPoints ( int anz_p, Points *point, Scale *scale );
void scalPoints ( int anz_p,  Points *point, Scale *scale );
void getScaleValues( int anz_n, int anz_p, Points *point, Nodes *node, Scale *scale);

/* dataMesh */
void descalNodes ( int anz_n, Nodes *node, Scale *scale );
void scalNodes ( int anz_n, Nodes *node, Scale *scale );
void calcLoadcases( int num_olc, Summen *anz, Nodes *node, Elements *elem,
		    Loadcases *lcase );
void calcAnimation( GLuint list_animate_light, GLuint *list_animate, int anim_steps, float anim_faktor,
                    int *anim_alfa, int halfperiode, int centerNode, Nodes *n_anim, int  elemGroupNr, Normalen *norm,
                    int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CTri3 *surf3,
                    CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, int lc,
                    Summen *anz, Scale *scale, Loadcases *lcase,  CHexa26 *chexa26, CHexa8 *cquad8 );
void calcSequence( GLuint list_animate_light, GLuint *list_animate, int *seqLC, int seq_steps, float anim_faktor, int halfperiode, int centerNode, Nodes *n_anim, int  elemGroupNr, Normalen *norm, int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CTri3 *surf3, CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, int lc, Summen *anz, Scale *scale, Loadcases *lcase,  CHexa26 *chexa26, CHexa8 *cquad8 );

/* defineEntity.h */
void operateAlias( unsigned char *name, unsigned char *type );
void del1Nod( int number ); 
void delNod( int anzn, int *number );
void nod( Summen *anz, Nodes **nptr, int flag, int nodnr, float x, float y, float z, int scalFlag );
void pre_nod( unsigned char *record);
void elem_CBeam( int elnr, int n1, int n2, int lag );
void elem_CTri3( int elnr, int n1, int n2, int n3, int lag );
void elem_CQuad4( int elnr, int n1, int n2, int n3, int n4, int lag );
void elem_CQuad8( int elnr, int n1, int n2, int n3, int n4, int n5, int n6, int n7, int n8, int lag );
void elem_CHexa8( int elnr, int n1, int n2, int n3, int n4, int n5, int n6, int n7, int n8, int lag  );
void elem_CHexa20( int elnr, int n1, int n2, int n3, int n4, int n5, int n6, int n7, int n8,
 int n9, int n10, int n11, int n12, int n13, int n14, int n15, int n16,
		   int n17, int n18, int n19, int n20, int setFlag );
void elem_CHexa26( int elnr, int n1, int n2, int n3, int n4, int n5, int n6, int n7, int n8,
 int n9, int n10, int n11, int n12, int n13, int n14, int n15, int n16,
 int n17, int n18, int n19, int n20, int n21, int n22, int n23, int n24, int n25, int n26, int lag );
int hashPoint( SumAsci *sumAsci, unsigned char *name, int nr);
int hashLine( SumAsci *sumAsci, unsigned char *name, int nr);
int hashLcmb( SumAsci *sumAsci, unsigned char *name, int nr);
int hashSurf( SumAsci *sumAsci, unsigned char *name, int nr);
int hashBody( SumAsci *sumAsci, unsigned char *name, int nr);
void pre_elem( unsigned char *record);
void bak_delPnt( int anz, int *number );
void delPnt( int anz, int *number );
void pnt( unsigned char *name, float x, float y, float z, int scalFlag );
void pre_pnt( unsigned char *record);
void delLine( int anz, int *number );
void line_( unsigned char *name, unsigned char *p1, unsigned char *p2, unsigned char *trk, int div );
void pre_line( unsigned char *record);
void delLcmb( int anz, int *number );
int lcmb_( unsigned char *name, int add, int anz_l, unsigned char *ori, unsigned char *lin );
void pre_lcmb( unsigned char *record);
void bak_delSurf( int anz, int *number );
void delSurf( int anz, int *number );
int  surface( unsigned char *name, unsigned char *ori, unsigned char *blend, int anz_c, unsigned char *cori, unsigned char *corner );
void pre_gsur( unsigned char *record);
void pre_surf( unsigned char *record);
void bak_delBody( int anz, int *number );
void delBody( int anz, int *number );
int  gbod( unsigned char *name, unsigned char *blend, int anz_c, unsigned char *cori, unsigned char *corner );
void pre_gbod( unsigned char *record);
void pre_body( unsigned char *record);
void nurs( unsigned char *string );

/* dispLists.h */
void getCTri3Normalen( CTri3 *ctri3, Nodes *node, int *num_etype, Normalen *norm );
void getCQad4Normalen( CQuad4 *cquad4, Nodes *node, int *num_etype, Normalen *norm );
void getCQuad8Normalen( CHexa8 *cquad8, Nodes *node, int *num_etype,  Normalen *norm);
void getCHexa8Normalen( CHexa8 *chexa8, Nodes *node, int *num_etype, Normalen *norm );
void getCTet4Normalen( CTet4 *ctet4, Nodes *node, int *num_etype,  Normalen *norm);
void getCTet10Normalen( CTet10 *ctet10, Nodes *node, int *num_etype,  Normalen *norm);
void getCHexa26Normalen( CHexa26 *chexa26, Nodes *node, int *num_etype,  Normalen *norm);
void drawModelEdges( GLuint list_model_edges, int color, float width,
                     int numEdges, Nodes *node, Edges *edge );
void drawSurfEdges( GLuint list_surf_edges, int color, float width, int  elemGroupNr,
                     int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CHexa8 *surf8,
		    CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CTri6 *surf6, CHexa8 *cquad8 );
void drawElemEdges( GLuint list_elem_edges, int color, float width, int  elemGroupNr,
                     int *num_etype, Nodes *node, CHexa26 *chexa26, CHexa8 *chexa8,
		    CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CTet4 *ctet4, CTet10 *ctet10, CHexa8 *cquad8 );
void drawSurfaces_light( GLuint list_surf_light, unsigned char cullFlag, int  elemGroupNr, Normalen *norm,
                     int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CTri3 *surf3,
			 CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CHexa8 *cquad8 );
void drawElements_light( GLuint list_elem_light,  unsigned char cullFlag, int  elemGroupNr,  Normalen *norm,
                     int *num_etype, Nodes *node, CHexa26 *chexa26, CHexa8 *chexa8,
			 CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CTet4 *ctet4, CTet10 *ctet10, CHexa8 *cquad8 );
void drawSurfPoints_plot( GLuint list_surfpoints_plot, int  elemGroupNr,
                     int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CTri3 *surf3,
			  CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CHexa8 *cquad8 );
void drawSurfaces_plot( GLuint list_surf_plot,  unsigned char cullFlag, int  elemGroupNr,
                     int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CTri3 *surf3,
			CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CHexa8 *cquad8 );
void drawElemPoints_plot( GLuint list_elempoints_plot, int  elemGroupNr,
                     int *num_etype, Nodes *node, CHexa26 *chexa26, CHexa8 *chexa8,
			  CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CTet10 *ctet10, CHexa8 *cquad8  );


/* dispLists_index.c  index mode */
void drawSurfaces_load( GLuint list_surf_load,  unsigned char cullFlag, int  elemGroupNr, int *colNr,
                     int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CTri3 *surf3,
			CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CHexa8 *cquad8 );
void drawElements_load( GLuint list_elem_load,  unsigned char cullFlag, int  elemGroupNr, int *colNr,
                     int *num_etype, Nodes *node, CHexa26 *chexa26, CHexa8 *chexa8,
			CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CTet4 *ctet4, CTet10 *ctet10, CHexa8 *cquad8 );
void drawElements_elstress( GLuint list_elem_elstress,  unsigned char cullFlag, int  elemGroupNr,
                     int *num_etype, Nodes *node, CHexa26 *chexa26, CHexa8 *chexa8,
			    CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam );


/* dispLists_rgb.c  rgb mode */
void drawSurfaces_load_rgb( GLuint list_surf_load,  unsigned char cullFlag, int  elemGroupNr, Colours *colRgb,
                     int *num_stype, int *num_etype, Nodes *node, CQuad4 *surf4, CTri3 *surf3,
			CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CHexa8 *cquad8 );
void drawElements_load_rgb( GLuint list_elem_load,  unsigned char cullFlag, int  elemGroupNr, Colours *colRgb,
                     int *num_etype, Nodes *node, CHexa26 *chexa26, CHexa8 *chexa8,
			CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam, CTet4 *ctet4, CTet10 *ctet10, CHexa8 *cquad8 );
void drawElements_elstress_rgb( GLuint list_elem_elstress,  unsigned char cullFlag, int  elemGroupNr,
                     int *num_etype, Nodes *node, CHexa26 *chexa26, CHexa8 *chexa8,
			    CTri3 *ctri3, CQuad4 *cquad4, CBeam *cbeam );


/*  extFunktions.h */
void help( void );
int askGLError(unsigned char buffer[134]);
void defineColIndexes_light(void);
void defineColIndexes_load(void);
void defineColIndexes_set(void);
void zoom(float xcur0, float ycur0, float xcur1, float ycur1);
void qzoom(void);
void center(float x, float y, float z);
void qcenter(int x, int y);
void rot_u(float a);
void rot_r(float a);
void rot_c(float a);
void rot_x(float a);
void rot_y(float a);
void rot_z(float a);
void pre_rot( unsigned char *record);
int transform( unsigned char *record, int anz_n, Nodes *node );

/* frdviewer.h */
void nodalLoadcase( int auswahl, int lc, Summen *anz, Scale *scale, Nodes *node,
                    Loadcases *lcase, float *value_n, int *num_etype, CHexa26 *chexa26, Colours *colRgb
                   , int *colNr, int offset, int maxIndex, int steps, unsigned char *entityName );
void elementLoadcase( int auswahl, int lc, Summen *anz, Scale *scale, Loadcases *lcase,
                      int *num_etype, CBeam *cbeam, int offset, int maxIndex, int steps );
void allocUtilityData(void);
void entryfunktion( int state );
void WindowState( int state );
void Mouse( int x, int y );
void MouseState( int button, int state, int x, int y );
void menu( int selection );
void updateDispLists(void);
void ConfigureAndShowWindow_Light(void);
void ConfigureAndShowWindow_Load( void );
void ConfigureAndShowWindow_Sequence( void );
int  plot( unsigned char *record );
int  plus( unsigned char *record );
void changeAnimation( int selection );
void newTimePerPeriode( int selection );
void tuneAnimation( int selection );
void stepsAnimation( int selection );
void redraw(void);
void selectEntity( int selection );
void selectLoadcase( int selection );
void selectView( int selection );
void newLc( unsigned char *record);
void Keyboard( unsigned char gkey, int x, int y );
void moveModel(void);
void DrawGraficLoad( void );
void DrawGraficLight( void );
void DrawGraficAnimate( void );
void DrawGraficSequence( void );
void DrawPickedItems(void);
void DrawAxes();
void iniDrawMenu(void);
void DrawMenuLoad( void );
void DrawMenuLight( void );
void DrawMenuAnimate( void );
void DrawMenuSet( void );
void reshape( int width, int height );
void initLightAndMaterial_rgb( void );
void initLightAndMaterial_index( void );

/* iniMeshData.h     */
void makeSurfaces(void);
void iniMeshData( unsigned char *datin );

/* mergEntity.h */
int mergeNod( int nod, int lock );
int mergePnt( int *pnt_indx, int anz_p, int indx, int lock );
int mergeLine( int indx );
int mergeLcmb( int indx );
int mergeSurf( int indx );
void pre_merge( unsigned char *record );

/* orient.h */
int orientLcmb( int nr, SumGeo *anz, Lcmb *lcmb, Lines *line, Points    *point );
int orientSurf( int nr, SumGeo *anz, Gsur *surf, Lcmb *lcmb, Lines *line, Points    *point);
int orientBody( int nr, SumGeo *anz, Gbod *body, Gsur *surf, Lcmb *lcmb, Lines *line, Points *point );
void orientSet( unsigned char *record);

/* pickFunktions.h    */
int hitAction( GLuint *name, unsigned char *type, int x, int y );
int pickstack( GLuint name );
int processHits( GLint hits, GLuint *buffer, unsigned char *type, unsigned char *mode, int x, int y );
void pick( unsigned char gkey, int x, int y );
void goPicking( int x, int y, unsigned char *type );
void drawListsPick(void);
void qali(void);
void qnod(void);
void qpnt(void);
void qcnt(void);
void qenq(void);
void qflp(void);
void qdiv(void);
void qadd( unsigned char *record);
void qdel( void );
void qrem( unsigned char *record);
void qlin(void);
void qsur(void);
void qbod(void);
void qspl(void);
void moveNode(int n, int x, int y);
void movePoint(int p, int x, int y);
void createPoint( int x, int y );
int createLine( unsigned char *pnt, int flag );
void splitLine( unsigned char *lin, int x, int y  );
void pre_align( int nr, int flag );
void drawListsPick(void);
void moveModel(void);
void updateDispLists(void);
void flip( unsigned char type, int indx);

/* sendSet.h  */
void sendSurfNormalen( unsigned char *setname, unsigned char *format, Summen *anz, Nodes *node, Elements *e_enqire, Loadcases *lcase , Scale *scale );
void sendPressure( unsigned char *setname, unsigned char *format, Summen *anz, Nodes *node, Elements *e_enqire, unsigned char *value );
void sendSPC( unsigned char *setname, unsigned char *format, Summen *anz, Nodes *node, Elements *e_enqire, unsigned char *value );
void sendSliders( unsigned char *setname, unsigned char *format, Summen *anz, Nodes *node, Elements *e_enqire, unsigned char *spcType );
void sendNames( unsigned char *setname, unsigned char *format, Summen *anz, Nodes *node, Elements *e_enqire );

/* setFunktions.h  */
void iniSetFunktions(void);
void drawNodes_plot( int num, int *name, Nodes *node , int col, unsigned char type);
void drawElements_plot( int num, int *name, Nodes *node, Elements *e_enqire, int col, unsigned char type );
void drawPoints_plot( int num, int *name, Points *point , int col, unsigned char type);
void drawLines_plot( int num, int *name, Lines *line , Points *point, int col, unsigned char type);
void drawSurfs_plot( int num, int *name, Gsur *surf, Lcmb *lcmb, Lines *line , Points *point, int col, unsigned char type);
void drawBodys_plot( int num, int *name, Gbod *body, Gsur *surf, Lcmb *lcmb, Lines *line , Points *point, int col, unsigned char type);
void drawNurs_plot( int num, int *name, Nurbs *nurbs , int col, unsigned char type);
int getSetNr(unsigned char *setname);
int getAliasNr(unsigned char *name);
int getNodNr(int nodnr );
int getElemNr (int elnr );
int getE_enqNr (int elnr );
int getPntNr(unsigned char *name);
int getLineNr(unsigned char *name);
int getLcmbNr(unsigned char *name);
int getSurfNr(unsigned char *name);
int getBodyNr(unsigned char *name);
int getNursNr(unsigned char *name);
void delSet( unsigned char *setname);
void seto( unsigned char *setname );
void setc( unsigned char *setname );
int seta( unsigned char *setname, unsigned char *type, unsigned char *name);
int setr( unsigned char *setname, unsigned char *type, unsigned char *name);
void pre_seta( unsigned char *record);
void pre_setr( unsigned char *record);
void pre_del( unsigned char *record);
void pre_move( unsigned char *record);
int prnt(unsigned char *record);
int  zap(unsigned char *record);
void groupAdd( int *hasGroup, unsigned char *record );
unsigned char getLetter(int i);
int getFamName( int n, unsigned char *c );
int getNewName( unsigned char *name, unsigned char *type );
int commandoInterpreter( unsigned char *type, unsigned char *string, int na, FILE *handle1 );
void readfbd( unsigned char *datin );
void readfrd( unsigned char *datin, unsigned char *setname );
void pre_read( unsigned char *record );
void pre_elty( unsigned char *record );
void pre_mesh( unsigned char *record );
void completeSet( unsigned char *setname, unsigned char *type );
void pre_write( unsigned char *record );
void senddata( unsigned char *setname, unsigned char *format, Summen *anz, Nodes *node, Elements *e_enqire,
               Loadcases *lcase , Scale *scale, int complete );
void drawSets(int mode);
void rep( int j );

