/* --------------------------------------------------------------------  */
/*                          CALCULIX                                     */
/*                   - GRAPHICAL INTERFACE -                             */
/*                                                                       */
/*     A 3-dimensional pre- and post-processor for finite elements       */
/*              Copyright (C) 1996 Klaus Wittig                          */
/*                                                                       */
/*     This program is free software; you can redistribute it and/or     */
/*     modify it under the terms of the GNU General Public License as    */
/*     published by the Free Software Foundation; version 2 of           */
/*     the License.                                                      */
/*                                                                       */
/*     This program is distributed in the hope that it will be useful,   */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of    */ 
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      */
/*     GNU General Public License for more details.                      */
/*                                                                       */
/*     You should have received a copy of the GNU General Public License */
/*     along with this program; if not, write to the Free Software       */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         */
/* --------------------------------------------------------------------  */


/* TODO */
/*
 Bugs:
*/

#include <cgx.h>
#include  <time.h>

#define     TEST            0       /* debugging */


/* data */
extern int       num_etype[13];
extern int       num_stype[13];
extern int       numEdges;
extern int       anz_normalen;
extern int       num_onod;          /* nr-of-original-nodes (nodes from input-file) */
extern int       num_olc;           /* nr-of-original-loadcases (lc from input-file) */

extern Summen    anz[1];
extern Nodes     *node;
extern Elements  *elem;
extern Elements  *e_enqire;         /* elem-array by elem-number instead of elem[index]... */
extern Datasets  *lcase;

extern float     *value_n;
extern float     *vp;
extern Nodes     *n_anim;
extern Normalen  *norm;
extern CTri3     *surf3;            /* he20(he26) surfs      */
extern CTri6     *surf6;            /* tet10 surfs      */
extern CQuad4    *surf4;            /* he8-surfs & elemedges */
extern CHexa8    *surf8;            /* he20 elem-edges        */
extern Edges     *edge;             /* model-edges           */
extern CBeam     *cbeam;
extern CBeam3    *cbeam3;
extern CTri3     *ctri3;
extern CTri6     *ctri6;
extern CQuad4    *cquad4;
extern CHexa8    *cquad8;
extern CHexa8    *chexa8;
extern CTet4     *ctet4;
extern CTet10    *ctet10;
extern CHexa20   *chexa20;
extern CPenta6   *cpenta6;
extern CPenta20  *cpenta20;
extern Scale     scale[1];
extern Faces     *face;

extern Sets      *set;

extern int       lc;                                      /* aktiver Dataset (4C, 5C)*/
extern int       elemGroupNr, elemGroup[99];   /* aktiv element-group, 99-possible groups */
extern int       elemMat[99];      /*  Material Numbers, Number of Materials stored in elemMat[0] */


/* find the element face */
int getFaceNr(int elem_nr, int *nod)
{
  int j,k,n, f[6], ni[8];

  n=0;
  if((e_enqire[elem_nr].type==1)||(e_enqire[elem_nr].type==4)||(e_enqire[elem_nr].type==2)||(e_enqire[elem_nr].type==5))
  {
    for(j=0; j<8; j++) 
      for(k=0; k<4; k++)
        if(e_enqire[elem_nr].nod[j]==nod[k])
          ni[n++]=j+1;
    if (n==4)
    {
      /* check which sides are involved */
      for (j=0; j<6; j++) f[j]=0;
      for (j=0; j<4; j++)
      {
        if ((ni[j]==1)||(ni[j]==2)||(ni[j]==3)||(ni[j]==4)) f[0]++;
        if ((ni[j]==5)||(ni[j]==8)||(ni[j]==7)||(ni[j]==6)) f[1]++;
        if ((ni[j]==1)||(ni[j]==5)||(ni[j]==6)||(ni[j]==2)) f[2]++;
        if ((ni[j]==2)||(ni[j]==6)||(ni[j]==7)||(ni[j]==3)) f[3]++;
        if ((ni[j]==3)||(ni[j]==7)||(ni[j]==8)||(ni[j]==4)) f[4]++;
        if ((ni[j]==4)||(ni[j]==8)||(ni[j]==5)||(ni[j]==1)) f[5]++;
      }
    }
    for (j=0; j<6; j++){ if(f[j]==4) return(j); }
  }
  else if((e_enqire[elem_nr].type==3)||(e_enqire[elem_nr].type==6))
  {
    for(j=0; j<4; j++) 
      for(k=0; k<3; k++)
        if(e_enqire[elem_nr].nod[j]==nod[k])
          ni[n++]=j+1;
    if (n==3)
    {
      /* check which sides are involved */
      for (j=0; j<4; j++) f[j]=0;
      for (j=0; j<3; j++)
      {
        if ((ni[j]==1)||(ni[j]==3)||(ni[j]==2)) f[0]++;
        if ((ni[j]==1)||(ni[j]==2)||(ni[j]==4)) f[1]++;
        if ((ni[j]==2)||(ni[j]==3)||(ni[j]==4)) f[2]++;
        if ((ni[j]==3)||(ni[j]==1)||(ni[j]==4)) f[3]++;
      }
    }
    for (j=0; j<4; j++){ if(f[j]==3) return(j); }
  }
  return(-1);
}



/* --- define the surfaces of the part which are derived from the volume-elements -- */
void makeSurfaces()
{
  int  i,j,ne, ns;
  int  *pfaces, *pfacesN, *pfacesN1;

  numEdges=0;
  for (i=0; i<13; i++) num_stype[i]=0; 

  /* first all volume elements */
  if ( num_etype[1] > 0 )
  {
    num_stype[9] = selectDisplayFacesHe8 ( chexa8, num_etype[1], &pfaces, &numEdges);

    if ( (surf4 = (CQuad4 *)realloc( (CQuad4 *)surf4, (num_stype[9]+1)*sizeof(CQuad4)))==NULL )
      printf("\n\n ERROR: realloc failed surf4\n\n") ;
    if ( (edge = (Edges *)realloc( (Edges *)edge, (numEdges+1) * sizeof(Edges))) == NULL )
      printf("\n\n ERROR: realloc failed edge\n\n") ;

    pfacesN = pfaces;
    for (i=0; i<num_stype[9]; i++)
    {
      surf4[i].nod[0]= *(pfacesN++) ;
      surf4[i].nod[1]= *(pfacesN++) ;
      surf4[i].nod[2]= *(pfacesN++) ;
      surf4[i].nod[3]= *(pfacesN++) ;
      surf4[i].elem_nr = *(pfacesN++) ;
      surf4[i].group = *(pfacesN++) ;
      surf4[i].mat = 0 ;   /* -1 nur kante, 0 kante und surf, 1 nur surf */
    }
    for (i=0; i<numEdges; i++)
    {
      edge[i].p1= *(pfacesN++) ;
      edge[i].p2= *(pfacesN++) ;
    }

    /* define the faces */
    if ( (face = (Faces *)realloc( (Faces *)face, (num_stype[0]+num_stype[9]+1) * sizeof(Faces))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    pfacesN = pfaces;
    for (i=num_stype[0]; i<num_stype[0]+num_stype[9]; i++)
    {
      face[i].nod[0]= *(pfacesN++) ;
      face[i].nod[1]= *(pfacesN++) ;
      face[i].nod[2]= *(pfacesN++) ;
      face[i].nod[3]= *(pfacesN++) ;
      face[i].elem_nr= *(pfacesN++) ;
      face[i].group  = *(pfacesN++) ;
      face[i].type   = 9 ;
      face[i].mat    = 0 ; 
      face[i].nr     = getFaceNr(face[i].elem_nr, face[i].nod);
    }
    num_stype[0]+=num_stype[9];

    free(pfaces);
  }

  if ( num_etype[2] > 0 )
  {
    ne=numEdges;
    ns = selectDisplayFacesPe6 ( cpenta6, num_etype[2], &pfaces, &numEdges);

    numEdges+= ne;
    if ( (surf4 = (CQuad4 *)realloc( (CQuad4 *)surf4, (num_stype[9]+ns+1) * sizeof(CQuad4))) == NULL )
      printf("\n\n ERROR: realloc failed surf4\n\n") ;
    if ( (edge = (Edges *)realloc( (Edges *)edge, (numEdges+1) * sizeof(Edges))) == NULL )
      printf("\n\n ERROR: realloc failed edge\n\n") ;

    pfacesN = pfaces;
    for (i=num_stype[9]; i<num_stype[9]+ns; i++)
    {
      surf4[i].nod[0]= *(pfacesN++) ;
      surf4[i].nod[1]= *(pfacesN++) ;
      surf4[i].nod[2]= *(pfacesN++) ;
      surf4[i].nod[3]= *(pfacesN++) ;
      surf4[i].elem_nr = *(pfacesN++) ;
      surf4[i].group = *(pfacesN++) ;
      surf4[i].mat = 0 ;
    }
    for (i=ne; i<numEdges; i++)
    {
      edge[i].p1= *(pfacesN++) ;
      edge[i].p2= *(pfacesN++) ;
    }

    /* define the faces */
    if ( (face = (Faces *)realloc( (Faces *)face, (num_stype[0]+ns+1) * sizeof(Faces))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    pfacesN = pfaces;
    for (i=num_stype[0]; i<num_stype[0]+ns; i++)
    {
      face[i].nod[0]= *(pfacesN++) ;
      face[i].nod[1]= *(pfacesN++) ;
      face[i].nod[2]= *(pfacesN++) ;
      face[i].nod[3]= *(pfacesN++) ;
      face[i].elem_nr= *(pfacesN++) ;
      face[i].group  = *(pfacesN++) ;
      face[i].type   = 9 ;
      face[i].mat    = 0 ; 
      face[i].nr     = getFaceNr(face[i].elem_nr, face[i].nod);
    }
    num_stype[0]+=ns;

    free(pfaces);
    num_stype[9]+=ns;
  }

  if ( num_etype[3] > 0 )
  {
    ne=numEdges;
    ns = selectDisplayFacesTet4 ( ctet4, num_etype[3], &pfaces, &numEdges);
    numEdges+= ne;

    if ( (surf3 = (CTri3 *)realloc( (CTri3 *)surf3, (num_stype[7]+ns+1) * sizeof(CTri3)))==NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    if ( (edge = (Edges *)realloc( (Edges *)edge, (numEdges+1) * sizeof(Edges))) == NULL )
      printf("\n\n ERROR: realloc failed edge\n\n") ;

    pfacesN = pfaces;
    for (i=num_stype[7]; i<num_stype[7]+ns; i++)
    {
      surf3[i].nod[0]= *(pfacesN++) ;
      surf3[i].nod[1]= *(pfacesN++) ;
      surf3[i].nod[2]= *(pfacesN++) ;pfacesN++;
      surf3[i].elem_nr = *(pfacesN++) ;
      surf3[i].group = *(pfacesN++) ;
      surf3[i].mat = 0 ;   /* -1 nur kante, 0 kante und surf, 1 nur surf */
    }
    for (i=ne; i<numEdges; i++)
    {
      edge[i].p1= *pfacesN++ ;
      edge[i].p2= *(pfacesN++) ;
    }

    /* define the faces */
    if ( (face = (Faces *)realloc( (Faces *)face, (num_stype[0]+ns+1) * sizeof(Faces))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    pfacesN = pfaces;
    for (i=num_stype[0]; i<num_stype[0]+ns; i++)
    {
      face[i].nod[0]= *(pfacesN++) ;
      face[i].nod[1]= *(pfacesN++) ;
      face[i].nod[2]= *(pfacesN++) ;pfacesN++;
      face[i].elem_nr= *(pfacesN++) ;
      face[i].group  = *(pfacesN++) ;
      face[i].type   = 7 ;
      face[i].mat    = 0 ; 
      face[i].nr     = getFaceNr(face[i].elem_nr, face[i].nod);
    }
    num_stype[0]+=ns;

    free(pfaces);
    num_stype[7]+=ns;
  }

  if ( num_etype[4] > 0 )
  {
    ne=numEdges;
    ns = selectDisplayFacesHe20( chexa20, num_etype[4], &pfaces, &numEdges); 

    numEdges+= ne;
    if ( (surf3 = (CTri3 *)realloc( (CTri3 *)surf3, (num_stype[7]+ns+1) * sizeof(CTri3))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    if ( (surf8 = (CHexa8 *)realloc( (CHexa8 *)surf8, (num_stype[10]+ns/8+1) * sizeof(CHexa8))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;

    if ( (edge = (Edges *)realloc((Edges *)edge, (numEdges+1) * sizeof(Edges))) == NULL )
      printf("\n\n ERROR: realloc failed edge\n\n" ) ;

    pfacesN = pfaces;
    for (i=num_stype[10]; i<num_stype[10]+ns/8; i++)
    {
      surf8[i].nod[0]= *(pfacesN    ) ;
      surf8[i].nod[1]= *(pfacesN+13 ) ;
      surf8[i].nod[2]= *(pfacesN+22 ) ;
      surf8[i].nod[3]= *(pfacesN+8  ) ;
      surf8[i].nod[4]= *(pfacesN+1  ) ;
      surf8[i].nod[5]= *(pfacesN+16 ) ;
      surf8[i].nod[6]= *(pfacesN+10 ) ;
      surf8[i].nod[7]= *(pfacesN+2  ) ;
      surf8[i].elem_nr= *(pfacesN+24) ;
      surf8[i].group  = *(pfacesN+25) ;
      surf8[i].mat = -1 ; /* -1 nur kante, 0 kante und surf, 1 nur surf */
      pfacesN+=26;
    }
    pfacesN = pfaces;
    pfacesN1 = pfaces+24;
    for (i=num_stype[7]; i<num_stype[7]+ns; i++)
    {
      if (pfacesN >= pfacesN1) { pfacesN1+=26; pfacesN+= 2; }
      surf3[i].nod[0]= *(pfacesN) ;
      surf3[i].nod[1]= *(pfacesN+1) ;
      surf3[i].nod[2]= *(pfacesN+2) ;
      surf3[i].elem_nr = *(pfacesN1) ;
      surf3[i].group = *(pfacesN1+1) ;
      surf3[i].mat = 1 ; /* -1 nur kante, 0 kante und surf, 1 nur surf */
      pfacesN+= 3;
    }
    pfacesN+= 2;
    for (i=ne; i<numEdges; i+=2)
    {
      edge[i].p1= *pfacesN ;
      edge[i].p2= *(pfacesN+1) ;
      edge[i+1].p1= *(pfacesN+1) ;
      edge[i+1].p2= *(pfacesN+2) ;
      pfacesN+= 3;
    }

    /* define the faces */
    if ( (face = (Faces *)realloc( (Faces *)face, (num_stype[0]+ns/8+1) * sizeof(Faces))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    pfacesN = pfaces;
    for (i=num_stype[0]; i<num_stype[0]+ns/8; i++)
    {
      face[i].nod[0]= *(pfacesN    ) ;
      face[i].nod[1]= *(pfacesN+13 ) ;
      face[i].nod[2]= *(pfacesN+22 ) ;
      face[i].nod[3]= *(pfacesN+8  ) ;
      face[i].nod[4]= *(pfacesN+1  ) ;
      face[i].nod[5]= *(pfacesN+16 ) ;
      face[i].nod[6]= *(pfacesN+10 ) ;
      face[i].nod[7]= *(pfacesN+2  ) ;
      face[i].elem_nr= *(pfacesN+24) ;
      face[i].group  = *(pfacesN+25) ;
      face[i].type   = 10 ;
      face[i].mat    = 0 ; 
      face[i].nr     = getFaceNr(face[i].elem_nr, face[i].nod);
      pfacesN+=26;
    }
    num_stype[0]+=ns/8;

    free(pfaces);
    num_stype[7]+=ns;
    num_stype[10]+=ns/8;
  }

  if ( num_etype[5] > 0 )
  {
    ne=numEdges;
    ns = selectDisplayFacesPe20 ( cpenta20, num_etype[5], &pfaces, &numEdges); 

    numEdges+= ne;
    if ( (surf3 = (CTri3 *)realloc( (CTri3 *)surf3, (num_stype[7]+ns+1) * sizeof(CTri3))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    if ( (surf8 = (CHexa8 *)realloc( (CHexa8 *)surf8, (num_stype[10]+ns/8+1) * sizeof(CHexa8))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;

    if ( (edge = (Edges *)realloc((Edges *)edge, (numEdges+1) * sizeof(Edges))) == NULL )
      printf("\n\n ERROR: realloc failed edge\n\n" ) ;

    pfacesN = pfaces;
    for (i=num_stype[10]; i<num_stype[10]+ns/8; i++)
    {
      surf8[i].nod[0]= *(pfacesN    ) ;
      surf8[i].nod[1]= *(pfacesN+13 ) ;
      surf8[i].nod[2]= *(pfacesN+22 ) ;
      surf8[i].nod[3]= *(pfacesN+8  ) ;
      surf8[i].nod[4]= *(pfacesN+1  ) ;
      surf8[i].nod[5]= *(pfacesN+16 ) ;
      surf8[i].nod[6]= *(pfacesN+10 ) ;
      surf8[i].nod[7]= *(pfacesN+2  ) ;
      surf8[i].elem_nr= *(pfacesN+24) ;
      surf8[i].group  = *(pfacesN+25) ;
      surf8[i].mat = -1 ; /* -1 nur kante, 0 kante und surf, 1 nur surf */
      pfacesN+=26;
    }
    pfacesN = pfaces;
    pfacesN1 = pfaces+24;
    for (i=num_stype[7]; i<num_stype[7]+ns; i++)
    {
      if (pfacesN >= pfacesN1) { pfacesN1+=26; pfacesN+= 2; }
      surf3[i].nod[0]= *(pfacesN) ;
      surf3[i].nod[1]= *(pfacesN+1) ;
      surf3[i].nod[2]= *(pfacesN+2) ;
      surf3[i].elem_nr = *(pfacesN1) ;
      surf3[i].group = *(pfacesN1+1) ;
      surf3[i].mat = 1 ; /* -1 nur kante, 0 kante und surf, 1 nur surf */
      pfacesN+= 3;
    }
    pfacesN+= 2;

    for (i=ne; i<numEdges; i+=2)
    {
      edge[i].p1= *pfacesN ;
      edge[i].p2= *(pfacesN+1) ;
      edge[i+1].p1= *(pfacesN+1) ;
      edge[i+1].p2= *(pfacesN+2) ;
      pfacesN+= 3;
    }

    /* define the faces */
    if ( (face = (Faces *)realloc( (Faces *)face, (num_stype[0]+ns/8+1) * sizeof(Faces))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    pfacesN = pfaces;
    for (i=num_stype[0]; i<num_stype[0]+ns/8; i++)
    {
      face[i].nod[0]= *(pfacesN    ) ;
      face[i].nod[1]= *(pfacesN+13 ) ;
      face[i].nod[2]= *(pfacesN+22 ) ;
      face[i].nod[3]= *(pfacesN+8  ) ;
      face[i].nod[4]= *(pfacesN+1  ) ;
      face[i].nod[5]= *(pfacesN+16 ) ;
      face[i].nod[6]= *(pfacesN+10 ) ;
      face[i].nod[7]= *(pfacesN+2  ) ;
      face[i].elem_nr= *(pfacesN+24) ;
      face[i].group  = *(pfacesN+25) ;
      face[i].type   = 10 ;
      face[i].mat    = 0 ; 
      face[i].nr     = getFaceNr(face[i].elem_nr, face[i].nod);
      pfacesN+=26;
    }
    num_stype[0]+=ns/8;

    free(pfaces);
    num_stype[7]+=ns;
    num_stype[10]+=ns/8;
  }
  
  if ( num_etype[6] > 0 )
  {
    ne=numEdges;
    ns= selectDisplayFacesTet10 ( ctet10, num_etype[6], &pfaces, &numEdges);
    numEdges+= ne;

    if ( (surf3 = (CTri3 *)realloc( (CTri3 *)surf3, (num_stype[7]+ns+1) * sizeof(CTri3)))==NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    if ( (surf6 = (CTri6 *)realloc( (CTri6 *)surf6, (num_stype[7]+ns/4+1) * sizeof(CTri6)))==NULL)
      printf("\n\n ERROR: realloc failed surf6\n\n") ;

    if ( (edge = (Edges *)realloc((Edges *)edge, (numEdges+1) * sizeof(Edges))) == NULL )
      printf("\n\n ERROR: realloc failed edge\n\n" ) ;

    pfacesN = pfaces;
    for (i=num_stype[8]; i<num_stype[8]+ns/4; i++)
    {
      surf6[i].nod[0]= *(pfacesN    ) ;
      surf6[i].nod[1]= *(pfacesN+12  ) ;
      surf6[i].nod[2]= *(pfacesN+17 ) ;
      surf6[i].nod[3]= *(pfacesN+6 ) ;
      surf6[i].nod[4]= *(pfacesN+16 ) ;
      surf6[i].nod[5]= *(pfacesN+15 ) ;
      surf6[i].elem_nr= *(pfacesN+18) ;
      surf6[i].group  = *(pfacesN+19) ;
      surf6[i].mat = -1 ; /* -1 nur kante, 0 kante und surf, 1 nur surf */
      pfacesN+=20;
    }
    pfacesN = pfaces;
    for (i=num_stype[7]; i<num_stype[7]+ns; i++)
    {
      surf3[i].nod[0]= *(pfacesN++) ;
      surf3[i].nod[1]= *(pfacesN++) ;
      surf3[i].nod[2]= *(pfacesN++) ;
      surf3[i].elem_nr = *(pfacesN++) ;
      surf3[i].group = *(pfacesN++) ;
      surf3[i].mat = 1 ; /* -1 nur kante, 0 kante und surf, 1 nur surf */
    }
    for (i=ne; i<numEdges; i+=2)
    {
      edge[i].p1= *(pfacesN++) ;
      edge[i].p2= *(pfacesN) ;
      edge[i+1].p1= *(pfacesN++) ;
      edge[i+1].p2= *(pfacesN++) ;
    }

    /* define the faces */
    if ( (face = (Faces *)realloc( (Faces *)face, (num_stype[0]+ns/4+1) * sizeof(Faces))) == NULL )
      printf("\n\n ERROR: realloc failed surf3\n\n") ;
    pfacesN = pfaces;
    for (i=num_stype[0]; i<num_stype[0]+ns/4; i++)
    {
      face[i].nod[0]= *(pfacesN    ) ;
      face[i].nod[1]= *(pfacesN+12 ) ;
      face[i].nod[2]= *(pfacesN+17 ) ;
      face[i].nod[3]= *(pfacesN+6  ) ;
      face[i].nod[4]= *(pfacesN+16 ) ;
      face[i].nod[5]= *(pfacesN+15 ) ;
      face[i].elem_nr= *(pfacesN+18) ;
      face[i].group  = *(pfacesN+19) ;
      face[i].type   = 8 ;
      face[i].mat    = 0 ; 
      face[i].nr     = getFaceNr(face[i].elem_nr, face[i].nod);
      pfacesN+=20;
    }
    num_stype[0]+=ns/4;

    free(pfaces);
    num_stype[7]+=ns;
    num_stype[8]+=ns/4;
  }

  /* the shell elements: */
  ne=num_stype[0];
  num_stype[0]+=num_etype[7]+num_etype[8]+num_etype[9]+num_etype[10];
  if ( (face = (Faces *)realloc( (Faces *)face, (num_stype[0]+1) * sizeof(Faces))) == NULL )
    printf("\n\n ERROR: realloc failed surf3\n\n") ;
  if ( num_etype[7] > 0 )
  {
    ns=numEdges;
    numEdges+=findCTri3Edges( ctri3, num_etype[7], &pfaces);
    pfacesN = pfaces;
 
    if ( (edge = (Edges *)realloc((Edges *)edge, (numEdges+1) * sizeof(Edges))) == NULL )
      printf("\n\n ERROR: realloc failed edge\n\n" ) ;
    for (i=ns; i<numEdges; i++)
    {
      edge[i].p1= *(pfacesN++) ;
      edge[i].p2= *(pfacesN) ;
      pfacesN+=4;
    }
    free(pfaces);

    for (i=0; i<num_etype[7]; i++)
    {
      for(j=0;j<3;j++) face[ne].nod[j]= ctri3[i].nod[j] ;
      face[ne].elem_nr= ctri3[i].elem_nr ;
      face[ne].group  = ctri3[i].group ;
      face[ne].type   = 7 ;
      face[ne].mat    = 0 ; 
      face[ne++].nr     = -1;
    }
  }
  if ( num_etype[8] > 0 )
  {
    for (i=0; i<num_etype[8]; i++)
    {
      for(j=0;j<6;j++) face[ne].nod[j]= ctri6[i].nod[j] ;
      face[ne].elem_nr= ctri6[i].elem_nr ;
      face[ne].group  = ctri6[i].group ;
      face[ne].type   = 8 ;
      face[ne].mat    = 0 ; 
      face[ne++].nr     = -1;
    }
  }
  if ( num_etype[9] > 0 )
  {
    for (i=0; i<num_etype[9]; i++)
    {
      for(j=0;j<4;j++) face[ne].nod[j]=     cquad4[i].nod[j] ;
      face[ne].elem_nr= cquad4[i].elem_nr ;
      face[ne].group  = cquad4[i].group ;
      face[ne].type   = 9 ;
      face[ne].mat    = 0 ; 
      face[ne++].nr     = -1;
    }
  }
  if ( num_etype[10] > 0 )
  {
    for (i=0; i<num_etype[10]; i++)
    {
      for(j=0;j<8;j++) face[ne].nod[j]=     cquad8[i].nod[j] ;
      face[ne].elem_nr= cquad8[i].elem_nr ;
      face[ne].group  = cquad8[i].group ;
      face[ne].type   = 10 ;
      face[ne].mat    = 0 ; 
      face[ne++].nr     = -1;
    }
  }

  if((set[0].face =(int *)realloc((int *)set[0].face,(num_stype[0]+1)*sizeof(int))) == NULL)
    printf(" ERROR: malloc failed in set[%d]:%s\n\n", 0, set[0].name);
  set[0].anz_f=num_stype[0];
  for (i=0; i<set[0].anz_f; i++) set[0].face[i]=i;
}


void iniMeshData( unsigned char *datin, unsigned char *format )
{
  int i;
  static clock_t t0, t1 ;
  int ***facebuf=NULL;

  t0=clock() ;
  if(compare(format,"frd",3)==3)
  { 
    printf (" Try to read ccx results\n\n");
    if ( (readfrd( datin, anz, &node, &elem, &lcase)) == -1) { exit(-1);}
  }
  else if(compare(format,"ccx",3)==3)
  { 
    printf (" Try to read ccx input\n\n");
    anz->n=0;
    anz->e=0;
    anz->l=0;
    anz->emax=-MAXVALUE;  anz->emin=MAXVALUE;
    anz->nmax=-MAXVALUE;  anz->nmin=MAXVALUE;
    if ( (readccx( datin, anz, &node, &elem, &lcase, &facebuf)) == -1) { exit(-1);}
  }
  else if(compare(format,"duns2d",6)==6)
  { 
    printf (" Try to read 2D-duns results\n\n");
    if ( (  readDuns( datin, anz, &node, &elem, &lcase, 9)) == -1) { exit(-1);}
  }
  else if(compare(format,"duns3d",6)==6)
  { 
    printf (" Try to read 3D-duns results\n\n");
    if ( (  readDuns( datin, anz, &node, &elem, &lcase, 1)) == -1) { exit(-1);}
  }
  else if(compare(format,"stl",3)==3)
  { 
    printf (" Try to read stl file\n\n");
    if ( (  readStl( datin, anz, &node, &elem, &lcase)) == -1) { exit(-1);}
  }
  t1=clock() ;
  printf (" Elements: %d Nodes:%d Datasets:%d ",anz->e,anz->n,anz->l);
  printf (" MinElemNr: %d MaxElemNr: %d MinNodNr:%d MaxNodNr:%d ",anz->emin,anz->emax,anz->nmin,anz->nmax);
  printf (" read in %f sec \n",(float)(t1-t0)/CLOCKS_PER_SEC);
  for( i=1; i<13; i++) 
  {
    printf(" found elements of type %d: %d\n", i, anz->etype[i]);
  }

  iniElements(anz, elem, facebuf);
  free(elem); elem=NULL;
}



void iniElements(Summen *anz, Elements *elem, int ***facebuf)
{
  int  i, n, n1, n2, setNr;

  /* flag the nodes */
  for( i=0; i<anz->n; i++) node[node[i].nr].pflag=0;
  for (i=0; i<13; i++)  num_etype[i]=0;

  if (anz->etype[1] > 0)
  {
    if ( (chexa8 = (CHexa8 *)malloc( (anz->etype[1]+1) * sizeof(CHexa8))) == NULL )
      printf("\n\n ERROR: malloc failed chexa8\n\n") ;
  }
  if (anz->etype[2] > 0)
  {
    if ( (cpenta6 = (CPenta6 *)malloc( (anz->etype[2]+1) * sizeof(CPenta6))) == NULL )
      printf("\n\n ERROR: malloc failed cpenta6\n\n") ;
  }
  if (anz->etype[3] > 0)
  {
    if ( (ctet4 = (CTet4 *)malloc( (anz->etype[3]+1) * sizeof(CTet4))) == NULL )
      printf("\n\n ERROR: malloc failed ctet4\n\n") ;
  }
  if (anz->etype[4] > 0)
  {
    if ( (chexa20 = (CHexa20 *)malloc( (anz->etype[4]+1) * sizeof(CHexa20))) == NULL )
      printf("\n\n ERROR: malloc failed chexa20\n\n") ;
    /* allocate additional amount of nodes */
    if ( (node = (Nodes *)realloc( (Nodes *)node, (anz->nmax+anz->etype[4]*6+1) * sizeof(Nodes))) == NULL )
      printf("\n\n ERROR: realloc failed node\n\n") ;
  }
  if (anz->etype[5] > 0)
  {
    if ( (cpenta20 = (CPenta20 *)malloc( (anz->etype[5]+1) * sizeof(CPenta20))) == NULL )
      printf("\n\n ERROR: malloc failed cpenta20\n\n") ;
    /* allocate additional amount of nodes */
    if ( (node = (Nodes *)realloc( (Nodes *)node, (anz->nmax+anz->etype[5]*5+anz->etype[4]*6+1) * sizeof(Nodes))) == NULL )
      printf("\n\n ERROR: realloc failed node\n\n") ;
  }
  if (anz->etype[6] > 0)
  {
    if ( (ctet10 = (CTet10 *)malloc( (anz->etype[6]+1) * sizeof(CTet10))) == NULL )
      printf("\n\n ERROR: malloc failed ctet10\n\n") ;
  }
  if (anz->etype[7] > 0)
  {
    if ( (ctri3 = (CTri3 *)malloc( (anz->etype[7]+1) * sizeof(CTri3))) == NULL )
      printf("\n\n ERROR: malloc failed ctri3\n\n") ;
  }
  if (anz->etype[8] > 0)
  {
    if ( (ctri6 = (CTri6 *)malloc( (anz->etype[8]+1) * sizeof(CTri6))) == NULL )
      printf("\n\n ERROR: malloc failed ctri6\n\n") ;
  }
  if (anz->etype[9] > 0)
  {
    if ( (cquad4 = (CQuad4 *)malloc( (anz->etype[9]+1) * sizeof(CQuad4))) == NULL )
      printf("\n\n ERROR: malloc failed cquad4\n\n") ;
  }
  if (anz->etype[10] > 0)
  {
    if ( (cquad8 = (CHexa8 *)malloc( (anz->etype[10]+1) * sizeof(CHexa8))) == NULL )
      printf("\n\n ERROR: malloc failed cquad8\n\n") ;
  }
  if (anz->etype[11] > 0)
  {
    if ( (cbeam = (CBeam *)malloc( (anz->etype[11]+1) * sizeof(CBeam))) == NULL )
      printf("\n\n ERROR: malloc failed cbeam\n\n") ;
  }
  if (anz->etype[12] > 0)
  {
    if ( (cbeam3 = (CBeam3 *)malloc( (anz->etype[12]+1) * sizeof(CBeam3))) == NULL )
      printf("\n\n ERROR: malloc failed cbeam3\n\n") ;
  }
  elemGroup[0]=1;               /* anz Groups */
  elemGroup[1]=elem[0].group;   /* 1. Group */
  num_onod      = anz->nmax;
  num_olc       = anz->l;

  /* ------------- fill the element struktures -------------------------------- */

  /* e_enqire is used as e_enqire[elem[i].nr].xx   */
  if ( (e_enqire = (Elements *)realloc((Elements *)e_enqire, (anz->emax+1) * sizeof(Elements))) == NULL )
    printf(" ERROR: realloc in iniMeshData(), e_enqire\n\n") ;
  for (i=0; i<anz->e; i++)
  {
    if (elem[i].group != elemGroup[elemGroup[0]])
    {
      for (n=1; n<elemGroup[0]; n++) if (elem[i].group == elemGroup[n]) goto have_grp;
      elemGroup[0]++; elemGroup[elemGroup[0]]=elem[i].group;
      have_grp:;
    }
    if (elem[i].mat != elemMat[elemMat[0]])
    {
      for (n=1; n<elemMat[0]; n++) if (elem[i].mat == elemMat[n]) goto have_mat;
      elemMat[0]++; elemMat[elemMat[0]]=elem[i].mat;
      have_mat:;
    }
    if (elem[i].type == 1)
    {
      chexa8[num_etype[1]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      chexa8[num_etype[1]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      chexa8[num_etype[1]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[1];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<8; n++)
      {
        chexa8[num_etype[1]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[1]++;
    }
    else if (elem[i].type == 2)
    {
      cpenta6[num_etype[2]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      cpenta6[num_etype[2]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      cpenta6[num_etype[2]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[2];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<6; n++)
      {
        cpenta6[num_etype[2]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[2]++;
    }
    else if (elem[i].type == 3)
    {
      ctet4[num_etype[3]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      ctet4[num_etype[3]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      ctet4[num_etype[3]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[3];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<4; n++)
      {
        ctet4[num_etype[3]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[3]++;
    }
    else if (elem[i].type == 4)
    {
      for (n=0; n<3; n++)  
      {
        anz->nmax++;
        node[anz->n].nr = anz->nmax;
        node[anz->nmax].pflag=1;

        node[anz->nmax].nx = -1./4.* (
          node[elem[i].nod[0+n]].nx+node[elem[i].nod[1+n]].nx    +
          node[elem[i].nod[5+n]].nx+node[elem[i].nod[4+n]].nx )  + 1./2.*(
          node[elem[i].nod[8+n]].nx+node[elem[i].nod[13+n]].nx   +
          node[elem[i].nod[16+n]].nx+node[elem[i].nod[12+n]].nx) ;

        node[anz->nmax].ny = -1./4.* (
          node[elem[i].nod[0+n]].ny+node[elem[i].nod[1+n]].ny    +
          node[elem[i].nod[5+n]].ny+node[elem[i].nod[4+n]].ny )  + 1./2.*(
          node[elem[i].nod[8+n]].ny+node[elem[i].nod[13+n]].ny   +
          node[elem[i].nod[16+n]].ny+node[elem[i].nod[12+n]].ny) ;

        node[anz->nmax].nz = -1./4.* (
          node[elem[i].nod[0+n]].nz+node[elem[i].nod[1+n]].nz    +
          node[elem[i].nod[5+n]].nz+node[elem[i].nod[4+n]].nz )  + 1./2.*(
          node[elem[i].nod[8+n]].nz+node[elem[i].nod[13+n]].nz   +
          node[elem[i].nod[16+n]].nz+node[elem[i].nod[12+n]].nz) ;

        chexa20[num_etype[4]].nod[n+20]=node[anz->n].nr;
        anz->n++;
      }
        anz->nmax++;
        node[anz->n].nr = anz->nmax;
        node[anz->nmax].pflag=1;

        node[anz->nmax].nx = -1./4.* (
          node[elem[i].nod[3]].nx+node[elem[i].nod[0]].nx    +
          node[elem[i].nod[4]].nx+node[elem[i].nod[7]].nx )  + 1./2.*(
          node[elem[i].nod[11]].nx+node[elem[i].nod[12]].nx   +
          node[elem[i].nod[19]].nx+node[elem[i].nod[15]].nx) ;

        node[anz->nmax].ny = -1./4.* (
          node[elem[i].nod[3]].ny+node[elem[i].nod[0]].ny    +
          node[elem[i].nod[4]].ny+node[elem[i].nod[7]].ny )  + 1./2.*(
          node[elem[i].nod[11]].ny+node[elem[i].nod[12]].ny   +
          node[elem[i].nod[19]].ny+node[elem[i].nod[15]].ny) ;

        node[anz->nmax].nz = -1./4.* (
          node[elem[i].nod[3]].nz+node[elem[i].nod[0]].nz    +
          node[elem[i].nod[4]].nz+node[elem[i].nod[7]].nz )  + 1./2.*(
          node[elem[i].nod[11]].nz+node[elem[i].nod[12]].nz   +
          node[elem[i].nod[19]].nz+node[elem[i].nod[15]].nz) ;

        chexa20[num_etype[4]].nod[23]=node[anz->n].nr;
        anz->n++;

      for (n=0; n<2; n++) 
      {
        anz->nmax++;
        node[anz->n].nr = anz->nmax;
        node[anz->nmax].pflag=1;

        n1=n*4;
        n2=n*8;
        node[anz->nmax].nx = -1./4.* (
          node[elem[i].nod[0+n1]].nx+node[elem[i].nod[1+n1]].nx    +
          node[elem[i].nod[2+n1]].nx+node[elem[i].nod[3+n1]].nx )  + 1./2.*(
          node[elem[i].nod[8+n2]].nx+node[elem[i].nod[9+n2]].nx   +
          node[elem[i].nod[10+n2]].nx+node[elem[i].nod[11+n2]].nx) ;

        node[anz->nmax].ny = -1./4.* (
          node[elem[i].nod[0+n1]].ny+node[elem[i].nod[1+n1]].ny    +
          node[elem[i].nod[2+n1]].ny+node[elem[i].nod[3+n1]].ny )  + 1./2.*(
          node[elem[i].nod[8+n2]].ny+node[elem[i].nod[9+n2]].ny   +
          node[elem[i].nod[10+n2]].ny+node[elem[i].nod[11+n2]].ny) ;

        node[anz->nmax].nz = -1./4.* (
          node[elem[i].nod[0+n1]].nz+node[elem[i].nod[1+n1]].nz    +
          node[elem[i].nod[2+n1]].nz+node[elem[i].nod[3+n1]].nz )  + 1./2.*(
          node[elem[i].nod[8+n2]].nz+node[elem[i].nod[9+n2]].nz   +
          node[elem[i].nod[10+n2]].nz+node[elem[i].nod[11+n2]].nz) ;

        chexa20[num_etype[4]].nod[n+24]=node[anz->n].nr;
        anz->n++;
      }

      chexa20[num_etype[4]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      chexa20[num_etype[4]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      chexa20[num_etype[4]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[4];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<20; n++)
      {
        chexa20[num_etype[4]].nod[n]=elem[i].nod[n];
      }
      for (n=0; n<26; n++)
      {
        e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[4]++;
    }
    else if (elem[i].type == 5)
    {
      for (n=0; n<2; n++)  
      {
        anz->nmax++;
        node[anz->n].nr = anz->nmax;
        node[anz->nmax].pflag=1;

        node[anz->nmax].nx = -1./4.* (
          node[elem[i].nod[0+n]].nx+node[elem[i].nod[1+n]].nx    +
          node[elem[i].nod[4+n]].nx+node[elem[i].nod[3+n]].nx )  + 1./2.*(
          node[elem[i].nod[6+n]].nx+node[elem[i].nod[10+n]].nx   +
          node[elem[i].nod[12+n]].nx+node[elem[i].nod[ 9+n]].nx) ;

        node[anz->nmax].ny = -1./4.* (
          node[elem[i].nod[0+n]].ny+node[elem[i].nod[1+n]].ny    +
          node[elem[i].nod[4+n]].ny+node[elem[i].nod[3+n]].ny )  + 1./2.*(
          node[elem[i].nod[6+n]].ny+node[elem[i].nod[10+n]].ny   +
          node[elem[i].nod[12+n]].ny+node[elem[i].nod[ 9+n]].ny) ;

        node[anz->nmax].nz = -1./4.* (
          node[elem[i].nod[0+n]].nz+node[elem[i].nod[1+n]].nz    +
          node[elem[i].nod[4+n]].nz+node[elem[i].nod[3+n]].nz )  + 1./2.*(
          node[elem[i].nod[6+n]].nz+node[elem[i].nod[10+n]].nz   +
          node[elem[i].nod[12+n]].nz+node[elem[i].nod[ 9+n]].nz) ;

        cpenta20[num_etype[5]].nod[n+15]=node[anz->n].nr;
        anz->n++;
      }
        anz->nmax++;
        node[anz->n].nr = anz->nmax;
        node[anz->nmax].pflag=1;

        node[anz->nmax].nx = -1./4.* (
          node[elem[i].nod[2]].nx+node[elem[i].nod[0]].nx    +
          node[elem[i].nod[3]].nx+node[elem[i].nod[5]].nx )  + 1./2.*(
          node[elem[i].nod[ 8]].nx+node[elem[i].nod[ 9]].nx   +
          node[elem[i].nod[14]].nx+node[elem[i].nod[11]].nx) ;

        node[anz->nmax].ny = -1./4.* (
          node[elem[i].nod[2]].ny+node[elem[i].nod[0]].ny    +
          node[elem[i].nod[3]].ny+node[elem[i].nod[5]].ny )  + 1./2.*(
          node[elem[i].nod[ 8]].ny+node[elem[i].nod[ 9]].ny   +
          node[elem[i].nod[14]].ny+node[elem[i].nod[11]].ny) ;

        node[anz->nmax].nz = -1./4.* (
          node[elem[i].nod[2]].nz+node[elem[i].nod[0]].nz    +
          node[elem[i].nod[3]].nz+node[elem[i].nod[5]].nz )  + 1./2.*(
          node[elem[i].nod[ 8]].nz+node[elem[i].nod[ 9]].nz   +
          node[elem[i].nod[14]].nz+node[elem[i].nod[11]].nz) ;

        cpenta20[num_etype[5]].nod[17]=node[anz->n].nr;
        anz->n++;

        anz->nmax++;
        node[anz->n].nr = anz->nmax;
        node[anz->nmax].pflag=1;

        node[anz->nmax].nx = -1./4.* (
          node[elem[i].nod[0]].nx+node[elem[i].nod[2]].nx    +
          node[elem[i].nod[1]].nx+node[elem[i].nod[0]].nx )  + 1./2.*(
          node[elem[i].nod[ 8]].nx+node[elem[i].nod[ 7]].nx   +
          node[elem[i].nod[ 6]].nx+node[elem[i].nod[ 0]].nx) ;

        node[anz->nmax].ny = -1./4.* (
          node[elem[i].nod[0]].ny+node[elem[i].nod[2]].ny    +
          node[elem[i].nod[1]].ny+node[elem[i].nod[0]].ny )  + 1./2.*(
          node[elem[i].nod[ 8]].ny+node[elem[i].nod[ 7]].ny   +
          node[elem[i].nod[ 6]].ny+node[elem[i].nod[ 0]].ny) ;

        node[anz->nmax].nz = -1./4.* (
          node[elem[i].nod[0]].nz+node[elem[i].nod[2]].nz    +
          node[elem[i].nod[1]].nz+node[elem[i].nod[0]].nz )  + 1./2.*(
          node[elem[i].nod[ 8]].nz+node[elem[i].nod[ 7]].nz   +
          node[elem[i].nod[ 6]].nz+node[elem[i].nod[ 0]].nz) ;

        cpenta20[num_etype[5]].nod[18]=node[anz->n].nr;
        anz->n++;

        anz->nmax++;
        node[anz->n].nr = anz->nmax;
        node[anz->nmax].pflag=1;

        node[anz->nmax].nx = -1./4.* (
          node[elem[i].nod[3]].nx+node[elem[i].nod[4]].nx    +
          node[elem[i].nod[5]].nx+node[elem[i].nod[3]].nx )  + 1./2.*(
          node[elem[i].nod[12]].nx+node[elem[i].nod[13]].nx   +
          node[elem[i].nod[14]].nx+node[elem[i].nod[ 3]].nx) ;

        node[anz->nmax].ny = -1./4.* (
          node[elem[i].nod[3]].ny+node[elem[i].nod[4]].ny    +
          node[elem[i].nod[5]].ny+node[elem[i].nod[3]].ny )  + 1./2.*(
          node[elem[i].nod[12]].ny+node[elem[i].nod[13]].ny   +
          node[elem[i].nod[14]].ny+node[elem[i].nod[ 3]].ny) ;

        node[anz->nmax].nz = -1./4.* (
          node[elem[i].nod[3]].nz+node[elem[i].nod[4]].nz    +
          node[elem[i].nod[5]].nz+node[elem[i].nod[3]].nz )  + 1./2.*(
          node[elem[i].nod[12]].nz+node[elem[i].nod[13]].nz   +
          node[elem[i].nod[14]].nz+node[elem[i].nod[ 3]].nz) ;

        cpenta20[num_etype[5]].nod[19]=node[anz->n].nr;
        anz->n++;

      cpenta20[num_etype[5]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      cpenta20[num_etype[5]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      cpenta20[num_etype[5]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[5];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<15; n++)
      {
        cpenta20[num_etype[5]].nod[n]=elem[i].nod[n];
      }
      for (n=0; n<20; n++)
      {
        e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[5]++;
    }
    else if (elem[i].type == 6)
    {
      ctet10[num_etype[6]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      ctet10[num_etype[6]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      ctet10[num_etype[6]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[6];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<10; n++)
      {
        ctet10[num_etype[6]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[6]++;
    }
    else if (elem[i].type == 7)
    {
      ctri3[num_etype[7]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      ctri3[num_etype[7]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      ctri3[num_etype[7]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[7];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<3; n++)
      {
        ctri3[num_etype[7]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[7]++;
    }
    else if (elem[i].type == 8)
    {
      ctri6[num_etype[8]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      ctri6[num_etype[8]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      ctri6[num_etype[8]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[8];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<6; n++)
      {
        ctri6[num_etype[8]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[8]++;
    }
    else if (elem[i].type == 9)
    {
      cquad4[num_etype[9]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      cquad4[num_etype[9]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      cquad4[num_etype[9]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[9];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<4; n++)
      {
        cquad4[num_etype[9]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[9]++;
    }
    else if (elem[i].type == 10)
    {
      cquad8[num_etype[10]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      cquad8[num_etype[10]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      cquad8[num_etype[10]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[10];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<8; n++)
      {
        cquad8[num_etype[10]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[10]++;
    }
    else if (elem[i].type == 11)
    {
      cbeam[num_etype[11]].elem_nr = e_enqire[i].nr    =elem[i].nr;
      cbeam[num_etype[11]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      cbeam[num_etype[11]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[11];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<2; n++)
      {
        cbeam[num_etype[11]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[11]++;
    }
    else if (elem[i].type == 12)
    {
      cbeam3[num_etype[12]].elem_nr = e_enqire[i].nr    = elem[i].nr;
      cbeam3[num_etype[12]].group   = e_enqire[ elem[i].nr ].group = elem[i].group;
      cbeam3[num_etype[12]].mat     = e_enqire[ elem[i].nr ].mat   = elem[i].mat;
      e_enqire[ elem[i].nr ].elem_nr  = num_etype[12];
      e_enqire[ elem[i].nr ].type  = elem[i].type;
      for (n=0; n<3; n++)
      {
        cbeam3[num_etype[12]].nod[n]=e_enqire[ elem[i].nr ].nod[n]=elem[i].nod[n];
      }
      num_etype[12]++;
    }
  }

  makeSurfaces();

  /* if an ccx input was read then the faces created in makeSurfaces() must be assigned to the sets */
  /* the faces were stored in facebuf */
  /* go through all sets and see if faces are stored */
  if(facebuf!=NULL)
  {
    for(n=facebuf[0][0][0]; n>=0; n--)
    {
      setNr=facebuf[n][0][1];
      /* store the face indexes in the set */
      for(i=0; i<set[0].anz_f; i++)
      {
        n2=face[i].nr;
        n1=face[i].elem_nr;
        if(facebuf[n][n1][n2]==1) seta( setNr, "f", i);
      }
    
      /* free facebuf[n] */
      for(i=0; i<anz->emax+1; i++) free(facebuf[n][i]); free(facebuf[n]); 
    }
    free(facebuf);
  }

  allocUtilityData();
}

