#include <cgx.h>
#include <assert.h>
#include <float.h>

/**/
typedef struct {double x, y, z, w;} wpoint;

typedef struct 
{
	int    n;
        int    degree;
	wpoint  *polygon;
	double *knots;
}
nurbs;

wpoint *pointAt(const nurbs *curve, double u)
{
        static wpoint p;
	wpoint old, new;
	double alpha, knoti, knotikj;
	wpoint *olda, *newa;
	wpoint *oldd, *newd, *tmp;

	wpoint *polygon;
	double *knots;
	int n;

	int r, i, j, k, l;

	polygon = curve->polygon;
	knots = curve->knots;
	n = curve->n;
	k = curve->degree+1;

	if( (olda = (wpoint *) malloc((int)(k+1) * sizeof(wpoint)))==NULL)
        { printf("ERROR: malloc failed in pointAt()\n"); return(NULL); }
	if((newa  = (wpoint *) malloc((int)(k+1) * sizeof(wpoint)))==NULL)
        { printf("ERROR: malloc failed in pointAt()\n"); return(NULL); }


        if(knots[0]>0.)
        {
          for (i=n+k-1; i>=0; i--)
            knots[i] -= knots[0];
          for (i=0; i<n+k; i++)
            knots[i] /= knots[n+k-1];
          /*          assert(fabs(knots[0]) <= DBL_EPSILON);
          assert(fabs(knots[n+k-1]-1.) <= DBL_EPSILON);
          */
        }

	oldd = olda;
	newd = newa;
	/* find span r */
	for (i = curve->degree; i < curve->n; i++)
		if (u < knots[i]) break;
	r = i - 1;

	l = r-k+1;
	if (l<0)       oldd[0] = polygon[l+n];
	else if (l>=n) oldd[0] = polygon[l-n];
	else           oldd[0] = polygon[l];
	for (i = r-k+1+1; i<=r; i++)
	{
		if (i<0)       newd[0] = polygon[i+n];
		else if (i>=n) newd[0] = polygon[i-n];
		else           newd[0] = polygon[i];
		if (i<0)           knoti = knots[i+n-1] - 1.;
		else if (i>=n)	   knoti = 1. + knots[i-n+1];
		else               knoti = knots[i];
		for (j=1; j<i-r+k; j++)
		{
			l = i+k-j;
			if (l<0)           knotikj = knots[l+n-1] - 1.;
			else if (l>=n) knotikj = 1. + knots[l-n+1];
			else               knotikj = knots[l];
                        if((knotikj - knoti)==0.) return(NULL);
			else alpha = (u-knoti) / (knotikj - knoti);
			/* newd[j] = (1-alpha)*oldd[j-1] + alpha*newd[j-1]; */
			old = oldd[j-1];
			new = newd[j-1];
			p.x = (1-alpha)*old.x + alpha*new.x;
			p.y = (1-alpha)*old.y + alpha*new.y;
			p.z = (1-alpha)*old.z + alpha*new.z;
			p.w = (1-alpha)*old.w + alpha*new.w;
			newd[j] = p;
		}
		/* swap newd oldd */
		tmp = newd;
		newd = oldd;
		oldd = tmp;
	}
	p = oldd[k-1];
        free(olda);
        free(newa);
        return(&p);
}


int nurbl2seq(int nurlNr, Nurbl *nurbl, Points *point)
{
  int   setNr=-1;
  double u;
  wpoint *p, *cp;
  int i, pnr;
  nurbs nurbsi;
  unsigned char name[MAX_LINE_LENGTH];

  /* printf("nurlname:%s p:%d k:%d e:%d\n",nurbl[nurlNr].name,nurbl[nurlNr].u_npnt,nurbl[nurlNr].u_nknt,nurbl[nurlNr].u_exp); */

  nurbsi.n = nurbl[nurlNr].u_npnt;
  nurbsi.degree = nurbl[nurlNr].u_exp;
  nurbsi.polygon=NULL;
  nurbsi.knots=NULL;

  getNewName( name, "se" );
  if( (setNr=pre_seta( name, "is", 0)) <0 ) return(-1);

  if( (cp = (wpoint *)malloc(nurbl[nurlNr].u_npnt*sizeof(wpoint) ) )==NULL) 
  { printf(" ERROR: malloc failure\n"); goto mfail; }
  if( (nurbsi.knots = (double *)malloc(nurbl[nurlNr].u_nknt*sizeof(double) ) )==NULL) 
  { printf(" ERROR: malloc failure\n"); goto mfail; }

  for(i=0; i<nurbl[nurlNr].u_npnt; i++)
  {
    cp[i].x=nurbl[nurlNr].weight[i]*point[nurbl[nurlNr].ctlpnt[i]].px; 
    cp[i].y=nurbl[nurlNr].weight[i]*point[nurbl[nurlNr].ctlpnt[i]].py; 
    cp[i].z=nurbl[nurlNr].weight[i]*point[nurbl[nurlNr].ctlpnt[i]].pz; 
    cp[i].w=nurbl[nurlNr].weight[i];
  } 
  nurbsi.polygon=cp;
  for(i=0; i<nurbl[nurlNr].u_nknt; i++)
  {
    nurbsi.knots[i]=nurbl[nurlNr].uknt[i]/nurbl[nurlNr].uknt[nurbl[nurlNr].u_nknt-1];
  } 

  seta( setNr, "p", nurbl[nurlNr].ctlpnt[0]  );
  for (u=1./nurbl[nurlNr].u_npnt; u<0.999; u+=1./nurbl[nurlNr].u_npnt)
  {
    p = pointAt(&nurbsi, u);
    if(p)
    {
      getNewName( name, "p" );
      pnr= pnt( name, p->x/p->w, p->y/p->w, p->z/p->w, 0 );
      seta( setNr, "p", pnr );
    }
    else
    {
      printf(" WARNING: nurbl:%s is evaluated with errors at u:%f\n", nurbl[nurlNr].name, u); 
      exit(-1);
    }
  }
  seta( setNr, "p", nurbl[nurlNr].ctlpnt[nurbl[nurlNr].u_npnt-1]  );

 mfail:;
  free(nurbsi.polygon);
  free(nurbsi.knots);
  return(setNr);
}
