/* --------------------------------------------------------------------  */
/*                          CALCULIX                                     */
/*                   - GRAPHICAL INTERFACE -                             */
/*                                                                       */
/*     A 3-dimensional pre- and post-processor for finite elements       */
/*              Copyright (C) 1996 Klaus Wittig                          */
/*                                                                       */
/*     This program is free software; you can redistribute it and/or     */
/*     modify it under the terms of the GNU General Public License as    */
/*     published by the Free Software Foundation; version 2 of           */
/*     the License.                                                      */
/*                                                                       */
/*     This program is distributed in the hope that it will be useful,   */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of    */ 
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      */
/*     GNU General Public License for more details.                      */
/*                                                                       */
/*     You should have received a copy of the GNU General Public License */
/*     along with this program; if not, write to the Free Software       */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         */
/* --------------------------------------------------------------------  */

#include <cgx.h>


/* --------------------------------------------------------------------  */
/* Userfunctions                                                         */
/* interface to the mesh and datasets                                    */
/*                                                                       */
/* string:  parameter list                                               */
/*                                                                       */
/* Structures: (See also the sections ELEMENTS and RESULT FORMAT in the  */
/*              manual and the file extUtil.h                            */
/*                                                                       */
/* typedef struct {                                                      */
/*   unsigned char  model[MAX_LINE_LENGTH];    haeder model              */
/*   int   n;                                  number of nodes           */
/*   int   e;                                  number of elements        */
/*   int   l;                             number of loadcases (Datasets) */
/*   int   etype[100];             number of elements of a certain type  */
/*   int   nmax;                               maximum nodenumber        */
/*   int   nmin;                               minimum nodenumber        */
/*   int   emax;                               maximum elemnumber        */
/*   int   emin;                               minimum elemnumber        */
/* } Summen;                                                             */
/*                                                                       */
/* typedef struct {                                                      */
/*   int   nr;                                                           */
/*   unsigned char  pflag;       1 if used for display purposes          */
/*                              -1 if the node is deleted                */
/*                               0 default                               */
/*   float nx;                   coordinates                             */
/*   float ny;                                                           */
/*   float nz;                                                           */
/* } Nodes;                                                              */
/*                                                                       */
/* typedef struct {                                                      */
/*   int nr;                external element number                      */
/*   int elem_nr;           element-index of a certain type (CBeam etc.) */
/*   int type;              element type (1:CHexa8)                      */
/*   int group;                                                          */
/*   int mat;                                                            */
/*   int attr;                 -1:unstructured mesh                      */
/*                              0:default                                */
/*                              1:reduced integration                    */
/*                              2:incompatible modes                     */
/*   int nod[MAX_NODES_PER_ELEMENT+1];                                   */
/* } Elements;                                                           */
/*                                                                       */
/* typedef struct {                                                      */
/*   unsigned char  **compName;                                          */
/*   unsigned char  **icname;                                            */
/*   unsigned char  name[MAX_LINE_LENGTH];                               */
/*   float value;                                                        */
/*   int ncomps;                                                         */
/*   int irtype;                                                         */
/*   int *ictype;                                                        */
/*   int *icind1;                                                        */
/*   int *icind2;                                                        */
/*   int *iexist;                                                        */
/*   float **dat;           node related data                            */
/*   float ***edat;         element related data                         */
/*   float *max;            maximum datum                                */
/*   float *min;            minimum datum                                */
/*   int *nmax;             node with maximum datum                      */
/*   int *nmin;             node with minimum datum                      */
/* } Datasets;                                                           */ 
/*                                                                       */
/*                                                                       */
/* --------------------------------------------------------------------  */
                                                                 
                                                                
void userFunction(unsigned char *string, Summen *sum, Nodes **nptr, Elements **eptr, Datasets **lptr)
{                                                               
  int i,l,n;                                                    
  Nodes *node;                                                  
  Elements *elem;                                               
  Datasets *lcase;                                              
                                                                
  node=*nptr;                                                   
  elem=*eptr;                                                   
  lcase=*lptr;                                                  
                                                                
                                                                
  if(compareStrings(string, "hydro")>0)                         
  {                                                             
    /* calculate the hydrostatic stress and the deviator */     
    for(l=0; l<sum->l; l++)                                     
    {                                                           
      /* use only stresses */
      if (compare(lcase[l].name,"STRESS",6)==6)
      {
        printf(" calc hydrostatic stress and deviator for LC[%d]: %s \n", l, lcase[l].name); 
  
        /* create a new dataset */
        if ( (lcase = (Datasets *)realloc((Datasets *)lcase, (sum->l+1) * sizeof(Datasets))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );

        lcase[sum->l].ncomps = 4;
        lcase[sum->l].irtype = 1;
        lcase[sum->l].value  = lcase[l].value ;
  
        if ( (lcase[sum->l].nmax = (int *)malloc( lcase[sum->l].ncomps * sizeof(int))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        if ( (lcase[sum->l].nmin = (int *)malloc( lcase[sum->l].ncomps * sizeof(int))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        if ( (lcase[sum->l].max = (float *)malloc( lcase[sum->l].ncomps * sizeof(float))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        if ( (lcase[sum->l].min = (float *)malloc( lcase[sum->l].ncomps * sizeof(float))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        if ( (lcase[sum->l].dat = (float **)malloc( lcase[sum->l].ncomps * sizeof(float *))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        if ( (lcase[sum->l].compName = (unsigned char **)malloc( lcase[sum->l].ncomps * sizeof(unsigned char *))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        if ( (lcase[sum->l].icname = (unsigned char **)malloc( lcase[sum->l].ncomps * sizeof(unsigned char *))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        for(i=0; i<lcase[sum->l].ncomps; i++)
        {
          if ( (lcase[sum->l].dat[i] = (float *)malloc( (sum->nmax+1) * sizeof(float))) == NULL )
            printf("\n\n ERROR: malloc failure\n\n" );	               
          if ( (lcase[sum->l].compName[i] = (unsigned char *)malloc( MAX_LINE_LENGTH * sizeof(unsigned char))) == NULL )
             printf("\n\n ERROR: malloc failed\n\n" );
          if ( (lcase[sum->l].icname[i] = (unsigned char *)malloc( MAX_LINE_LENGTH * sizeof(unsigned char))) == NULL )
             printf("\n\n ERROR: malloc failed\n\n" );
          lcase[sum->l].max[i]=-MAXVALUE;
          lcase[sum->l].min[i]=MAXVALUE;
        }
        if ( (lcase[sum->l].ictype = (int *)malloc( lcase[sum->l].ncomps * sizeof(int))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        if ( (lcase[sum->l].icind1 = (int *)malloc( lcase[sum->l].ncomps * sizeof(int))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        if ( (lcase[sum->l].icind2 = (int *)malloc( lcase[sum->l].ncomps * sizeof(int))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
        if ( (lcase[sum->l].iexist = (int *)malloc( lcase[sum->l].ncomps * sizeof(int))) == NULL )
          printf("\n\n ERROR: malloc failure\n\n" );
  
        for(i=0; i<lcase[sum->l].ncomps; i++)
        {
          lcase[sum->l].ictype[i] = 1;
          lcase[sum->l].icind1[i] = i+1;
          lcase[sum->l].icind2[i] = 0;
          lcase[sum->l].iexist[i] = 0;
        }
        sprintf( lcase[sum->l].name, "HYD&DEV");
        strcpy( lcase[sum->l].compName[0], "SM");
        strcpy( lcase[sum->l].compName[1], "SXX-SM");
        strcpy( lcase[sum->l].compName[2], "SYY-SM");
        strcpy( lcase[sum->l].compName[3], "SZZ-SM");
  
        for(n=0; n<sum->n; n++)
        {
          /* calc average Princ Stress sigm */
          lcase[sum->l].dat[0][node[n].nr]=
            (lcase[l].dat[8][node[n].nr]+lcase[l].dat[9][node[n].nr]+lcase[l].dat[10][node[n].nr])/3.;
         
          /* calc the deviator */
          for(i=0; i<3; i++)
            lcase[sum->l].dat[i+1][node[n].nr]=lcase[l].dat[i][node[n].nr]-lcase[sum->l].dat[0][node[n].nr];

          for(i=0; i<lcase[sum->l].ncomps; i++)
          {
            if (lcase[sum->l].dat[i][node[n].nr] >  lcase[sum->l].max[i])
            {  lcase[sum->l].max[i]=lcase[sum->l].dat[i][node[n].nr]; lcase[sum->l].nmax[i]=node[n].nr; }
            if (lcase[sum->l].dat[i][node[n].nr] <  lcase[sum->l].min[i])
            {  lcase[sum->l].min[i]=lcase[sum->l].dat[i][node[n].nr]; lcase[sum->l].nmin[i]=node[n].nr; }
          }
        }
        sum->l++;
      }
    }
  }
  else printf(" ERROR userfunction:%s not known\n", string);

  *nptr=node;
  *eptr=elem;
  *lptr=lcase;
}
