// libSNL - Simple Nurbs Library
// Copyright Scott A.E. Lanham, Australia.
// ---------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

// *** Small Utility Classes and Functions ***

#include "snlUtil.h"

// Static member declarations
// --------------------------

binCoefs    bCoefs;  // Force constructor to build array.
int         binCoefs::binCoefArray [ MAX_BINOMIAL ] [ MAX_BINOMIAL ];

binCoefs::binCoefs()
{
    for ( int k = 0; k < MAX_BINOMIAL; k ++ )

        for ( int i = 0; i < MAX_BINOMIAL; i ++ )
        {
            if ( k == i )
                binCoefArray [ k ] [ i ] = 1;
            else if ( i == 0 )
                binCoefArray [ k ] [ i ] = 1;
            else if ( i > k )
                binCoefArray [ k ] [ i ] = 0;
            else
                binCoefArray [ k ] [ i ] = binCoefArray [ k - 1 ] [ i ] + binCoefArray [ k - 1 ] [ i - 1 ];
        }
}

double distToLine ( snlPoint lineStart, snlPoint lineEnd, snlPoint compare )
{
    // Calculate distance to line from point.
    // --------------------------------------
    // lineStart:    Start of line.
    // lineEnd:      End of line.
    // compare:      Point to compare to line.
    
    lineStart.normalise();
    lineEnd.normalise();
    compare.normalise();
    
    snlVector lineV ( lineStart, lineEnd );    
    
    snlVector testV ( lineStart, compare );

    // Calculate dot product.
    double dotP = lineV.dot ( testV );

    // Project test vector onto baseline vector.
    double proj = dotP / lineV.length();

    // Length of test vector.
    double testLength = testV.length();

    // Length of normal from baseline to test point.
    double normDist = sqrt ( testLength * testLength - proj * proj );

    return normDist;
}

snlVector projectToLine ( snlPoint lineStart, snlPoint lineEnd, snlPoint compare )
{
    // Calculate vector perpendicular to line from point.
    // --------------------------------------------------
    // lineStart:    Start of line.
    // lineEnd:      End of line.
    // compare:      Point to project to line.

    lineStart.normalise();
    lineEnd.normalise();
    compare.normalise();
    
    snlVector lineV ( lineStart, lineEnd );    
    
    snlVector testV ( lineStart, compare );

    // Calculate dot product.
    double dotP = lineV.dot ( testV );

    // Calculated length of test vector after it is projected onto baseline vector.
    double proj = dotP / lineV.length();

    // Generate projected vector.
    
    lineV.length ( proj );
    lineV -= testV;

    return lineV;
}

